@@ -1,24 +1,24 @@
 # Command-line interface for jdaviz
 
+import inspect
 import os
 import pathlib
-import sys
-import tempfile
-
-from voila.app import Voila
-from voila.configuration import VoilaConfiguration
 
 from jdaviz import __version__
-from jdaviz.app import _verbosity_levels
+from jdaviz.app import _verbosity_levels, ALL_JDAVIZ_CONFIGS
+from jdaviz import configs
 
 __all__ = ['main']
 
+CONFIGS_DIR = str(pathlib.Path(inspect.getfile(configs)).parent)
 JDAVIZ_DIR = pathlib.Path(__file__).parent.resolve()
+DEFAULT_VERBOSITY = 'warning'
+DEFAULT_HISTORY_VERBOSITY = 'info'
 
 
 def main(filepaths=None, layout='default', instrument=None, browser='default',
-         theme='light', verbosity='warning', history_verbosity='info',
-         hotreload=False):
+         theme='auto', verbosity=DEFAULT_VERBOSITY, history_verbosity=DEFAULT_HISTORY_VERBOSITY,
+         host='localhost', port=0, hotreload=False):
     """
     Start a Jdaviz application instance with data loaded from FILENAME.
 
@@ -32,22 +32,19 @@ def main(filepaths=None, layout='default', instrument=None, browser='default',
         Specifies which instrument parser to use for Mosviz, if applicable.
     browser : str, optional
         Path to browser executable.
-    theme : {'light', 'dark'}
-        Theme to use for Voila app or Jupyter Lab.
+    theme : {'auto', 'light', 'dark'}
+        Theme to use for application.
     verbosity : {'debug', 'info', 'warning', 'error'}
         Verbosity of the popup messages in the application.
     history_verbosity : {'debug', 'info', 'warning', 'error'}
         Verbosity of the history logger in the application.
+    host : str, optional
+        Host to bind the server to, default is 'localhost'.
+    port : int, optional
+        Port to bind the server to, default is 0 (finds an empty port).
     hotreload : bool
         Whether to enable hot-reloading of the UI (for development)
     """
-    import logging  # Local import to avoid possibly messing with JWST pipeline logger.
-
-    # Tornado Webserver py3.8 compatibility hotfix for windows
-    if sys.platform == 'win32':
-        import asyncio
-        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
-
     if filepaths:
         # Convert paths to posix string; windows paths are not JSON compliant
         file_list = [pathlib.Path(f).absolute().as_posix() for f in filepaths]
@@ -58,47 +55,32 @@ def main(filepaths=None, layout='default', instrument=None, browser='default',
     else:
         file_list = []
 
-    with open(JDAVIZ_DIR / "jdaviz_cli.ipynb") as f:
-        notebook_template = f.read()
-
-    start_dir = os.path.abspath('.')
+    if layout == '' and len(file_list) > 1:
+        raise ValueError("'layout' argument is required when specifying multiple files")
 
     # Keep track of start directory in environment variable so that it can be
     # easily accessed e.g. in the file load dialog.
-    os.environ['JDAVIZ_START_DIR'] = start_dir
-
-    nbdir = tempfile.mkdtemp()
+    os.environ['JDAVIZ_START_DIR'] = os.path.abspath('.')
 
+    from solara.__main__ import cli
+    from jdaviz import solara
+    solara.config = layout.capitalize()
+    solara.data_list = file_list
+    if layout == 'mosviz':
+        solara.load_data_kwargs = {'instrument': instrument}
+    solara.theme = theme
+    solara.jdaviz_verbosity = verbosity
+    solara.jdaviz_history_verbosity = history_verbosity
+    args = []
     if hotreload:
-        notebook_template = notebook_template.replace("# PREFIX", "from jdaviz import enable_hot_reloading; enable_hot_reloading()")  # noqa: E501
-
-    with open(os.path.join(nbdir, 'notebook.ipynb'), 'w') as nbf:
-        nbf.write(
-            notebook_template
-            .replace('CONFIG', layout.capitalize())
-            .replace('DATA_LIST', str(file_list))
-            .replace('JDAVIZ_VERBOSITY', verbosity)
-            .replace('JDAVIZ_HISTORY_VERBOSITY', history_verbosity)
-            # Mosviz specific changes
-            .replace('load_data(data', 'load_data(directory=data' if layout == 'mosviz' else 'load_data(data')  # noqa: E501
-            .replace(') #ADDITIONAL_LOAD_DATA_ARGS', f', instrument=\'{instrument}\')' if layout == 'mosviz' else ')')  # noqa: E501
-            .strip()
-        )
-
-    os.chdir(nbdir)
-
-    try:
-        logging.getLogger('tornado.access').disabled = True
-        Voila.notebook_path = 'notebook.ipynb'
-        VoilaConfiguration.template = 'jdaviz-default'
-        VoilaConfiguration.enable_nbextensions = True
-        VoilaConfiguration.file_whitelist = ['.*']
-        VoilaConfiguration.theme = theme
-        if browser != 'default':
-            Voila.browser = browser
-        sys.exit(Voila().launch_instance(argv=[]))
-    finally:
-        os.chdir(start_dir)
+        args += ['--auto-restart']
+    else:
+        args += ['--production']
+    cli(['run', 'jdaviz.solara',
+         '--theme-loader', 'plain',
+         '--theme-variant', theme,
+         '--host', host,
+         '--port', port] + args)
 
 
 def _main(config=None):
@@ -109,8 +91,7 @@ def _main(config=None):
                                      'loaded from FILENAME.')
     filepaths_nargs = '*'
     if config is None:
-        parser.add_argument('layout', choices=['cubeviz', 'specviz', 'specviz2d',
-                                               'mosviz', 'imviz'],
+        parser.add_argument('--layout', default='', choices=ALL_JDAVIZ_CONFIGS,
                             help='Configuration to use.')
     if (config == "mosviz") or ("mosviz" in sys.argv):
         filepaths_nargs = 1
@@ -126,13 +107,13 @@ def _main(config=None):
                         help='Verbosity of the application for popup snackbars.')
     parser.add_argument('--history-verbosity', choices=_verbosity_levels, default='info',
                         help='Verbosity of the logger history.')
-    if sys.version_info >= (3, 9):
-        # Also enables --no-hotreload
-        parser.add_argument('--hotreload', action=argparse.BooleanOptionalAction, default=False,
-                            help='Whether to enable hot-reloading of the UI (for development).')
-    else:
-        parser.add_argument('--hotreload', action='store_true', default=False,
-                            help='Enable hot-reloading of the UI (for development).')
+    parser.add_argument('--host', type=str, default='localhost',
+                        help='Host to bind the server to, defaults to localhost.')
+    parser.add_argument('--port', type=int, default=0,
+                        help='Port to bind the server to, defaults to 0 (finds an empty port).')
+    # Also enables --no-hotreload
+    parser.add_argument('--hotreload', action=argparse.BooleanOptionalAction, default=False,
+                        help='Whether to enable hot-reloading of the UI (for development).')
     parser.add_argument('--version', action='version', version=f'%(prog)s {__version__}')
     args = parser.parse_args()
 
@@ -143,4 +124,28 @@ def _main(config=None):
 
     main(filepaths=args.filepaths, layout=layout, instrument=args.instrument, browser=args.browser,
          theme=args.theme, verbosity=args.verbosity, history_verbosity=args.history_verbosity,
-         hotreload=args.hotreload)
+         host=args.host, port=args.port, hotreload=args.hotreload)
+
+
+def _specviz():
+    _main(config='specviz')
+
+
+def _specviz2d():
+    _main(config='specviz2d')
+
+
+def _imviz():
+    _main(config='imviz')
+
+
+def _cubeviz():
+    _main(config='cubeviz')
+
+
+def _mosviz():
+    _main(config='mosviz')
+
+
+if __name__ == "__main__":
+    _main()
