@@ -1,5 +1,7 @@
-from .tools import *  # noqa
+from .mixins import *  # noqa
 from .viewers import *  # noqa
 from .parsers import *  # noqa
 from .moment_maps.moment_maps import *  # noqa
 from .slice.slice import * # noqa
+from .spectral_extraction.spectral_extraction import * # noqa
+from .tools import *  # noqa
