@@ -10,7 +10,8 @@ from jdaviz.core.custom_traitlets import FloatHandleEmpty
 from jdaviz.core.events import SnackbarMessage
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin, DatasetSelectMixin,
-                                        SelectPluginComponent, AddResultsMixin)
+                                        SelectPluginComponent, AddResultsMixin,
+                                        with_spinner)
 from jdaviz.core.user_api import PluginUserApi
 
 __all__ = ['GaussianSmooth']
@@ -31,6 +32,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
     * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
       Dataset to use for computing line statistics.
     * ``mode`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
@@ -42,7 +44,6 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
     """
     template_file = __file__, "gaussian_smooth.vue"
     stddev = FloatHandleEmpty(1).tag(sync=True)
-    selected_data_is_1d = Bool(True).tag(sync=True)
     show_modes = Bool(False).tag(sync=True)
     mode_items = List().tag(sync=True)
     mode_selected = Unicode().tag(sync=True)
@@ -50,34 +51,41 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        self._default_spectrum_viewer_reference_name = kwargs.get(
-            "spectrum_viewer_reference_name", "spectrum-viewer"
-        )
-        self._default_flux_viewer_reference_name = kwargs.get(
-            "flux_viewer_reference_name", "flux-viewer"
-        )
-
         if self.config == "cubeviz":
+            self.docs_description = 'Smooth data cube spatially or spectrally with a Gaussian kernel.'  # noqa
             self.show_modes = True
-            # retrieve the data from the cube, not the collapsed 1d spectrum
             self.dataset._viewers = [
                 self._default_flux_viewer_reference_name,
                 self._default_spectrum_viewer_reference_name
             ]
             # clear the cache in case the spectrum-viewer selection was already cached
             self.dataset._clear_cache()
-        elif self.config == "mosviz":
+        elif self.config in ("mosviz", "specviz2d"):
             # only allow smoothing 1d spectra
             self.dataset._viewers = [self._default_spectrum_viewer_reference_name]
             self.dataset._clear_cache()
 
+        self.dataset.add_filter('not_from_this_plugin')
+
         self.mode = SelectPluginComponent(self,
                                           items='mode_items',
                                           selected='mode_selected',
                                           manual_options=['Spectral', 'Spatial'])
 
-        # set the filter on the viewer options
-        self._update_viewer_filters()
+        # set the filter on the dataset and viewer options
+        self._update_dataset_viewer_filters()
+
+    @property
+    def _default_spectrum_viewer_reference_name(self):
+        return getattr(
+            self.app._jdaviz_helper, '_default_spectrum_viewer_reference_name', 'spectrum-viewer'
+        )
+
+    @property
+    def _default_flux_viewer_reference_name(self):
+        return getattr(
+            self.app._jdaviz_helper, '_default_flux_viewer_reference_name', 'flux-viewer'
+        )
 
     @property
     def user_api(self):
@@ -105,23 +113,30 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
             self.app.return_data_label(f"{dataset}{smooth_type} {stddev}", check_unique=False))
 
     @observe("dataset_selected")
-    def _on_data_selected(self, event={}):
+    def _update_viewer_filters(self, event={}):
         if not hasattr(self, 'dataset'):
             # during initial init, this can trigger before the component is initialized
             return
 
-        # NOTE: if this is ever used anywhere else, it should be moved into DatasetSelect
         if self.dataset.selected_dc_item is not None:
-            self.selected_data_is_1d = len(self.dataset.selected_dc_item.data.shape) == 1
+            selected_data_is_1d = len(self.dataset.selected_dc_item.data.shape) == 1
+
+            if selected_data_is_1d:
+                # only want spectral viewers in the options
+                self.add_results.viewer.filters = ['is_spectrum_viewer']
+            else:
+                # only want image viewers in the options
+                self.add_results.viewer.filters = ['is_image_viewer']
 
     @observe("mode_selected")
-    def _update_viewer_filters(self, event={}):
+    def _update_dataset_viewer_filters(self, event={}):
         if event.get('new', self.mode_selected) == 'Spatial':
-            # only want image viewers in the options
-            self.add_results.viewer.filters = ['is_image_viewer']
+            # MUST be a cube
+            self.dataset.add_filter('layer_in_flux_viewer')
         else:
-            # only want spectral viewers in the options
-            self.add_results.viewer.filters = ['is_spectrum_viewer']
+            # can either be a cube OR a spectrum
+            self.dataset.remove_filter('layer_in_flux_viewer')
+        self._update_viewer_filters()
 
     def vue_apply(self, event={}):
         self.smooth(add_data=True)
@@ -169,6 +184,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         return results
 
+    @with_spinner()
     def spectral_smooth(self):
         """
         Smooth the input spectrum along the spectral axis.  To add the resulting spectrum into
@@ -192,6 +208,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         return spec_smoothed
 
+    @with_spinner('spinner')
     def spatial_smooth(self):
         """
         Use astropy convolution machinery to smooth the spatial dimensions of
@@ -203,12 +220,7 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
         cube : `~specutils.Spectrum1D`
             The smoothed cube
         """
-        # Get information from the flux component
-        attribute = self.dataset.selected_dc_item.main_components[0]
-
-        cube = self.dataset.get_object(cls=Spectrum1D,
-                                       attribute=attribute,
-                                       statistic=None)
+        cube = self.dataset.selected_obj
         flux_unit = cube.flux.unit
 
         # Extend the 2D kernel to have a length 1 spectral dimension, so that
@@ -217,8 +229,16 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
 
         convolved_data = convolve(cube, kernel)
 
+        # Copy 3D WCS from input cube.
+        data = self.dataset.selected_dc_item
+        # Similar to coords_info logic.
+        if '_orig_spec' in getattr(data, 'meta', {}):
+            w = data.meta['_orig_spec'].wcs
+        else:
+            w = data.coords
+
         # Create a new cube with the old metadata. Note that astropy
         # convolution generates values for masked (NaN) data.
-        newcube = Spectrum1D(flux=convolved_data * flux_unit, wcs=cube.wcs)
+        newcube = Spectrum1D(flux=convolved_data * flux_unit, wcs=w)
 
         return newcube
