@@ -1,4 +1,4 @@
-from traitlets import Bool, List, observe
+from traitlets import Any, Bool, List, observe
 
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import PluginTemplateMixin, DatasetSelectMixin
@@ -18,6 +18,7 @@ class MetadataViewer(PluginTemplateMixin, DatasetSelectMixin):
 
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
     * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
       Dataset to expose the metadata.
     * :attr:`show_primary`:
@@ -34,6 +35,7 @@ class MetadataViewer(PluginTemplateMixin, DatasetSelectMixin):
     show_primary = Bool(False).tag(sync=True)
     has_comments = Bool(False).tag(sync=True)
     metadata = List([]).tag(sync=True)
+    metadata_filter = Any().tag(sync=True)  # string or None
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -53,6 +55,10 @@ class MetadataViewer(PluginTemplateMixin, DatasetSelectMixin):
 
     @observe("dataset_selected")
     def show_metadata(self, event):
+        if not hasattr(self, 'dataset'):  # pragma: no cover
+            # plugin not fully initialized
+            return
+
         data = self.dataset.selected_dc_item
         if (data is None or not hasattr(data, 'meta') or not isinstance(data.meta, dict)
                 or len(data.meta) < 1):
