@@ -6,24 +6,26 @@ import astropy.units as u
 from specutils import Spectrum1D
 from specutils.utils import QuantityModel
 from traitlets import Bool, List, Unicode, observe
-from glue.core.data import Data
 
-from jdaviz.core.events import SnackbarMessage
+from jdaviz.core.events import SnackbarMessage, GlobalDisplayUnitChanged
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin,
                                         SelectPluginComponent,
                                         SpectralSubsetSelectMixin,
-                                        SubsetSelect,
                                         DatasetSelectMixin,
                                         DatasetSpectralSubsetValidMixin,
+                                        NonFiniteUncertaintyMismatchMixin,
                                         AutoTextField,
-                                        AddResultsMixin)
+                                        AddResultsMixin,
+                                        TableMixin,
+                                        with_spinner)
 from jdaviz.core.custom_traitlets import IntHandleEmpty
 from jdaviz.core.user_api import PluginUserApi
 from jdaviz.configs.default.plugins.model_fitting.fitting_backend import fit_model_to_spectrum
 from jdaviz.configs.default.plugins.model_fitting.initializers import (MODELS,
                                                                        initialize,
                                                                        get_model_parameters)
+from jdaviz.utils import _eqv_flux_to_sb_pixel
 
 __all__ = ['ModelFitting']
 
@@ -39,7 +41,8 @@ class _EmptyParam:
 @tray_registry('g-model-fitting', label="Model Fitting", viewer_requirements='spectrum')
 class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                    SpectralSubsetSelectMixin, DatasetSpectralSubsetValidMixin,
-                   AddResultsMixin):
+                   NonFiniteUncertaintyMismatchMixin,
+                   AddResultsMixin, TableMixin):
     """
     See the :ref:`Model Fitting Plugin Documentation <specviz-model-fitting>` for more details.
 
@@ -48,10 +51,12 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
+    * ``cube_fit``
+      Only exposed for Cubeviz.  Whether to fit the model to the cube instead of to the
+      collapsed spectrum.
     * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
       Dataset to fit the model.
-    * ``spatial_subset`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`):
-      Only exposed for Cubeviz.  Spatially collapsed spectrum to use to fit the model.
     * ``spectral_subset`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`)
     * ``model_component`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`)
     * ``poly_order``
@@ -59,13 +64,12 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     * :meth:`create_model_component`
     * :meth:`remove_model_component`
     * :meth:`model_components`
+    * :meth:`valid_model_components`
     * :meth:`get_model_component`
     * :meth:`set_model_component`
     * :meth:`reestimate_model_parameters`
     * ``equation`` (:class:`~jdaviz.core.template_mixin.AutoTextField`)
-    * ``cube_fit``
-      Only exposed for Cubeviz.  Whether to fit the model to the cube instead of to the
-      collapsed spectrum.
+    * :meth:`equation_components`
     * ``add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
     * ``residuals_calculate`` (bool)
       Whether to calculate and expose the residuals (model minus data).
@@ -78,9 +82,6 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     template_file = __file__, "model_fitting.vue"
     form_valid_model_component = Bool(False).tag(sync=True)
 
-    spatial_subset_items = List().tag(sync=True)
-    spatial_subset_selected = Unicode().tag(sync=True)
-
     # model components:
     model_comp_items = List().tag(sync=True)
     model_comp_selected = Unicode().tag(sync=True)
@@ -112,26 +113,12 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        self._default_spectrum_viewer_reference_name = kwargs.get(
-            "spectrum_viewer_reference_name", "spectrum-viewer"
-        )
-        self._default_flux_viewer_reference_name = kwargs.get(
-            "flux_viewer_reference_name", "flux-viewer"
-        )
         self._units = {}
         self._fitted_model = None
         self._fitted_spectrum = None
         self.component_models = []
         self._initialized_models = {}
         self._display_order = False
-        if self.config == "cubeviz":
-            self.spatial_subset = SubsetSelect(self,
-                                               'spatial_subset_items',
-                                               'spatial_subset_selected',
-                                               default_text='Entire Cube',
-                                               allowed_type='spatial')
-        else:
-            self.spatial_subset = None
 
         # create the label first so that when model_component defaults to the first selection,
         # the label automatically defaults as well
@@ -143,9 +130,11 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                                                      selected='model_comp_selected',
                                                      manual_options=list(MODELS.keys()))
 
-        # when accessing the selected data, access the spectrum-viewer version
-        self.dataset._viewers = [self._default_spectrum_viewer_reference_name]
-        # require entries to be in spectrum-viewer (not other cubeviz images, etc)
+        if self.config == 'cubeviz':
+            # use mean whenever extracting the 1D spectrum of a cube to initialize model params
+            self.dataset._spectral_extraction_function = 'Mean'
+        # by default, require entries to be in spectrum-viewer (not other cubeviz images, etc)
+        # in cubeviz, the cube_fit toggle will then replace this filter to filter for cubes
         self.dataset.add_filter('layer_in_spectrum_viewer')
 
         self.equation = AutoTextField(self, 'model_equation', 'model_equation_default',
@@ -154,21 +143,46 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self.residuals = AutoTextField(self, 'residuals_label', 'residuals_label_default',
                                        'residuals_label_auto', 'residuals_label_invalid_msg')
 
+        headers = ['model', 'data_label', 'spectral_subset', 'equation']
+        if self.config == 'cubeviz':
+            headers += ['cube_fit']
+
+        self.table.headers_avail = headers
+        self.table.headers_visible = headers
+        # when model parameters are added as columns, only show the value/unit columns by default
+        # (other columns can be show in the dropdown by the user)
+        self.table._new_col_visible = lambda colname: colname.split(':')[-1] not in ('fixed', 'uncert', 'std')  # noqa
+
         # set the filter on the viewer options
         self._update_viewer_filters()
 
+        self.hub.subscribe(self, GlobalDisplayUnitChanged,
+                           handler=self._on_global_display_unit_changed)
+
+    @property
+    def _default_spectrum_viewer_reference_name(self):
+        return getattr(
+            self.app._jdaviz_helper, '_default_spectrum_viewer_reference_name', 'spectrum-viewer'
+        )
+
+    @property
+    def _default_flux_viewer_reference_name(self):
+        return getattr(
+            self.app._jdaviz_helper, '_default_flux_viewer_reference_name', 'flux-viewer'
+        )
+
     @property
     def user_api(self):
         expose = ['dataset']
-        if self.config == "cubeviz":
-            expose += ['spatial_subset']
-        expose += ['spectral_subset', 'model_component', 'poly_order', 'model_component_label',
-                   'model_components', 'create_model_component', 'remove_model_component',
-                   'get_model_component', 'set_model_component', 'reestimate_model_parameters',
-                   'equation', 'add_results', 'residuals_calculate', 'residuals']
         if self.config == "cubeviz":
             expose += ['cube_fit']
-        expose += ['calculate_fit']
+        expose += ['spectral_subset', 'model_component', 'poly_order', 'model_component_label',
+                   'model_components', 'valid_model_components',
+                   'create_model_component', 'remove_model_component',
+                   'get_model_component', 'set_model_component', 'reestimate_model_parameters',
+                   'equation', 'equation_components',
+                   'add_results', 'residuals_calculate', 'residuals']
+        expose += ['calculate_fit', 'clear_table', 'export_table']
         return PluginUserApi(self, expose=expose)
 
     def _param_units(self, param, model_type=None):
@@ -223,10 +237,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
             m["parameters"] = temp_params
 
-        # Trick traitlets into updating the displayed values
-        component_models = self.component_models
-        self.component_models = []
-        self.component_models = component_models
+        self.send_state('component_models')
 
     def _update_parameters_from_QM(self):
         """
@@ -265,10 +276,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                 temp_params += temp_param
             m["parameters"] = temp_params
 
-        # Trick traitlets into updating the displayed values
-        component_models = self.component_models
-        self.component_models = []
-        self.component_models = component_models
+        self.send_state('component_models')
 
     def _update_initialized_parameters(self):
         # If the user changes a parameter value, we need to change it in the
@@ -292,11 +300,37 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         else:
             return False
 
-    def _get_1d_spectrum(self):
-        # retrieves the 1d spectrum (accounting for spatial subset for cubeviz, if necessary)
-        return self.dataset.selected_spectrum_for_spatial_subset(self.spatial_subset_selected) # noqa
+    def _update_viewer_filters(self, event={}):
+        if event.get('new', self.cube_fit):
+            # only want image viewers in the options
+            self.add_results.viewer.filters = ['is_image_viewer']
+        else:
+            # only want spectral viewers in the options
+            self.add_results.viewer.filters = ['is_spectrum_viewer']
 
-    @observe("dataset_selected", "spatial_subset_selected")
+    @observe('cube_fit')
+    def _cube_fit_changed(self, event={}):
+        self._update_viewer_filters(event=event)
+
+        sb_unit = self.app._get_display_unit('sb')
+        spectral_y_unit = self.app._get_display_unit('spectral_y')
+        if event.get('new'):
+            self._units['y'] = sb_unit
+            self.dataset.add_filter('is_flux_cube')
+            self.dataset.remove_filter('layer_in_spectrum_viewer')
+        else:
+            self._units['y'] = spectral_y_unit
+            self.dataset.add_filter('layer_in_spectrum_viewer')
+            self.dataset.remove_filter('is_flux_cube')
+
+        self.dataset._clear_cache()
+        if sb_unit != spectral_y_unit:
+            # We make the user hit the reestimate button themselves
+            for model_index, comp_model in enumerate(self.component_models):
+                self.component_models[model_index]["compat_display_units"] = False
+            self.send_state('component_models')
+
+    @observe("dataset_selected")
     def _dataset_selected_changed(self, event=None):
         """
         Callback method for when the user has selected data from the drop down
@@ -315,7 +349,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             # during initial init, this can trigger before the component is initialized
             return
 
-        selected_spec = self._get_1d_spectrum()
+        selected_spec = self.dataset.selected_obj
         if selected_spec is None:
             return
 
@@ -323,11 +357,6 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # (won't affect calculations because these locations are masked)
         selected_spec.flux[np.isnan(selected_spec.flux)] = 0.0
 
-        self._units["x"] = str(
-            selected_spec.spectral_axis.unit)
-        self._units["y"] = str(
-            selected_spec.flux.unit)
-
     def _default_comp_label(self, model, poly_order=None):
         abbrevs = {'BlackBody': 'BB', 'PowerLaw': 'PL', 'Lorentz1D': 'Lo'}
         abbrev = abbrevs.get(model, model[0].upper())
@@ -441,6 +470,16 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                      "parameters": [], "model_kwargs": {}}
         model_cls = MODELS[model_comp]
 
+        # Need to set the units the first time we initialize a model component, after this
+        # we listen for display unit changes
+        if self._units.get('x', '') == '':
+            self._units['x'] = self.app._get_display_unit('spectral')
+        if self._units.get('y', '') == '':
+            if self.cube_fit:
+                self._units['y'] = self.app._get_display_unit('sb')
+            else:
+                self._units['y'] = self.app._get_display_unit('spectral_y')
+
         if model_comp == "Polynomial1D":
             # self.poly_order is the value in the widget for creating
             # the new model component.  We need to store that with the
@@ -469,29 +508,101 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
             initial_values[param_name] = initial_val
 
-        masked_spectrum = self._apply_subset_masks(self._get_1d_spectrum(), self.spectral_subset)
+        if self.cube_fit:
+            # We need to input the whole cube when initializing the model so the units are correct.
+            if self.dataset_selected in self.app.data_collection.labels:
+                data = self.app.data_collection[self.dataset_selected].get_object(statistic=None)
+            else:  # User selected some subset from spectrum viewer, just use original cube
+                data = self.app.data_collection[0].get_object(statistic=None)
+            masked_spectrum = self._apply_subset_masks(data, self.spectral_subset)
+        else:
+            masked_spectrum = self._apply_subset_masks(self.dataset.selected_spectrum,
+                                                       self.spectral_subset)
         mask = masked_spectrum.mask
+        if mask is not None:
+            if mask.ndim == 3:
+                spectral_mask = mask.all(axis=(0, 1))
+            else:
+                spectral_mask = mask
+            init_x = masked_spectrum.spectral_axis[~spectral_mask]
+            orig_flux_shape = masked_spectrum.flux.shape
+            init_y = masked_spectrum.flux[~mask]
+            if mask.ndim == 3:
+                init_y = init_y.reshape(orig_flux_shape[0],
+                                        orig_flux_shape[1],
+                                        len(init_x))
+        else:
+            init_x = masked_spectrum.spectral_axis
+            init_y = masked_spectrum.flux
+
+        # equivs for spectral density and flux<>flux/pix2. revisit
+        # when generalizing plugin UC equivs.
+        equivs = _eqv_flux_to_sb_pixel() + u.spectral_density(init_x)
+        init_y = init_y.to(self._units['y'], equivs)
+
         initialized_model = initialize(
             MODELS[model_comp](name=comp_label,
                                **initial_values,
                                **new_model.get("model_kwargs", {})),
-            masked_spectrum.spectral_axis[~mask] if mask is not None else masked_spectrum.spectral_axis,  # noqa
-            masked_spectrum.flux[~mask] if mask is not None else masked_spectrum.flux)
+            init_x, init_y)
 
         # need to loop over parameters again as the initializer may have overridden
-        # the original default value
+        # the original default value. However, if we toggled cube_fit, we may need to override
         for param_name in get_model_parameters(model_cls, new_model["model_kwargs"]):
             param_quant = getattr(initialized_model, param_name)
             new_model["parameters"].append({"name": param_name,
                                             "value": param_quant.value,
                                             "unit": str(param_quant.unit),
                                             "fixed": False})
-
         self._initialized_models[comp_label] = initialized_model
 
         new_model["Initialized"] = True
+        new_model["initialized_display_units"] = self._units.copy()
+
+        new_model["compat_display_units"] = True  # always compatible at time of creation
         return new_model
 
+    def _check_model_component_compat(self, axes=['x', 'y'], display_units=None):
+        if display_units is None:
+            display_units = [u.Unit(self._units[ax]) for ax in axes]
+
+        disp_physical_types = [unit.physical_type for unit in display_units]
+
+        for model_index, comp_model in enumerate(self.component_models):
+            compat = True
+            for ax, ax_physical_type in zip(axes, disp_physical_types):
+                comp_unit = u.Unit(comp_model["initialized_display_units"][ax])
+                compat = comp_unit.physical_type == ax_physical_type
+                if not compat:
+                    break
+            self.component_models[model_index]["compat_display_units"] = compat
+
+        # length hasn't changed, so we need to force the traitlet to update
+        self.send_state("component_models")
+        self._check_model_equation_invalid()
+
+    def _on_global_display_unit_changed(self, msg):
+        if msg.axis == 'spectral_y':
+            axis = 'y'
+        elif msg.axis == 'spectral':
+            axis = 'x'
+        else:
+            return
+
+        if axis == 'y' and self.cube_fit:
+            # The units have to be in surface brightness for a cube fit.
+            uc = self.app._jdaviz_helper.plugins['Unit Conversion']
+
+            if msg.unit != uc._obj.sb_unit_selected:
+                self._units[axis] = uc._obj.sb_unit_selected
+                self._check_model_component_compat([axis], [u.Unit(uc._obj.sb_unit_selected)])
+                return
+
+        # update internal tracking of current units
+        self._units[axis] = str(msg.unit)
+
+        self._check_model_component_compat([axis], [msg.unit])
+
     def remove_model_component(self, model_component_label):
         """
         Remove an existing model component.
@@ -531,6 +642,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         comp = {"model_type": model_component['model_type'],
                 "parameters": {p['name']: {'value': p['value'],
                                            'unit': p['unit'],
+                                           'std': p.get('std', np.nan),
                                            'fixed': p['fixed']} for p in model_component['parameters']}}  # noqa
 
         if parameter is not None:
@@ -620,6 +732,9 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # length hasn't changed, so we need to force the traitlet to update
         self.send_state("component_models")
 
+        # model units may have changed, need to re-check their compatibility with display units
+        self._check_model_component_compat()
+
         # return user-friendly info on revised model
         return self.get_model_component(model_component_label)
 
@@ -630,6 +745,20 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         """
         return [x["id"] for x in self.component_models]
 
+    @property
+    def valid_model_components(self):
+        """
+        List of the labels of existing valid (due to display units) model components
+        """
+        return [x["id"] for x in self.component_models if x["compat_display_units"]]
+
+    @property
+    def equation_components(self):
+        """
+        List of the labels of model components in the current equation
+        """
+        return re.split(r'[+*/-]', self.equation.value.replace(' ', ''))
+
     def vue_add_model(self, event):
         self.create_model_component()
 
@@ -637,10 +766,41 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self.remove_model_component(event)
 
     @observe('model_equation')
-    def _model_equation_changed(self, event):
+    def _check_model_equation_invalid(self, event=None):
         # Length is a dummy check to test the infrastructure
         if len(self.model_equation) == 0:
-            self.model_equation_invalid_msg = 'model equation is required'
+            self.model_equation_invalid_msg = 'model equation is required.'
+            return
+        if '' in self.equation_components:
+            # includes an operator without a variable (ex: 'C+')
+            self.model_equation_invalid_msg = 'incomplete equation.'
+            return
+
+        components_not_existing = [comp for comp in self.equation_components
+                                   if comp not in self.model_components]
+        if len(components_not_existing):
+            if len(components_not_existing) == 1:
+                msg = "is not an existing model component."
+            else:
+                msg = "are not existing model components."
+            self.model_equation_invalid_msg = f'{", ".join(components_not_existing)} {msg}'
+            return
+        components_not_valid = [comp for comp in self.equation_components
+                                if comp not in self.valid_model_components]
+        if len(components_not_valid):
+            if len(components_not_valid) == 1:
+                msg = ("is currently disabled because it has"
+                       " incompatible units with the current display units."
+                       " Remove the component from the equation,"
+                       " re-estimate its free parameters to use the new units"
+                       " or revert the display units.")
+            else:
+                msg = ("are currently disabled because they have"
+                       " incompatible units with the current display units."
+                       " Remove the components from the equation,"
+                       " re-estimate their free parameters to use the new units"
+                       " or revert the display units.")
+            self.model_equation_invalid_msg = f'{", ".join(components_not_valid)} {msg}'
             return
         self.model_equation_invalid_msg = ''
 
@@ -649,8 +809,6 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         label_comps = []
         if hasattr(self, 'dataset') and (len(self.dataset.labels) > 1 or self.app.config == 'mosviz'):  # noqa
             label_comps += [self.dataset_selected]
-        if self.cube_fit:
-            label_comps += ["cube-fit"]
         label_comps += ["model"]
         self.results_label_default = " ".join(label_comps)
 
@@ -658,15 +816,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     def _set_residuals_label_default(self, event={}):
         self.residuals_label_default = self.results_label+" residuals"
 
-    @observe("cube_fit")
-    def _update_viewer_filters(self, event={}):
-        if event.get('new', self.cube_fit):
-            # only want image viewers in the options
-            self.add_results.viewer.filters = ['is_image_viewer']
-        else:
-            # only want spectral viewers in the options
-            self.add_results.viewer.filters = ['is_spectrum_viewer']
-
+    @with_spinner()
     def calculate_fit(self, add_data=True):
         """
         Calculate the fit.
@@ -683,14 +833,44 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         fitted spectrum/cube
         residuals (if ``residuals_calculate`` is set to ``True``)
         """
+
         if not self.spectral_subset_valid:
             valid, spec_range, subset_range = self._check_dataset_spectral_subset_valid(return_ranges=True)  # noqa
             raise ValueError(f"spectral subset '{self.spectral_subset.selected}' {subset_range} is outside data range of '{self.dataset.selected}' {spec_range}")  # noqa
+        if len(self.model_equation_invalid_msg):
+            raise ValueError(f"model equation is invalid: {self.model_equation_invalid_msg}")
 
         if self.cube_fit:
-            return self._fit_model_to_cube(add_data=add_data)
+            ret = self._fit_model_to_cube(add_data=add_data)
         else:
-            return self._fit_model_to_spectrum(add_data=add_data)
+            ret = self._fit_model_to_spectrum(add_data=add_data)
+
+        if ret is None:  # pragma: no cover
+            # something went wrong in the fitting call and (hopefully) already raised a warning,
+            # but we don't have anything to add to the table
+            return ret
+
+        if self.cube_fit:
+            # cube fits are currently unsupported in tables
+            return ret
+
+        row = {'model': self.results_label if add_data else '',
+               'data_label': self.dataset_selected,
+               'spectral_subset': self.spectral_subset_selected,
+               'equation': self.equation.value}
+
+        equation_components = self.equation_components
+        for comp_ind, comp in enumerate(equation_components):
+            for param_name, param_dict in self.get_model_component(comp).get('parameters', {}).items():  # noqa
+                colprefix = f"{comp}:{param_name}_{comp_ind}"
+                row[colprefix] = param_dict.get('value')
+                row[f"{colprefix}:unit"] = param_dict.get('unit')
+                row[f"{colprefix}:fixed"] = param_dict.get('fixed')
+                row[f"{colprefix}:std"] = param_dict.get('std')
+
+        self.table.add_item(row)
+
+        return ret
 
     def vue_apply(self, event):
         self.calculate_fit()
@@ -705,8 +885,8 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             return
         models_to_fit = self._reinitialize_with_fixed()
 
-        masked_spectrum = self._apply_subset_masks(self._get_1d_spectrum(), self.spectral_subset)
-
+        masked_spectrum = self._apply_subset_masks(self.dataset.selected_spectrum,
+                                                   self.spectral_subset)
         try:
             fitted_model, fitted_spectrum = fit_model_to_spectrum(
                 masked_spectrum,
@@ -721,6 +901,10 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             self.hub.broadcast(msg)
             return
 
+        selected_spec = self.dataset.selected_obj
+        if '_pixel_scale_factor' in selected_spec.meta:
+            fitted_spectrum.meta['_pixel_scale_factor'] = selected_spec.meta['_pixel_scale_factor']
+
         self._fitted_model = fitted_model
         self._fitted_spectrum = fitted_spectrum
 
@@ -777,6 +961,10 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         else:
             spec = data.get_object(cls=Spectrum1D, statistic=None)
 
+        sb_unit = self.app._get_display_unit('sb')
+        if spec.flux.unit != sb_unit:
+            spec = spec.with_flux_unit(sb_unit)
+
         snackbar_message = SnackbarMessage(
             "Fitting model to cube...",
             loading=True, sender=self)
@@ -785,9 +973,14 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # Retrieve copy of the models with proper "fixed" dictionaries
         models_to_fit = self._reinitialize_with_fixed()
 
-        # Apply masks from selected subsets
-        for subset in [self.spatial_subset, self.spectral_subset]:
-            spec = self._apply_subset_masks(spec, subset)
+        # Apply masks from selected spectral subset
+        spec = self._apply_subset_masks(spec, self.spectral_subset)
+        # Also mask out NaNs for fitting. Simply adding filter_non_finite to the cube fit
+        # didn't work out of the box, so doing this for now.
+        if spec.mask is None:
+            spec.mask = np.isnan(spec.flux)
+        else:
+            spec.mask = spec.mask | np.isnan(spec.flux)
 
         try:
             fitted_model, fitted_spectrum = fit_model_to_spectrum(
@@ -811,17 +1004,13 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                 temp_label = "{} ({}, {})".format(self.results_label, m["x"], m["y"])
                 self.app.fitted_models[temp_label] = m["model"]
 
-        count = max(map(lambda s: int(next(iter(re.findall(r"\d$", s)), 0)),
-                        self.data_collection.labels)) + 1
+        output_cube = Spectrum1D(flux=fitted_spectrum.flux, wcs=fitted_spectrum.wcs)
 
-        label = self.app.return_data_label(f"{self.results_label}[Cube]", ext=count)
-
-        # Create new glue data object
-        output_cube = Data(label=label,
-                           coords=fitted_spectrum.wcs,
-                           flux=fitted_spectrum.flux.value)
-        output_cube.get_component('flux').units = fitted_spectrum.flux.unit.to_string()
+        selected_spec = self.dataset.selected_obj
+        if '_pixel_scale_factor' in selected_spec.meta:
+            output_cube.meta['_pixel_scale_factor'] = selected_spec.meta['_pixel_scale_factor']
 
+        # Create new data entry for glue
         if add_data:
             self.add_results.add_results_from_plugin(output_cube)
             self._set_default_results_label()
@@ -836,8 +1025,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     def _apply_subset_masks(self, spectrum, subset_component):
         """
         For a spectrum/spectral cube ``spectrum``, add a mask attribute
-        if none exists. Mask excludes non-selected spectral and/or
-        spatial subsets.
+        if none exists. Mask excludes non-selected spectral subsets.
         """
         # only look for a mask if there is a selected subset:
         if subset_component.selected == subset_component.default_text:
