@@ -3,8 +3,9 @@ from .viewers import *  # noqa
 from .image_viewer_creator import *  # noqa
 from .parsers import *  # noqa
 from .coords_info import *  # noqa
-from .links_control import *  # noqa
+from .orientation import *  # noqa
 from .compass import *  # noqa
 from .aper_phot_simple import *  # noqa
 from .line_profile_xy import *  # noqa
-from .catalogs import *  # noqa
\ No newline at end of file
+from .catalogs import *  # noqa
+from .footprints import *  # noqa
