@@ -1,5 +1,3 @@
-import os
-
 import numpy as np
 import numpy.ma as ma
 from astropy import units as u
@@ -7,17 +5,23 @@ from astropy.table import QTable
 from astropy.coordinates import SkyCoord
 from traitlets import List, Unicode, Bool, Int, observe
 
-from jdaviz.configs.default.plugins.data_tools.file_chooser import FileChooser
 from jdaviz.core.events import SnackbarMessage
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin, ViewerSelectMixin,
-                                        SelectPluginComponent)
+                                        FileImportSelectPluginComponent, HasFileImportSelect,
+                                        with_spinner)
+from jdaviz.core.custom_traitlets import IntHandleEmpty
+
+from jdaviz.core.marks import CatalogMark
+
+from jdaviz.core.template_mixin import TableMixin
+from jdaviz.core.user_api import PluginUserApi
 
 __all__ = ['Catalogs']
 
 
 @tray_registry('imviz-catalogs', label="Catalog Search")
-class Catalogs(PluginTemplateMixin, ViewerSelectMixin):
+class Catalogs(PluginTemplateMixin, ViewerSelectMixin, HasFileImportSelect, TableMixin):
     """
     See the :ref:`Catalog Search Plugin Documentation <imviz-catalogs>` for more details.
 
@@ -26,74 +30,81 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin):
 
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
     """
     template_file = __file__, "catalogs.vue"
+    uses_active_status = Bool(True).tag(sync=True)
     catalog_items = List([]).tag(sync=True)
     catalog_selected = Unicode("").tag(sync=True)
-    from_file = Unicode().tag(sync=True)
-    from_file_message = Unicode().tag(sync=True)
     results_available = Bool(False).tag(sync=True)
     number_of_results = Int(0).tag(sync=True)
+    max_gaia_sources = IntHandleEmpty(1000).tag(sync=True)
+
+    # setting the default table headers and values
+    _default_table_values = {
+            'Right Ascension (degrees)': np.nan,
+            'Declination (degrees)': np.nan,
+            'Object ID': np.nan,
+            'id': np.nan,
+            'x_coord': np.nan,
+            'y_coord': np.nan}
+
+    @property
+    def user_api(self):
+        return PluginUserApi(self, expose=('clear_table', 'export_table',))
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        cat_options = ['SDSS', 'Gaia']
+        if not self.app.state.settings.get('server_is_remote', False):
+            cat_options.append('From File...')
+        self.catalog = FileImportSelectPluginComponent(self,
+                                                       items='catalog_items',
+                                                       selected='catalog_selected',
+                                                       manual_options=cat_options)
 
-        self.catalog = SelectPluginComponent(self,
-                                             items='catalog_items',
-                                             selected='catalog_selected',
-                                             manual_options=['SDSS', 'From File...'])
-
-        # file chooser for From File
-        start_path = os.environ.get('JDAVIZ_START_DIR', os.path.curdir)
-        self._file_upload = FileChooser(start_path)
-        self.components = {'g-file-import': self._file_upload}
-        self._file_upload.observe(self._on_file_path_changed, names='file_path')
-        self._cached_table_from_file = {}
+        # set the custom file parser for importing catalogs
+        self.catalog._file_parser = self._file_parser
         self._marker_name = 'catalog_results'
 
-    def _on_file_path_changed(self, event):
-        self.from_file_message = 'Checking if file is valid'
-        path = event['new']
-        if (path is not None
-                and not os.path.exists(path)
-                or not os.path.isfile(path)):
-            self.from_file_message = 'File path does not exist'
-            return
+        # initializing the headers in the table that is displayed in the UI
+        headers = ['Right Ascension (degrees)', 'Declination (degrees)',
+                   'Object ID', 'x_coord', 'y_coord']
 
+        self.table.headers_avail = headers
+        self.table.headers_visible = headers
+        self.table._default_values_by_colname = self._default_table_values
+        self.table._selected_rows_changed_callback = lambda msg: self.plot_selected_points()
+        self.table.item_key = 'id'
+        self.table.show_rowselect = True
+
+    @staticmethod
+    def _file_parser(path):
         try:
             table = QTable.read(path)
         except Exception:
-            self.from_file_message = 'Could not parse file with astropy.table.QTable.read'
-            return
+            return 'Could not parse file with astropy.table.QTable.read', {}
 
         if 'sky_centroid' not in table.colnames:
-            self.from_file_message = 'Table does not contain required sky_centroid column'
-            return
+            return 'Table does not contain required sky_centroid column', {}
 
-        # since we loaded the file already to check if its valid, we might as well cache the table
-        # so we don't have to re-load it when clicking search.  We'll only keep the latest entry
-        # though, but store in a dict so we can catch if the file path was changed from the API
-        self._cached_table_from_file = {path: table}
-        self.from_file_message = ''
+        return '', {path: table}
 
-    @observe('from_file')
-    def _from_file_changed(self, event):
-        if len(event['new']):
-            if not os.path.exists(event['new']):
-                raise ValueError(f"{event['new']} does not exist")
-            self.catalog.selected = 'From File...'
-        else:
-            # NOTE: select_default will change the value even if the current value is valid
-            # (so will change from 'From File...' to the first entry in the dropdown)
-            self.catalog.select_default()
+    @with_spinner()
+    def search(self, error_on_fail=False):
+        """Search the catalog, display markers on the viewer, and return results if available.
 
-    def vue_set_file_from_dialog(self, *args, **kwargs):
-        self.from_file = self._file_upload.file_path
+        Parameters
+        ----------
+        error_on_fail : bool
+            Throw exception when query fails instead of a red snackbar message.
+            This is useful for debugging.
+
+        Returns
+        -------
+        skycoord_table : `~astropy.coordinates.SkyCoord` or `None`
+            Sky coordinates (or None if no results available).
 
-    def search(self):
-        """
-        Search the catalog, display markers on the viewer, and return a table of results (or None
-        if no results available)
         """
         # calling clear in the case the user forgot after searching
         self.clear()
@@ -143,9 +154,11 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin):
                 query_region_result = SDSS.query_region(skycoord_center, radius=zoom_radius,
                                                         data_release=17)
             except Exception as e:  # nosec
-                self.hub.broadcast(SnackbarMessage(
-                    f"Failed to query {self.catalog_selected} with c={skycoord_center} and "
-                    f"r={zoom_radius}: {repr(e)}", color='error', sender=self))
+                errmsg = (f"Failed to query {self.catalog_selected} with c={skycoord_center} and "
+                          f"r={zoom_radius}: {repr(e)}")
+                if error_on_fail:
+                    raise Exception(errmsg) from e
+                self.hub.broadcast(SnackbarMessage(errmsg, color='error', sender=self))
                 query_region_result = None
 
             if query_region_result is None:
@@ -162,10 +175,19 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin):
                                       query_region_result['dec'],
                                       unit='deg')
 
+        elif self.catalog_selected == 'Gaia':
+            from astroquery.gaia import Gaia, conf
+
+            with conf.set_temp("ROW_LIMIT", self.max_gaia_sources):
+                sources = Gaia.query_object(skycoord_center, radius=zoom_radius,
+                                            columns=('source_id', 'ra', 'dec'))
+            self.app._catalog_source_table = sources
+            skycoord_table = SkyCoord(sources['ra'], sources['dec'], unit='deg')
+
         elif self.catalog_selected == 'From File...':
             # all exceptions when going through the UI should have prevented setting this path
             # but this exceptions might be raised here if setting from_file from the UI
-            table = self._cached_table_from_file.get(self.from_file, QTable.read(self.from_file))
+            table = self.catalog.selected_obj
             self.app._catalog_source_table = table
             skycoord_table = table['sky_centroid']
 
@@ -195,19 +217,114 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin):
         filtered_pair_pixel_table = np.array(np.hsplit(filtered_table, 2))
         x_coordinates = np.squeeze(filtered_pair_pixel_table[0])
         y_coordinates = np.squeeze(filtered_pair_pixel_table[1])
+
+        if self.catalog_selected in ["SDSS", "Gaia"]:
+            for row, x_coord, y_coord in zip(self.app._catalog_source_table,
+                                             x_coordinates, y_coordinates):
+                if self.catalog_selected == "SDSS":
+                    row_id = row["objid"]
+                elif self.catalog_selected == "Gaia":
+                    row_id = row["SOURCE_ID"]
+                # Check if the row contains the required keys
+                row_info = {'Right Ascension (degrees)': row['ra'],
+                            'Declination (degrees)': row['dec'],
+                            'Object ID': row_id.astype(str),
+                            'id': len(self.table),
+                            'x_coord': x_coord,
+                            'y_coord': y_coord}
+                self.table.add_item(row_info)
+
+        # NOTE: If performance becomes a problem, see
+        # https://docs.astropy.org/en/stable/table/index.html#performance-tips
+        if self.catalog_selected == 'From File...':
+            for row, x_coord, y_coord in zip(self.app._catalog_source_table,
+                                             x_coordinates, y_coordinates):
+                # Check if the row contains the required keys
+                row_info = {'Right Ascension (degrees)': row['sky_centroid'].ra.deg,
+                            'Declination (degrees)': row['sky_centroid'].dec.deg,
+                            'Object ID': str(row.get('label', 'N/A')),
+                            'id': len(self.table),
+                            'x_coord': x_coord,
+                            'y_coord': y_coord}
+
+                self.table.add_item(row_info)
+
         filtered_skycoord_table = viewer.state.reference_data.coords.pixel_to_world(x_coordinates,
                                                                                     y_coordinates)
 
         # QTable stores all the filtered sky coordinate points to be marked
         catalog_results = QTable({'coord': filtered_skycoord_table})
+
         self.number_of_results = len(catalog_results)
-
         # markers are added to the viewer based on the table
-        viewer.marker = {'color': 'red', 'alpha': 0.8, 'markersize': 5, 'fill': False}
+        viewer.marker = {'color': 'blue', 'alpha': 0.8, 'markersize': 30, 'fill': False}
         viewer.add_markers(table=catalog_results, use_skycoord=True, marker_name=self._marker_name)
-
         return skycoord_table
 
+    def _get_mark(self, viewer):
+        matches = [mark for mark in viewer.figure.marks if isinstance(mark, CatalogMark)]
+        if len(matches):
+            return matches[0]
+        mark = CatalogMark(viewer)
+        viewer.figure.marks = viewer.figure.marks + [mark]
+        return mark
+
+    @property
+    def marks(self):
+        return {viewer_id: self._get_mark(viewer)
+                for viewer_id, viewer in self.app._viewer_store.items()
+                if hasattr(viewer, 'figure')}
+
+    @observe('is_active')
+    def _on_is_active_changed(self, *args):
+        if self.disabled_msg:
+            return
+
+        for mark in self.marks.values():
+            mark.visible = self.is_active
+
+    def plot_selected_points(self):
+        selected_rows = self.table.selected_rows
+
+        x = [float(coord['x_coord']) for coord in selected_rows]
+        y = [float(coord['y_coord']) for coord in selected_rows]
+        self._get_mark(self.viewer.selected_obj).update_xy(getattr(x, 'value', x),
+                                                           getattr(y, 'value', y))
+
+    def vue_zoom_in(self, *args, **kwargs):
+        """This function will zoom into the image based on the selected points"""
+        selected_rows = self.table.selected_rows
+
+        x = [float(coord['x_coord']) for coord in selected_rows]
+        y = [float(coord['y_coord']) for coord in selected_rows]
+
+        # this works with single selected points
+        # zooming when the range is too large is not performing correctly
+        x_min = min(x) - 50
+        x_max = max(x) + 50
+        y_min = min(y) - 50
+        y_max = max(y) + 50
+
+        self.app._jdaviz_helper._default_viewer.set_limits(
+            x_min=x_min, x_max=x_max, y_min=y_min, y_max=y_max)
+
+        return (x_min, x_max), (y_min, y_max)
+
+    def import_catalog(self, catalog):
+        """
+        Import a catalog from a file path.
+
+        Parameters
+        ----------
+        catalog : str
+          Path to a file that can be parsed by astropy QTable
+        """
+        # TODO: self.catalog.import_obj for a QTable directly (see footprints implementation)
+        if isinstance(catalog, str):
+            self.catalog.import_file(catalog)
+        else:  # pragma: no cover
+            raise ValueError("catalog must be a string (file path)")
+
     def vue_do_search(self, *args, **kwargs):
         # calls self.search() which handles all of the searching logic
         self.search()
@@ -225,7 +342,7 @@ class Catalogs(PluginTemplateMixin, ViewerSelectMixin):
             viewer.remove_markers(marker_name=self._marker_name)
 
         elif self.results_available:
-            from jdaviz.configs.imviz.helper import layer_is_table_data
+            from jdaviz.utils import layer_is_table_data
 
             # resetting values
             self.results_available = False
