@@ -3,35 +3,45 @@ import numpy as np
 from traitlets import Bool, Unicode, observe
 
 from astropy import units as u
+from bqplot import LinearScale
 from glue.core import BaseData
-from glue.core.subset import RoiSubsetState
-from glue.core.subset_group import GroupedSubset
 from glue_jupyter.bqplot.image.layer_artist import BqplotImageSubsetLayerArtist
 
 from jdaviz.configs.cubeviz.plugins.viewers import CubevizImageView
 from jdaviz.configs.imviz.plugins.viewers import ImvizImageView
-from jdaviz.configs.mosviz.plugins.viewers import MosvizImageView, MosvizProfile2DView
+from jdaviz.configs.mosviz.plugins.viewers import (MosvizImageView, MosvizProfileView,
+                                                   MosvizProfile2DView)
+from jdaviz.configs.rampviz.plugins.viewers import RampvizImageView, RampvizProfileView
 from jdaviz.configs.specviz.plugins.viewers import SpecvizProfileView
-from jdaviz.core.events import ViewerAddedMessage
+from jdaviz.core.custom_units import PIX2
+from jdaviz.core.events import ViewerAddedMessage, GlobalDisplayUnitChanged
 from jdaviz.core.helpers import data_has_valid_wcs
-from jdaviz.core.marks import PluginScatter
+from jdaviz.core.marks import PluginScatter, PluginLine
 from jdaviz.core.registries import tool_registry
 from jdaviz.core.template_mixin import TemplateMixin, DatasetSelectMixin
+from jdaviz.core.validunits import check_if_unit_is_per_solid_angle
+from jdaviz.utils import flux_conversion, _eqv_pixar_sr
 
 __all__ = ['CoordsInfo']
 
-_supported_viewer_classes = (SpecvizProfileView,
-                             ImvizImageView,
-                             CubevizImageView,
-                             MosvizImageView,
-                             MosvizProfile2DView)
-
 
 @tool_registry('g-coords-info')
 class CoordsInfo(TemplateMixin, DatasetSelectMixin):
     template_file = __file__, "coords_info.vue"
 
-    dataset_icon = Unicode("").tag(sync=True)  # option for layer (auto, none, or specific layer)
+    _supported_viewer_classes = (SpecvizProfileView,
+                                 ImvizImageView,
+                                 CubevizImageView,
+                                 RampvizImageView,
+                                 RampvizProfileView,
+                                 MosvizImageView,
+                                 MosvizProfile2DView)
+
+    _viewer_classes_with_marker = (RampvizProfileView, SpecvizProfileView, MosvizProfile2DView)
+
+    dataset_icon = Unicode("").tag(
+        sync=True
+    )  # option for layer (auto, none, or specific layer)
     icon = Unicode("").tag(sync=True)  # currently exposed layer
 
     row1a_title = Unicode("").tag(sync=True)
@@ -53,27 +63,59 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
         self._marks = {}
         self._dict = {}  # dictionary representation of current mouseover info
         self._x, self._y = None, None  # latest known cursor positions
+        self.image_unit = None
 
         # subscribe/unsubscribe to mouse events across all existing viewers
         viewer_refs = []
         for viewer in self.app._viewer_store.values():
-            if isinstance(viewer, _supported_viewer_classes):
+            if isinstance(viewer, self._supported_viewer_classes):
                 self._create_viewer_callbacks(viewer)
                 viewer_refs.append(viewer.reference_id)
 
         self.dataset._manual_options = ['auto', 'none']
-        self.dataset.filters = ['layer_in_viewers']
+
+        self.dataset.filters = ['layer_in_viewers', 'is_not_wcs_only', 'layer_is_not_dq']
         if self.app.config == 'imviz':
             # filter out scatter-plot entries (from add_markers API, for example)
             self.dataset.add_filter('is_image')
-        # we also want to include auto-collapsed spectra (spatial subsets)
-        self.dataset._cubeviz_include_spatial_subsets()
 
         # subscribe to mouse events on any new viewers
         self.hub.subscribe(self, ViewerAddedMessage, handler=self._on_viewer_added)
+        if self.config == "cubeviz":
+            self.hub.subscribe(
+                self, GlobalDisplayUnitChanged, handler=self._on_global_display_unit_changed
+            )
+
+    def _create_marks_for_viewer(self, viewer, id=None):
+        if id is None:
+            id = viewer.reference_id
+        if id in self._marks:
+            return
+        if isinstance(viewer, MosvizProfile2DView):
+            self._marks[id] = PluginLine(viewer,
+                                         x=[0, 0], y=[0, 1],
+                                         scales={'x': viewer.scales['x'],
+                                                 'y': LinearScale(min=0, max=1)},
+                                         visible=False)
+        else:
+            self._marks[id] = PluginScatter(viewer,
+                                            marker='rectangle', stroke_width=1,
+                                            visible=False)
+        if isinstance(viewer, MosvizProfileView):
+            matched_id = f"{id}:matched"
+            self._marks[matched_id] = PluginLine(viewer,
+                                                 x=[0, 0], y=[0, 1],
+                                                 scales={'x': viewer.scales['x'],
+                                                         'y': LinearScale(min=0, max=1)},
+                                                 visible=False)
+            viewer.figure.marks = viewer.figure.marks + [self._marks[matched_id]]
+
+        viewer.figure.marks = viewer.figure.marks + [self._marks[id]]
 
     def _create_viewer_callbacks(self, viewer):
-        if isinstance(viewer, _supported_viewer_classes):
+        if isinstance(viewer, self._supported_viewer_classes):
+            if isinstance(viewer, self._viewer_classes_with_marker):
+                self._create_marks_for_viewer(viewer)
             callback = self._viewer_callback(viewer, self._viewer_mouse_event)
             viewer.add_event_callback(callback, events=['mousemove', 'mouseleave', 'mouseenter'])
 
@@ -82,6 +124,13 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
     def _on_viewer_added(self, msg):
         self._create_viewer_callbacks(self.app.get_viewer_by_id(msg.viewer_id))
 
+    def _on_global_display_unit_changed(self, msg):
+
+        # all cubes are converted to surface brightness so we just need to
+        # listen to SB for cubeviz unit changes
+        if msg.axis == "sb":
+            self.image_unit = u.Unit(msg.unit)
+
     @property
     def marks(self):
         """
@@ -94,13 +143,20 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
         # create marks for each of the spectral viewers (will need a listener event to create marks
         # for new viewers if dynamic creation of spectral viewers is ever supported)
         for id, viewer in self.app._viewer_store.items():
-            if isinstance(viewer, SpecvizProfileView):
-                self._marks[id] = PluginScatter(viewer,
-                                                marker='rectangle', stroke_width=1,
-                                                visible=False)
-                viewer.figure.marks = viewer.figure.marks + [self._marks[id]]
+            if isinstance(viewer, self._viewer_classes_with_marker):
+                self._create_marks_for_viewer(viewer, id)
         return self._marks
 
+    @property
+    def _matched_markers(self):
+        if self.app.config == 'specviz2d':
+            return {'specviz2d-0': ['specviz2d-1:matched'],
+                    'specviz2d-1': ['specviz2d-0']}
+        if self.app.config == 'mosviz':
+            return {'mosviz-1': ['mosviz-2:matched'],
+                    'mosviz-2': ['mosviz-1']}
+        return {}
+
     def as_text(self):
         return (f"{self.row1a_title} {self.row1a_text} {self.row1b_title} {self.row1b_text}".strip(),  # noqa
                 f"{self.row2_title} {self.row2_text}".strip(),
@@ -129,9 +185,12 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
 
     def _viewer_mouse_clear_event(self, viewer, data=None):
         self.reset_coords_display()
-        marks = self.marks.get(viewer._reference_id)
-        if marks is not None:
-            marks.visible = False
+        marker_ids = [viewer._reference_id] + self._matched_markers.get(viewer._reference_id, [])
+        for marker_id in marker_ids:
+            marks = self.marks.get(marker_id)
+            if marks is not None:
+                marks.visible = False
+        self.app.state.show_toolbar_buttons = True
 
     def _viewer_mouse_event(self, viewer, data):
         if data['event'] in ('mouseleave', 'mouseenter'):
@@ -179,12 +238,36 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
 
     def update_display(self, viewer, x, y):
         self._dict = {}
-        if isinstance(viewer, SpecvizProfileView):
+        if isinstance(viewer, (SpecvizProfileView, RampvizProfileView)):
             self._spectrum_viewer_update(viewer, x, y)
         elif isinstance(viewer,
-                        (ImvizImageView, CubevizImageView, MosvizImageView, MosvizProfile2DView)):
+                        (ImvizImageView, CubevizImageView,
+                         MosvizImageView, MosvizProfile2DView,
+                         RampvizImageView)
+                        ):
             self._image_viewer_update(viewer, x, y)
 
+    def _image_shape_inds(self, image):
+        # return the indices in image.shape for the x and y dimension, respectively
+        if image.ndim == 3:
+            # cubeviz case
+            return (0, 1)  # (ix_shape, iy_shape)
+        elif image.ndim == 2:
+            return (1, 0)  # (ix_shape, iy_shape)
+        else:  # pragma: no cover
+            raise ValueError(f'does not support ndim={image.ndim}')
+
+    def _get_cube_value(self, image, arr, x, y, viewer):
+        if image.ndim == 3:
+            # cubeviz case:
+            return arr[int(round(x)), int(round(y)), viewer.state.slices[-1]]
+        elif image.ndim == 2:
+            if isinstance(viewer, RampvizImageView):
+                x, y = y, x
+            return arr[int(round(y)), int(round(x))]
+        else:  # pragma: no cover
+            raise ValueError(f'does not support ndim={image.ndim}')
+
     def _image_viewer_update(self, viewer, x, y):
         # Display the current cursor coordinates (both pixel and world) as
         # well as data values. For now we use the first dataset in the
@@ -197,6 +280,7 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             self._viewer_mouse_clear_event(viewer)
             return
 
+        self.app.state.show_toolbar_buttons = False
         if self.dataset.selected == 'auto':
             image = active_layer.layer
         elif self.dataset.selected == 'none':
@@ -214,6 +298,16 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             else:
                 image = None
 
+        # If there is one, get the associated DQ layer for the active layer:
+        associated_dq_layers = None
+        available_plugins = [tray_item['name'] for tray_item in self.app.state.tray_items]
+        if 'g-data-quality' in available_plugins:
+            assoc_children = self.app._get_assoc_data_children(active_layer.layer.label)
+            if assoc_children:
+                data_quality_plugin = self.app.get_tray_item_from_name('g-data-quality')
+                viewer_obj = self.app.get_viewer(viewer)
+                associated_dq_layers = data_quality_plugin.get_dq_layers(viewer_obj)
+
         unreliable_pixel, unreliable_world = False, False
 
         self._dict['axes_x'] = x
@@ -242,7 +336,6 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
                     sky = image.coords.pixel_to_world(x, y).icrs
                 except Exception:  # WCS might not be celestial
                     coords_status = False
-                    self.reset_coords_display()
 
         elif isinstance(viewer, CubevizImageView):
             # TODO: This assumes data_collection[0] is the main reference
@@ -268,7 +361,7 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             if data_wcs:
                 try:
                     if wcs_ndim == 3:
-                        sky = data_wcs.pixel_to_world(viewer.state.slices[-1], y, x)[1].icrs
+                        sky = data_wcs.pixel_to_world(viewer.slice, y, x)[1].icrs
                     else:  # wcs_ndim == 2
                         sky = data_wcs.pixel_to_world(x, y).icrs
                 except Exception:
@@ -282,9 +375,17 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             slice_plugin = self.app._jdaviz_helper.plugins.get('Slice', None)
             if slice_plugin is not None and len(image.shape) == 3:
                 # float to be compatible with default value of nan
-                self._dict['slice'] = float(slice_plugin.slice)
-                self._dict['spectral_axis'] = slice_plugin.wavelength
-                self._dict['spectral_axis:unit'] = slice_plugin._obj.wavelength_unit
+                self._dict['slice'] = float(viewer.slice)
+                self._dict['spectral_axis'] = slice_plugin.value
+                self._dict['spectral_axis:unit'] = slice_plugin._obj.value_unit
+
+        elif isinstance(viewer, RampvizImageView):
+            coords_status = False
+
+            slice_plugin = self.app._jdaviz_helper.plugins.get('Slice', None)
+            if slice_plugin is not None and len(image.shape) == 3:
+                # float to be compatible with default value of nan
+                self._dict['slice'] = float(viewer.slice)
 
         elif isinstance(viewer, MosvizImageView):
 
@@ -323,9 +424,25 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             self.row3_title = ''
             self.row3_text = f'{world_ra_deg} {world_dec_deg} (deg)'
             self.row3_unreliable = unreliable_world
-            # TODO: use sky directly, but need to figure out how to have a compatible "blank" entry
-            self._dict['world'] = (sky.ra.value, sky.dec.value)
+            self._dict['world_ra'] = sky.ra.value
+            self._dict['world_dec'] = sky.dec.value
             self._dict['world:unreliable'] = unreliable_world
+        elif isinstance(viewer, MosvizProfile2DView) and hasattr(getattr(image, 'coords', None),
+                                                                 'pixel_to_world'):
+            # use WCS to expose the wavelength for a 2d spectrum shown in pixel space
+            try:
+                wave, pixel = image.coords.pixel_to_world(x, y)
+            except Exception:  # WCS might not be valid  # pragma: no cover
+                coords_status = False
+            else:
+                coords_status = True
+                self.row2_title = 'Wave'
+                self.row2_text = f'{wave.value:10.5e} {wave.unit.to_string()}'
+                self.row2_unreliable = False
+
+                self.row3_title = '\u00A0'
+                self.row3_text = ""
+                self.row3_unreliable = False
         else:
             self.row2_title = '\u00A0'
             self.row2_text = ""
@@ -340,7 +457,8 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
         self.row1a_title = 'Pixel'
         self.row1a_text = (fmt.format(x, y))
         self.row1_unreliable = unreliable_pixel
-        self._dict['pixel'] = (float(x), float(y))
+        self._dict['pixel_x'] = float(x)
+        self._dict['pixel_y'] = float(y)
         self._dict['pixel:unreliable'] = unreliable_pixel
 
         # Extract data values at this position.
@@ -355,47 +473,102 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
 
         # Extract data values at this position.
         # Check if shape is [x, y, z] or [y, x] and show value accordingly.
-        if image.ndim == 3:
-            # needed for cubeviz
-            ix_shape = 0
-            iy_shape = 1
-        elif image.ndim == 2:
-            ix_shape = 1
-            iy_shape = 0
-        else:  # pragma: no cover
-            raise ValueError(f'does not support ndim={image.ndim}')
+        ix_shape, iy_shape = self._image_shape_inds(image)
 
         if (-0.5 < x < image.shape[ix_shape] - 0.5 and -0.5 < y < image.shape[iy_shape] - 0.5
                 and hasattr(active_layer, 'attribute')):
+
             attribute = active_layer.attribute
+
             if isinstance(viewer, (ImvizImageView, MosvizImageView, MosvizProfile2DView)):
                 value = image.get_data(attribute)[int(round(y)), int(round(x))]
+                if associated_dq_layers is not None:
+                    associated_dq_layer = associated_dq_layers[0]
+                    dq_attribute = associated_dq_layer.state.attribute
+                    dq_data = associated_dq_layer.layer.get_data(dq_attribute)
+                    dq_value = dq_data[int(round(y)), int(round(x))]
                 unit = image.get_component(attribute).units
-            elif isinstance(viewer, CubevizImageView):
+            elif isinstance(viewer, (CubevizImageView, RampvizImageView)):
+                skip_spectral_density_eqv = False
                 arr = image.get_component(attribute).data
                 unit = image.get_component(attribute).units
-                if image.ndim == 3:
-                    value = arr[int(round(x)), int(round(y)), viewer.state.slices[-1]]
-                else:  # 2
-                    value = arr[int(round(y)), int(round(x))]
+                value = self._get_cube_value(
+                    image, arr, x, y, viewer
+                )
+
+                # We don't want to convert for things like moment maps, so check physical type
+                # If unit is flux per pix2, the type will be 'unknown' rather
+                # than surface brightness, so have to multiply the pix2 part out
+                # and check if the numerator is a spectral flux density
+                if check_if_unit_is_per_solid_angle(unit, return_unit=True) == PIX2:
+                    un = u.Unit(unit) * PIX2
+                    physical_type = un.physical_type
+                else:
+                    physical_type = u.Unit(unit).physical_type
+
+                if str(physical_type) not in ("spectral flux density",
+                                              "surface brightness"):
+                    skip_spectral_density_eqv = True
+
+                if self.image_unit is not None and not skip_spectral_density_eqv:
+                    if 'PIXAR_SR' in self.app.data_collection[0].meta:
+                        # Need current slice value and associated unit to use to compute
+                        # spectral density equivalencies that enable Flux to Flux conversions.
+                        # This is needed for units that are not directly convertible/translatable.
+                        slice = viewer.slice_value * u.Unit(self.app._get_display_unit('spectral'))
+
+                        value = flux_conversion(value, unit, self.image_unit,
+                                                eqv=_eqv_pixar_sr(self.app.data_collection[0].meta['PIXAR_SR']),  # noqa: E501
+                                                slice=slice)
+                        unit = self.image_unit
+
+                    elif self.image_unit.is_equivalent(unit):
+                        value = (value * u.Unit(unit)).to_value(u.Unit(self.image_unit))
+                        unit = self.image_unit
+
+                if associated_dq_layers is not None:
+                    associated_dq_layer = associated_dq_layers[0]
+                    dq_attribute = associated_dq_layer.state.attribute
+                    dq_data = associated_dq_layer.layer.get_data(dq_attribute)
+                    dq_value = self._get_cube_value(image, dq_data, x, y, viewer)
+
             self.row1b_title = 'Value'
-            self.row1b_text = f'{value:+10.5e} {unit}'
+
+            if associated_dq_layers is not None:
+                if np.isnan(dq_value):
+                    dq_text = ''
+                else:
+                    dq_text = f' (DQ: {int(dq_value):d})'
+            else:
+                dq_text = ''
+            self.row1b_text = f'{value:+10.5e} {unit}{dq_text}'
             self._dict['value'] = float(value)
-            self._dict['value:unit'] = unit
+            self._dict['value:unit'] = str(unit)
             self._dict['value:unreliable'] = unreliable_pixel
         else:
             self.row1b_title = ''
             self.row1b_text = ''
 
+        if isinstance(viewer, MosvizProfile2DView):
+            self.marks[viewer._reference_id].update_xy([x, x], [0, 1])
+            self.marks[viewer._reference_id].visible = True
+
+            for matched_marker_id in self._matched_markers.get(viewer._reference_id, []):
+                if coords_status and hasattr(getattr(image, 'coords', None), 'pixel_to_world'):
+                    # should already have wave computed from setting the coords-info
+                    matched_viewer = self.app.get_viewer(matched_marker_id.split(':matched')[0])
+                    wave = wave.to_value(matched_viewer.state.x_display_unit)
+                    self.marks[matched_marker_id].update_xy([wave, wave], [0, 1])
+                    self.marks[matched_marker_id].visible = True
+                else:
+                    self.marks[matched_marker_id].visible = False
+
     def _spectrum_viewer_update(self, viewer, x, y):
         def _cursor_fallback():
-            statistic = getattr(viewer.state, 'function', None)
-            cache_key = (viewer.state.layers[0].layer.label, statistic)
-            sp = self.app._get_object_cache.get(cache_key, viewer.data()[0])
             self._dict['axes_x'] = x
-            self._dict['axes_x:unit'] = sp.spectral_axis.unit.to_string()
+            self._dict['axes_x:unit'] = str(viewer.state.x_display_unit)
             self._dict['axes_y'] = y
-            self._dict['axes_y:unit'] = sp.flux.unit.to_string()
+            self._dict['axes_y:unit'] = str(viewer.state.y_display_unit)
             self._dict['data_label'] = ''
 
         def _copy_axes_to_spectral():
@@ -418,8 +591,6 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             self.row3_text = ''
             self.icon = 'mdi-cursor-default'
             self.marks[viewer._reference_id].visible = False
-            # get the units from the first layer
-            # TODO: replace with display units once implemented
             _cursor_fallback()
             _copy_axes_to_spectral()
             return
@@ -437,42 +608,42 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             if self.dataset.selected != 'auto' and self.dataset.selected != lyr.layer.label:
                 continue
 
-            if isinstance(lyr.layer, GroupedSubset):
-                if not isinstance(lyr.layer.subset_state, RoiSubsetState):
-                    # then this is a SPECTRAL subset
-                    continue
-                # For use later in data retrieval
-                subset_label = lyr.layer.label
-                data_label = lyr.layer.data.label
-            elif ((not isinstance(lyr.layer, BaseData)) or (lyr.layer.ndim not in (1, 3))):
+            if ((not isinstance(lyr.layer, BaseData)) or (lyr.layer.ndim not in (1, 3))):
                 continue
-            else:
-                subset_label = None
-                data_label = lyr.layer.label
+            data_label = lyr.layer.label
 
             try:
                 # Cache should have been populated when spectrum was first plotted.
                 # But if not (maybe user changed statistic), we cache it here too.
-                statistic = getattr(viewer.state, 'function', None)
-                cache_key = (lyr.layer.label, statistic)
+                cache_key = lyr.layer.label
                 if cache_key in self.app._get_object_cache:
                     sp = self.app._get_object_cache[cache_key]
                 else:
-                    sp = self._specviz_helper.get_data(data_label=data_label,
-                                                       subset_to_apply=subset_label)
+                    sp = self._specviz_helper.get_data(data_label=data_label)
                     self.app._get_object_cache[cache_key] = sp
 
+                # Calculations have to happen in the frame of viewer display units.
+                disp_wave = sp.spectral_axis.to_value(viewer.state.x_display_unit, u.spectral())
+
+                # temporarily here, may be removed after upstream units handling
+                # or will be generalized for any sb <-> flux
+                if '_pixel_scale_factor' in sp.meta:
+                    disp_flux = flux_conversion(sp.flux.value, sp.flux.unit, viewer.state.y_display_unit, spec=sp)  # noqa: E501
+                else:
+                    disp_flux = sp.flux.to_value(viewer.state.y_display_unit,
+                                                 u.spectral_density(sp.spectral_axis))
+
                 # Out of range in spectral axis.
                 if (self.dataset.selected != lyr.layer.label and
-                        (x < sp.spectral_axis.value.min() or x > sp.spectral_axis.value.max())):
+                        (x < disp_wave.min() or x > disp_wave.max())):
                     continue
 
-                cur_i = np.argmin(abs(sp.spectral_axis.value - x))
-                cur_wave = sp.spectral_axis[cur_i]
-                cur_flux = sp.flux[cur_i]
+                cur_i = np.argmin(abs(disp_wave - x))
+                cur_wave = disp_wave[cur_i]
+                cur_flux = disp_flux[cur_i]
 
-                dx = cur_wave.value - x
-                dy = cur_flux.value - y
+                dx = cur_wave - x
+                dy = cur_flux - y
                 cur_distance = math.sqrt(dx * dx + dy * dy)
                 if (closest_distance is None) or (cur_distance < closest_distance):
                     closest_distance = cur_distance
@@ -497,27 +668,39 @@ class CoordsInfo(TemplateMixin, DatasetSelectMixin):
             return
 
         self.row2_title = 'Wave'
-        self.row2_text = f'{closest_wave.value:10.5e} {closest_wave.unit.to_string()}'
-        self._dict['axes_x'] = closest_wave.value
-        self._dict['axes_x:unit'] = closest_wave.unit.to_string()
-        if closest_wave.unit != u.pix:
+        self.row2_text = f'{closest_wave:10.5e} {viewer.state.x_display_unit}'
+        self._dict['axes_x'] = closest_wave
+        self._dict['axes_x:unit'] = viewer.state.x_display_unit
+        if viewer.state.x_display_unit != u.pix:
             self.row2_text += f' ({int(closest_i)} pix)'
             if self.app.config == 'cubeviz':
                 # float to be compatible with nan
                 self._dict['slice'] = float(closest_i)
-                self._dict['spectral_axis'] = closest_wave.value
-                self._dict['spectral_axis:unit'] = closest_wave.unit.to_string()
+                self._dict['spectral_axis'] = closest_wave
+                self._dict['spectral_axis:unit'] = viewer.state.x_display_unit
             else:
                 # float to be compatible with nan
                 self._dict['index'] = float(closest_i)
 
+        if viewer.state.y_display_unit is None:
+            flux_unit = ""
+        else:
+            flux_unit = viewer.state.y_display_unit
         self.row3_title = 'Flux'
-        self.row3_text = f'{closest_flux.value:10.5e} {closest_flux.unit.to_string()}'
-        self._dict['axes_y'] = closest_flux.value
-        self._dict['axes_y:unit'] = closest_flux.unit.to_string()
+        self.row3_text = f'{closest_flux:10.5e} {flux_unit}'
+        self._dict['axes_y'] = closest_flux
+        self._dict['axes_y:unit'] = str(viewer.state.y_display_unit)
 
-        self.icon = closest_icon
+        if closest_icon is not None:
+            self.icon = closest_icon
+        else:
+            self.icon = ""
 
-        self.marks[viewer._reference_id].update_xy([closest_wave.value], [closest_flux.value])  # noqa
+        self.marks[viewer._reference_id].update_xy([closest_wave], [closest_flux])
         self.marks[viewer._reference_id].visible = True
+        for matched_marker_id in self._matched_markers.get(viewer._reference_id, []):
+            # NOTE: this currently assumes the the matched marker is a vertical line with a
+            # normalized y-scale
+            self.marks[matched_marker_id].update_xy([closest_i, closest_i], [0, 1])
+            self.marks[matched_marker_id].visible = True
         _copy_axes_to_spectral()
