@@ -1,18 +1,18 @@
 import numpy as np
 
+import astropy.units as u
 from astropy.wcs.utils import pixel_to_pixel
 from astropy.visualization import ImageNormalize, LinearStretch, PercentileInterval
 from glue.core.link_helpers import LinkSame
 from glue_jupyter.bqplot.image import BqplotImageView
 
 from jdaviz.configs.imviz import wcs_utils
-from jdaviz.configs.imviz.helper import layer_is_image_data, get_top_layer_index
 from jdaviz.core.astrowidgets_api import AstrowidgetsImageViewerMixin
 from jdaviz.core.events import SnackbarMessage
-from jdaviz.core.helpers import data_has_valid_wcs
 from jdaviz.core.registries import viewer_registry
 from jdaviz.core.freezable_state import FreezableBqplotImageViewerState
 from jdaviz.configs.default.plugins.viewers import JdavizViewerMixin
+from jdaviz.utils import data_has_valid_wcs, layer_is_image_data, get_top_layer_index
 
 __all__ = ['ImvizImageView']
 
@@ -24,8 +24,8 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
                     ['jdaviz:homezoom', 'jdaviz:prevzoom'],
                     ['jdaviz:boxzoommatch', 'jdaviz:boxzoom'],
                     ['jdaviz:panzoommatch', 'jdaviz:imagepanzoom'],
-                    ['bqplot:circle', 'bqplot:rectangle', 'bqplot:ellipse',
-                     'jdaviz:singlepixelregion'],
+                    ['bqplot:truecircle', 'bqplot:rectangle', 'bqplot:ellipse',
+                     'bqplot:circannulus'],
                     ['jdaviz:blinkonce', 'jdaviz:contrastbias'],
                     ['jdaviz:sidebar_plot', 'jdaviz:sidebar_export', 'jdaviz:sidebar_compass']
                 ]
@@ -35,6 +35,8 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        # provide reference from state back to viewer to use for zoom syncing
+        self.state._set_viewer(self)
         self.init_astrowidgets_api()
         self._subscribe_to_layers_update()
 
@@ -79,26 +81,20 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
             if key_pressed in ('b', 'B'):
                 self.blink_once(reversed=key_pressed=='B')  # noqa: E225
 
-            elif key_pressed == 'l' and self.line_profile_xy.plugin_opened:
-                # Same data as mousemove above.
-                image = self.active_image_layer.layer
-                x = data['domain']['x']
-                y = data['domain']['y']
-                if x is None or y is None:  # Out of bounds
-                    return
-                x, y, _, _ = self._get_real_xy(image, x, y)
-                self.line_profile_xy.selected_x = x
-                self.line_profile_xy.selected_y = y
-                self.line_profile_xy.selected_viewer = self.reference_id
-                self.line_profile_xy.vue_draw_plot()
-
     def blink_once(self, reversed=False):
         # Simple blinking of images - this will make it so that only one
         # layer is visible at a time and cycles through the layers.
 
-        # Exclude Subsets: They are global
+        # Exclude Subsets (they are global) and children via associated data
+
+        def is_parent(data):
+            return self.session.jdaviz_app._get_assoc_data_parent(data.label) is None
+
         valid = [ilayer for ilayer, layer in enumerate(self.state.layers)
-                 if layer_is_image_data(layer.layer)]
+                 if layer_is_image_data(layer.layer) and is_parent(layer.layer)]
+        children = [ilayer for ilayer, layer in enumerate(self.state.layers)
+                    if layer_is_image_data(layer.layer) and not is_parent(layer.layer)]
+
         n_layers = len(valid)
 
         if n_layers == 1:
@@ -126,7 +122,12 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
                 next_layer = valid[(valid.index(visible[-1]) + delta) % n_layers]
                 self.state.layers[next_layer].visible = True
 
-                for ilayer in (set(valid) - set([next_layer])):
+                # make invisible all parent layers other than the next layer:
+                layers_to_set_not_visible = set(valid) - set([next_layer])
+                # no child layers are visible by default:
+                layers_to_set_not_visible.update(set(children))
+
+                for ilayer in layers_to_set_not_visible:
                     self.state.layers[ilayer].visible = False
 
                 # We can display the active data label in Compass plugin.
@@ -139,7 +140,7 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
                             'imviz-line-profile-xy')
                     except KeyError:  # pragma: no cover
                         return
-                self.line_profile_xy.selected_viewer = self.reference_id
+                self.line_profile_xy.viewer_selected = self.reference_id
                 self.line_profile_xy.vue_draw_plot()
 
     def on_limits_change(self, *args):
@@ -149,6 +150,8 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
             if self.compass is not None:
                 self.compass.clear_compass()
             return
+        if i is None:
+            return
         self.set_compass(self.state.layers[i].layer)
 
     @property
@@ -159,9 +162,23 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
         except IndexError:
             data_label = ''
         else:
-            data_label = self.state.layers[i].layer.label
+            if i is None:
+                data_label = ''
+            else:
+                data_label = self.state.layers[i].layer.label
         return data_label
 
+    @property
+    def first_loaded_data(self):
+        """Data that is first loaded into the viewer.
+        This may not be the visible layer.
+        Returns `None` if no real data is loaded.
+        """
+        for lyr in self.layers:
+            data = lyr.layer
+            if layer_is_image_data(data):
+                return data
+
     def _get_real_xy(self, image, x, y, reverse=False):
         """Return real (X, Y) position and status in case of dithering as well as whether the
         results were within the bounding box of the reference data or required possibly inaccurate
@@ -182,11 +199,11 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
             # we aren't actually guaranteed to get a SkyCoord out, just for images
             # with valid celestial WCS
             try:
-                link_type = self.get_link_type(image.label)
+                align_by = self.get_alignment_method(image.label).lower()
 
                 # Convert X,Y from reference data to the one we are actually seeing.
                 # world_to_pixel return scalar ndarray that we need to convert to float.
-                if link_type == 'wcs':
+                if align_by == 'wcs':
                     if not reverse:
                         outside_ref_bounding_box = wcs_utils.data_outside_gwcs_bounding_box(
                             self.state.reference_data, x, y)
@@ -222,17 +239,20 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
         image, which can be inaccurate if given image is dithered and
         they are linked by WCS.
         """
-        if data_has_valid_wcs(image) and self.get_link_type(image.label) == 'wcs':
-            # Convert X,Y from reference data to the one we are actually seeing.
-            x = image.coords.world_to_pixel(self.state.reference_data.coords.pixel_to_world(
+        if self.state.reference_data.meta.get('_WCS_ONLY', False):
+            corner_world_coords = self.state.reference_data.coords.pixel_to_world(
                 (self.state.x_min, self.state.x_min, self.state.x_max, self.state.x_max),
-                (self.state.y_min, self.state.y_max, self.state.y_max, self.state.y_min)))
+                (self.state.y_min, self.state.y_max, self.state.y_max, self.state.y_min)
+            )
+            # Convert X,Y from reference data to the one we are actually seeing.
+            x = image.coords.world_to_pixel(corner_world_coords)
             zoom_limits = np.array(list(zip(x[0], x[1])))
         else:
             zoom_limits = np.array(((self.state.x_min, self.state.y_min),
                                     (self.state.x_min, self.state.y_max),
                                     (self.state.x_max, self.state.y_max),
                                     (self.state.x_max, self.state.y_min)))
+
         return zoom_limits
 
     def set_compass(self, image):
@@ -268,7 +288,7 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
                 if hasattr(layer_state, 'layer') and
                 layer_is_image_data(layer_state.layer)]
 
-    def get_link_type(self, data_label):
+    def get_alignment_method(self, data_label):
         """Find the type of ``glue`` linking between the given
         data label with the reference data in viewer.
 
@@ -279,8 +299,8 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
 
         Returns
         -------
-        link_type : {'pixels', 'wcs', 'self'}
-            One of the link types accepted by :func:`~jdaviz.configs.imviz.helper.link_image_data`
+        align_by : {'pixels', 'wcs', 'self'}
+            One of the link types accepted by the Orientation plugin
             or ``'self'`` if the data label belongs to the reference data itself.
 
         Raises
@@ -292,23 +312,61 @@ class ImvizImageView(JdavizViewerMixin, BqplotImageView, AstrowidgetsImageViewer
         if len(self.session.application.data_collection) == 0:
             raise ValueError('No reference data for link look-up')
 
-        # the original links were created against data_collection[0], not necessarily
-        # against the current viewer reference_data
-        ref_label = self.session.application.data_collection[0].label
+        ref_label = getattr(self.state.reference_data, 'label', None)
         if data_label == ref_label:
             return 'self'
 
-        link_type = None
+        if ref_label in self.state.wcs_only_layers:
+            return 'wcs'
+
+        align_by = None
         for elink in self.session.application.data_collection.external_links:
             elink_labels = (elink.data1.label, elink.data2.label)
             if data_label in elink_labels and ref_label in elink_labels:
                 if isinstance(elink, LinkSame):  # Assumes WCS link never uses LinkSame
-                    link_type = 'pixels'
+                    align_by = 'pixels'
                 else:  # If not pixels, must be WCS
-                    link_type = 'wcs'
+                    align_by = 'wcs'
                 break  # Might have duplicate, just grab first match
 
-        if link_type is None:
+        if align_by is None:
             raise ValueError(f'{data_label} not found in data collection external links')
 
-        return link_type
+        return align_by
+
+    def _get_fov(self, wcs=None):
+        if wcs is None:
+            wcs = self.state.reference_data.coords
+        if self.jdaviz_app._align_by != "wcs" or wcs is None:
+            return
+
+        # compute the mean of the height and width of the
+        # viewer's FOV on ``data`` in world units:
+        x_corners = [
+            self.state.x_min,
+            self.state.x_max,
+            self.state.x_min
+        ]
+        y_corners = [
+            self.state.y_min,
+            self.state.y_min,
+            self.state.y_max
+        ]
+
+        sky_corners = wcs.pixel_to_world(x_corners, y_corners)
+        height_sky = abs(sky_corners[0].separation(sky_corners[2]))
+        width_sky = abs(sky_corners[0].separation(sky_corners[1]))
+        fov_sky = u.Quantity([height_sky, width_sky]).mean()
+        return fov_sky
+
+    def _get_center_skycoord(self, data=None):
+        # get SkyCoord for the center of ``data`` in this viewer:
+        x_cen = (self.state.x_min + self.state.x_max) * 0.5
+        y_cen = (self.state.y_min + self.state.y_max) * 0.5
+
+        if (self.jdaviz_app._align_by == "wcs" or data is None
+                or data.label == self.state.reference_data.label):
+            return self.state.reference_data.coords.pixel_to_world(x_cen, y_cen)
+
+        if data.coords is not None:
+            return data.coords.pixel_to_world(x_cen, y_cen)
