@@ -14,8 +14,8 @@ from numpy.testing import assert_allclose
 from jdaviz.configs.imviz.tests.utils import BaseImviz_WCS_NoWCS, BaseImviz_WCS_WCS
 
 
-# TODO: Remove skip when https://github.com/bqplot/bqplot/issues/1393 is resolved.
-@pytest.mark.skip(reason="Cannot test due to file dialog popup")
+# TODO: Remove skip when https://github.com/bqplot/bqplot/pull/1397/files#r726500097 is resolved.
+@pytest.mark.skip(reason="Cannot test due to async JS callback")
 class TestSave(BaseImviz_WCS_NoWCS):
 
     def test_save(self, tmpdir):
@@ -23,39 +23,29 @@ class TestSave(BaseImviz_WCS_NoWCS):
         self.viewer.save(filename)
 
         # This only tests that something saved, not the content.
-        assert os.path.isfile(os.path.join(tmpdir.strpath, 'myimage.png'))
+        assert os.path.isfile(f'{filename}.png')
 
 
 class TestCenterOffset(BaseImviz_WCS_NoWCS):
 
     def test_center_offset_pixel(self):
         self.viewer.center_on((0, 1))
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-5, 5, -4, 6))
+        assert_allclose(self.viewer.get_limits(), (-5, 5, -4, 6))
 
         self.viewer.offset_by(1 * u.pix, -1 * u.dimensionless_unscaled)
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-4, 6, -5, 5))
+        assert_allclose(self.viewer.get_limits(), (-4, 6, -5, 5))
 
         self.viewer.offset_by(1, 0)
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-3, 7, -5, 5))
+        assert_allclose(self.viewer.get_limits(), (-3, 7, -5, 5))
 
         # Out-of-bounds centering is now allowed because it is needed
         # for dithering use case.
         self.viewer.center_on((-1, 99999))
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-6, 4, 9.99940e+04, 1.00004e+05))
+        assert_allclose(self.viewer.get_limits(), (-6, 4, 9.99940e+04, 1.00004e+05))
 
         # Sometimes invalid WCS also gives such output, should be no-op
         self.viewer.center_on((np.array(np.nan), np.array(np.nan)))
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-6, 4, 9.99940e+04, 1.00004e+05))
+        assert_allclose(self.viewer.get_limits(), (-6, 4, 9.99940e+04, 1.00004e+05))
 
     def test_center_offset_sky(self):
         # Blink to the one with WCS because the last loaded data is shown.
@@ -63,14 +53,11 @@ class TestCenterOffset(BaseImviz_WCS_NoWCS):
 
         sky = self.wcs.pixel_to_world(0, 1)
         self.viewer.center_on(sky)
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-5, 5, -4, 6))
+        assert_allclose(self.viewer.get_limits(), (-5, 5, -4, 6))
 
         dsky = 0.1 * u.arcsec
         self.viewer.offset_by(-dsky, dsky)
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
+        assert_allclose(self.viewer.get_limits(),
                         (-4.9, 5.1, -3.90000000002971, 6.09999999997029))
 
         # Cannot mix pixel with sky
@@ -96,28 +83,24 @@ class TestCenterOffset(BaseImviz_WCS_NoWCS):
 class TestCenter(BaseImviz_WCS_WCS):
 
     def test_center_on_pix(self):
-        self.imviz.link_data(link_type='wcs', error_on_fail=True)
+        self.imviz.link_data(align_by='wcs')
 
         # This is the second loaded data that is dithered by 1-pix.
         self.viewer.center_on((0, 0))
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-6, 4, -5, 5))
+        expected_position = [-6.75, 4.25, -5.75, 5.25]
+        rtol = 1e-4
+        assert_allclose(self.viewer.get_limits(), expected_position, rtol=rtol)
 
         # This is the first data.
         self.viewer.blink_once()
         self.viewer.center_on((0, 0))
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-5, 5, -5, 5))
+        assert_allclose(self.viewer.get_limits(), [-5.75, 5.25, -5.75, 5.25], rtol=rtol)
 
         # Centering by sky on second data.
         self.viewer.blink_once()
         sky = self.wcs_2.pixel_to_world(0, 0)
         self.viewer.center_on(sky)
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
-                        (-6, 4, -5, 5))
+        assert_allclose(self.viewer.get_limits(), expected_position, rtol=rtol)
 
 
 class TestZoom(BaseImviz_WCS_NoWCS):
@@ -133,8 +116,7 @@ class TestZoom(BaseImviz_WCS_NoWCS):
 
     def assert_zoom_results(self, zoom_level, x_min, x_max, y_min, y_max, dpix):
         assert_allclose(self.viewer.zoom_level, zoom_level)
-        assert_allclose((self.viewer.state.x_min, self.viewer.state.x_max,
-                         self.viewer.state.y_min, self.viewer.state.y_max),
+        assert_allclose(self.viewer.get_limits(),
                         (x_min + dpix, x_max + dpix,
                          y_min + dpix, y_max + dpix))
 
@@ -180,7 +162,7 @@ class TestCmapStretchCuts(BaseImviz_WCS_NoWCS):
             'Gray', 'Viridis', 'Plasma', 'Inferno', 'Magma', 'Purple-Blue',
             'Yellow-Green-Blue', 'Yellow-Orange-Red', 'Red-Purple', 'Blue-Green',
             'Hot', 'Red-Blue', 'Red-Yellow-Blue', 'Purple-Orange', 'Purple-Green',
-            'Rainbow', 'Seismic',
+            'Random', 'Rainbow', 'Seismic',
             'Reversed: Gray', 'Reversed: Viridis', 'Reversed: Plasma', 'Reversed: Inferno',
             'Reversed: Magma', 'Reversed: Hot', 'Reversed: Rainbow']
 
@@ -285,7 +267,7 @@ class TestMarkers(BaseImviz_WCS_NoWCS):
         self.viewer.add_markers(tbl)
         data = self.imviz.app.data_collection[2]
         assert data.label == 'default-marker-name'
-        assert data.style.color == '#ff0000'  # Converted to hex now but still red
+        assert data.style.color in ('red', '#ff0000')
         assert data.style.marker == 'o'
         assert_allclose(data.style.markersize, 5)
         assert_allclose(data.style.alpha, 1)
@@ -307,7 +289,7 @@ class TestMarkers(BaseImviz_WCS_NoWCS):
         self.viewer.add_markers(tbl, use_skycoord=True, marker_name='my_sky')
         data = self.imviz.app.data_collection[3]
         assert data.label == 'my_sky'
-        assert data.style.color == '#00ff00'  # Converted to hex now but still green
+        assert data.style.color in ((0, 1, 0), '#00ff00')
         assert data.style.marker == 'o'
         assert_allclose(data.style.markersize, 3)  # Glue default
         assert_allclose(data.style.alpha, 0.8)
@@ -317,7 +299,7 @@ class TestMarkers(BaseImviz_WCS_NoWCS):
         assert self.viewer.layers[3].state.fill is False
 
         # Make sure the other marker is not changed.
-        assert self.imviz.app.data_collection[2].style.color == '#ff0000'
+        assert self.imviz.app.data_collection[2].style.color in ('red', '#ff0000')
         assert self.viewer.layers[2].state.fill is True
 
         # TODO: How to check imviz.app.data_collection.links is correct?
