@@ -35,9 +35,7 @@ class TestCatalogs:
 
     # testing that the plugin search does not crash when no data/image is provided
     def test_plugin_no_image(self, imviz_helper):
-        self.imviz = imviz_helper
-
-        catalogs_plugin = self.imviz.app.get_tray_item_from_name('imviz-catalogs')
+        catalogs_plugin = imviz_helper.plugins["Catalog Search"]._obj
         catalogs_plugin.plugin_opened = True
         # running the search without any data loaded into Imviz
         catalogs_plugin.vue_do_search()
@@ -51,9 +49,7 @@ class TestCatalogs:
 
         imviz_helper.load_data(ndd, data_label='no_results_data')
 
-        self.imviz = imviz_helper
-
-        catalogs_plugin = self.imviz.app.get_tray_item_from_name('imviz-catalogs')
+        catalogs_plugin = imviz_helper.plugins["Catalog Search"]._obj
         catalogs_plugin.plugin_opened = True
         catalogs_plugin.vue_do_search()
 
@@ -88,15 +84,13 @@ class TestCatalogs:
                             'NAXIS2': 1489})
         imviz_helper.load_data(hdu1, data_label='has_wcs')
 
-        self.imviz = imviz_helper
-
-        catalogs_plugin = self.imviz.app.get_tray_item_from_name('imviz-catalogs')
+        catalogs_plugin = imviz_helper.plugins["Catalog Search"]._obj
         catalogs_plugin.plugin_opened = True
         # This basically calls the following under the hood:
         #   skycoord_center = SkyCoord(6.62754354, 1.54466139, unit="deg")
         #   zoom_radius = r_max = 3 * u.arcmin
         #   query_region_result = SDSS.query_region(skycoord_center, radius=zoom_radius, ...)
-        catalogs_plugin.vue_do_search()
+        catalogs_plugin.search(error_on_fail=True)
 
         # number of results should be > 500 or so
         # Answer was determined by running the search with the image in the notebook.
@@ -111,32 +105,45 @@ class TestCatalogs:
 
         # test loading from file
         table = imviz_helper.app._catalog_source_table
-        skycoord_table = SkyCoord(table['ra'],
-                                  table['dec'],
-                                  unit='deg')
-        qtable = QTable({'sky_centroid': skycoord_table})
+        qtable = QTable({'sky_centroid': SkyCoord(table['ra'], table['dec'], unit='deg'),
+                         'label': table['objid']})
         tmp_file = tmp_path / 'test.ecsv'
         qtable.write(tmp_file, overwrite=True)
 
         catalogs_plugin.from_file = str(tmp_file)
         # setting filename from API will automatically set catalog to 'From File...'
         assert catalogs_plugin.catalog.selected == 'From File...'
-        catalogs_plugin.vue_do_search()
+        catalogs_plugin.search(error_on_fail=True)
         assert catalogs_plugin.results_available
         assert catalogs_plugin.number_of_results == prev_results
 
+        catalogs_plugin.table.selected_rows = catalogs_plugin.table.items[0:2]
+        assert len(catalogs_plugin.table.selected_rows) == 2
+
+        assert imviz_helper.viewers['imviz-0']._obj.state.x_min == -0.5
+        assert imviz_helper.viewers['imviz-0']._obj.state.x_max == 2047.5
+        assert imviz_helper.viewers['imviz-0']._obj.state.y_min == -0.5
+        assert imviz_helper.viewers['imviz-0']._obj.state.y_max == 1488.5
+
+        catalogs_plugin.vue_zoom_in()
+
+        assert imviz_helper.viewers['imviz-0']._obj.state.x_min == 858.24969
+        assert imviz_helper.viewers['imviz-0']._obj.state.x_max == 958.38461
+        assert imviz_helper.viewers['imviz-0']._obj.state.y_min == 278.86265
+        assert imviz_helper.viewers['imviz-0']._obj.state.y_max == 378.8691
+
 
 def test_from_file_parsing(imviz_helper, tmp_path):
-    catalogs_plugin = imviz_helper.app.get_tray_item_from_name('imviz-catalogs')
+    catalogs_plugin = imviz_helper.plugins["Catalog Search"]._obj
 
     # _on_file_path_changed is fired when changing the selection in the file dialog
-    catalogs_plugin._on_file_path_changed({'new': './invalid_path'})
+    catalogs_plugin.catalog._on_file_path_changed({'new': './invalid_path'})
     assert catalogs_plugin.from_file_message == 'File path does not exist'
 
-    # observe('from_file') is fired when setting from_file from the API (or after clicking
-    # select in the file dialog)
-    with pytest.raises(ValueError, match='./invalid_path does not exist'):
-        catalogs_plugin.from_file = './invalid_path'
+    # observe('from_file') is fired when setting from_file from the API or via import_file
+    # (or after clicking select in the file dialog)
+    with pytest.raises(ValueError, match='./invalid_path is not a valid file path'):
+        catalogs_plugin.import_catalog('./invalid_path')
 
     # setting to a blank string from the API resets the catalog selection to the
     # default/first entry
@@ -145,13 +152,13 @@ def test_from_file_parsing(imviz_helper, tmp_path):
 
     not_table_file = tmp_path / 'not_table.tst'
     not_table_file.touch()
-    catalogs_plugin._on_file_path_changed({'new': not_table_file})
+    catalogs_plugin.catalog._on_file_path_changed({'new': not_table_file})
     assert catalogs_plugin.from_file_message == 'Could not parse file with astropy.table.QTable.read'  # noqa
 
     qtable = QTable({'not_sky_centroid': [1, 2, 3]})
     not_valid_table = tmp_path / 'not_valid_table.ecsv'
     qtable.write(not_valid_table, overwrite=True)
-    catalogs_plugin._on_file_path_changed({'new': not_valid_table})
+    catalogs_plugin.catalog._on_file_path_changed({'new': not_valid_table})
     assert catalogs_plugin.from_file_message == 'Table does not contain required sky_centroid column'  # noqa
 
 
@@ -167,18 +174,34 @@ def test_offline_ecsv_catalog(imviz_helper, image_2d_wcs, tmp_path):
     imviz_helper.load_data(ndd, data_label='data_with_wcs')
     assert len(imviz_helper.app.data_collection) == 1
 
-    catalogs_plugin = imviz_helper.plugins['Catalog Search']
-    catalogs_plugin._obj.from_file = tbl_file
-    catalogs_plugin._obj.catalog_selected = 'From File...'
-    out_tbl = catalogs_plugin._obj.search()
+    catalogs_plugin = imviz_helper.plugins['Catalog Search']._obj
+    catalogs_plugin.from_file = tbl_file
+    catalogs_plugin.catalog_selected = 'From File...'
+    out_tbl = catalogs_plugin.search(error_on_fail=True)
     assert len(out_tbl) == n_entries
-    assert catalogs_plugin._obj.number_of_results == n_entries
+    assert catalogs_plugin.number_of_results == n_entries
     assert len(imviz_helper.app.data_collection) == 2  # image + markers
 
-    catalogs_plugin._obj.clear()
-    assert not catalogs_plugin._obj.results_available
+    catalogs_plugin.clear()
+    assert not catalogs_plugin.results_available
     assert len(imviz_helper.app.data_collection) == 2  # markers still there, just hidden
 
-    catalogs_plugin._obj.clear(hide_only=False)
-    assert not catalogs_plugin._obj.results_available
+    catalogs_plugin.clear(hide_only=False)
+    assert not catalogs_plugin.results_available
     assert len(imviz_helper.app.data_collection) == 1  # markers gone for good
+
+    catalogs_plugin.table.selected_rows = [catalogs_plugin.table.items[0]]
+
+    assert len(catalogs_plugin.table.selected_rows) == 1
+
+    assert imviz_helper.viewers['imviz-0']._obj.state.x_min == -0.5
+    assert imviz_helper.viewers['imviz-0']._obj.state.x_max == 9.5
+    assert imviz_helper.viewers['imviz-0']._obj.state.y_min == -0.5
+    assert imviz_helper.viewers['imviz-0']._obj.state.y_max == 9.5
+
+    catalogs_plugin.vue_zoom_in()
+
+    assert imviz_helper.viewers['imviz-0']._obj.state.x_min == -49.99966
+    assert imviz_helper.viewers['imviz-0']._obj.state.x_max == 50.00034
+    assert imviz_helper.viewers['imviz-0']._obj.state.y_min == -48.99999
+    assert imviz_helper.viewers['imviz-0']._obj.state.y_max == 51.00001
