@@ -1,3 +1,4 @@
+import os
 import numpy as np
 import pytest
 from astropy import units as u
@@ -14,7 +15,8 @@ from stdatamodels import asdf_in_fits
 
 from jdaviz.configs.imviz.helper import split_filename_with_fits_ext
 from jdaviz.configs.imviz.plugins.parsers import (
-    parse_data, _validate_fits_image2d, _validate_bunit, _parse_image)
+    parse_data, _validate_fits_image2d, _validate_bunit, _parse_image, HAS_ROMAN_DATAMODELS)
+from jdaviz.core.custom_units import PIX2
 
 
 @pytest.mark.parametrize(
@@ -234,15 +236,15 @@ class TestParseImage:
 
     @pytest.mark.remote_data
     def test_parse_jwst_nircam_level2(self, imviz_helper):
-        # TODO: Change back to smaller number (30?) when ITSD is convinced it is them and not us.
-        #       Help desk ticket INC0183598, J. Quick.
-        filename = download_file(self.jwst_asdf_url_1, cache=True, timeout=100)
-
         # Default behavior: Science image
-        imviz_helper.load_data(filename)
+        with pytest.warns(UserWarning, match='You may be querying for a remote file'):
+            # if you don't pass a `cache` value, a warning should be raised:
+            imviz_helper.load_data(self.jwst_asdf_url_1, timeout=100)
+
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('DATA')
-        assert data.label == 'contents[DATA]'  # download_file returns cache loc
+        expected_label = os.path.splitext(os.path.basename(self.jwst_asdf_url_1))[0] + '[DATA]'
+        assert data.label == expected_label
         assert data.shape == (2048, 2048)
         assert isinstance(data.coords, GWCS)
         assert comp.units == 'MJy/sr'
@@ -251,7 +253,7 @@ class TestParseImage:
         # --- Since download is expensive, we attach GWCS-specific tests here. ---
 
         # Ensure interactive region supports GWCS. Also see test_regions.py
-        imviz_helper._apply_interactive_region('bqplot:circle',
+        imviz_helper._apply_interactive_region('bqplot:truecircle',
                                                (965, 1122),
                                                (976.9, 1110.1))  # Star
         imviz_helper._apply_interactive_region('bqplot:rectangle',
@@ -265,9 +267,11 @@ class TestParseImage:
         # Test simple aperture photometry plugin.
         phot_plugin = imviz_helper.app.get_tray_item_from_name('imviz-aper-phot-simple')
         phot_plugin.data_selected = 'contents[DATA]'
-        phot_plugin.subset_selected = 'Subset 1'
+        phot_plugin.aperture_selected = 'Subset 1'
+        assert phot_plugin.aperture.selected_validity.get('is_aperture')
         assert_allclose(phot_plugin.background_value, 0)
-        phot_plugin.bg_subset_selected = 'Subset 2'
+        phot_plugin.background_selected = 'Subset 2'
+        assert phot_plugin.aperture.selected_validity.get('is_aperture')
         assert_allclose(phot_plugin.background_value, 0.1741226315498352)  # Subset 2 median
         # NOTE: jwst.datamodels.find_fits_keyword("PHOTMJSR")
         phot_plugin.counts_factor = (data.meta['photometry']['conversion_megajanskys'] /
@@ -284,7 +288,7 @@ class TestParseImage:
         data_unit = u.MJy / u.sr
         assert_quantity_allclose(tbl[0]['background'], 0.1741226315498352 * data_unit)
         assert_quantity_allclose(tbl[0]['sum'], 4.486487e-11 * u.MJy, rtol=1e-6)
-        assert_quantity_allclose(tbl[0]['sum_aper_area'], 111.220234 * (u.pix * u.pix))
+        assert_quantity_allclose(tbl[0]['sum_aper_area'], 111.220234 * PIX2)
         assert_quantity_allclose(tbl[0]['pixarea_tot'], 9.33677e-14 * u.sr, rtol=1e-6)
         assert_quantity_allclose(tbl[0]['aperture_sum_counts'], 132061.576643 * u.count, rtol=1e-6)
         assert_quantity_allclose(tbl[0]['aperture_sum_counts_err'], 363.402775 * u.count)
@@ -310,7 +314,7 @@ class TestParseImage:
         # --- Back to parser testing below. ---
 
         # Request specific extension (name + ver, but ver is not used), use given label
-        imviz_helper.load_data(filename, ext=('DQ', 42),
+        imviz_helper.load_data(self.jwst_asdf_url_1, cache=True, ext=('DQ', 42),
                                data_label='jw01072001001_01101_00001_nrcb1_cal',
                                show_in_viewer=False)
         data = imviz_helper.app.data_collection[1]
@@ -320,6 +324,7 @@ class TestParseImage:
         assert comp.units == ''
 
         # Pass in HDUList directly + ext (name only), use given label
+        filename = download_file(self.jwst_asdf_url_1, cache=True)
         with fits.open(filename) as pf:
             imviz_helper.load_data(pf, ext='SCI',
                                    data_label='jw01072001001_01101_00001_nrcb1_cal',
@@ -360,12 +365,11 @@ class TestParseImage:
     @pytest.mark.remote_data
     def test_parse_jwst_niriss_grism(self, imviz_helper):
         """No valid image GWCS for Imviz, will fall back to loading without WCS."""
-        filename = download_file(self.jwst_asdf_url_2, cache=True)
-
-        imviz_helper.load_data(filename, show_in_viewer=False)
+        imviz_helper.load_data(self.jwst_asdf_url_2, cache=True, show_in_viewer=False)
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('DATA')
-        assert data.label == 'contents[DATA]'  # download_file returns cache loc
+        expected_label = os.path.splitext(os.path.basename(self.jwst_asdf_url_2))[0] + '[DATA]'
+        assert data.label == expected_label
         assert data.shape == (2048, 2048)
         assert data.coords is None
         assert comp.units == 'DN/s'
@@ -377,15 +381,16 @@ class TestParseImage:
         filename = download_file(url, cache=True)
 
         # Default behavior: Load first image
-        imviz_helper.load_data(filename)
+        imviz_helper.load_data(url, cache=True)
         data = imviz_helper.app.data_collection[0]
         comp = data.get_component('SCI,1')
-        assert data.label == 'contents[SCI,1]'  # download_file returns cache loc
-        assert data.shape == (4299, 4220)
+        expected_label = os.path.splitext(os.path.basename(url))[0] + '[SCI,1]'
+        assert data.label == expected_label
+        assert data.shape == (4299, 4219)
         assert_allclose(data.meta['PHOTFLAM'], 7.8711728E-20)
         assert isinstance(data.coords, WCS)
         assert comp.units == 'electron/s'
-        assert comp.data.shape == (4299, 4220)
+        assert comp.data.shape == (4299, 4219)
 
         # --- Since download is expensive, we attach FITS WCS-specific tests here. ---
 
@@ -395,7 +400,8 @@ class TestParseImage:
                                                (1512, 2611))  # Galaxy
         phot_plugin = imviz_helper.app.get_tray_item_from_name('imviz-aper-phot-simple')
         phot_plugin.data_selected = 'contents[SCI,1]'
-        phot_plugin.subset_selected = 'Subset 1'
+        phot_plugin.aperture_selected = 'Subset 1'
+        assert phot_plugin.aperture.selected_validity.get('is_aperture')
         phot_plugin.background_value = 0.0014  # Manual entry: Median on whole array
         assert_allclose(phot_plugin.pixel_area, 0.0025)  # Not used but still auto-populated
         phot_plugin.vue_do_aper_phot()
@@ -407,29 +413,29 @@ class TestParseImage:
         assert_allclose(sky.dec.deg, 10.802045612042956, rtol=1e-3)
         data_unit = u.electron / u.s
         assert_quantity_allclose(tbl[0]['background'], 0.0014 * data_unit)
-        assert_quantity_allclose(tbl[0]['sum'], 126.63604794 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['sum_aper_area'], 2583.959958 * (u.pix * u.pix), rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['sum'], 115.944737 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['sum_aper_area'], 2583.959958 * PIX2, rtol=1e-3)
         assert_array_equal(tbl[0]['pixarea_tot'], None)
         assert_array_equal(tbl[0]['aperture_sum_counts'], None)
         assert_array_equal(tbl[0]['aperture_sum_counts_err'], None)
         assert_array_equal(tbl[0]['counts_fac'], None)
         assert_array_equal(tbl[0]['aperture_sum_mag'], None)
         assert_array_equal(tbl[0]['flux_scaling'], None)
-        assert_quantity_allclose(tbl[0]['min'], -0.02858124 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['max'], 3.94278287 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['mean'], 0.04910025 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['median'], 0.02168917 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['mode'], -0.033133 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['std'], 0.144362 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['mad_std'], 0.0233135 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['var'], 0.02084 * (data_unit * data_unit), rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['biweight_location'], 0.02098 * data_unit, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['biweight_midvariance'], 0.00062222 * (data_unit * data_unit), rtol=1e-3)  # noqa
-        assert_quantity_allclose(tbl[0]['fwhm'], 25.22627 * u.pix, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['semimajor_sigma'], 12.9581226 * u.pix, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['semiminor_sigma'], 7.8490214 * u.pix, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['orientation'], 88.733606 * u.deg, rtol=1e-3)
-        assert_quantity_allclose(tbl[0]['eccentricity'], 0.795676, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['min'], -0.042749 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['max'], 1.588676 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['mean'], 0.04495 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['median'], 0.021585 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['mode'], -0.025143 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['std'], 0.102367 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['mad_std'], 0.023415 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['var'], 0.010479 * (data_unit * data_unit), rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['biweight_location'], 0.021173 * data_unit, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['biweight_midvariance'], 0.00062 * (data_unit * data_unit), rtol=1e-3)  # noqa
+        assert_quantity_allclose(tbl[0]['fwhm'], 21.538407 * u.pix, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['semimajor_sigma'], 10.04406 * u.pix, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['semiminor_sigma'], 8.150735 * u.pix, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['orientation'], 79.605418 * u.deg, rtol=1e-3)
+        assert_quantity_allclose(tbl[0]['eccentricity'], 0.584355, rtol=1e-3)
 
         # Request specific extension (name only), use given label
         imviz_helper.load_data(filename, ext='CTX', data_label='jclj01010_drz',
@@ -503,3 +509,9 @@ def test_load_valid_not_valid(imviz_helper):
     assert (len(imviz_helper.app.data_collection) == 1
             and imviz_helper.app.data_collection.labels == ['valid'])
     assert_allclose(imviz_helper.app.data_collection[0].get_component('DATA').data, 1)
+
+
+@pytest.mark.skipif(not HAS_ROMAN_DATAMODELS, reason="roman_datamodels is not installed")
+def test_roman_parser(imviz_helper, roman_imagemodel):
+    imviz_helper.load_data(roman_imagemodel, data_label='roman_wfi_image_model', ext='data')
+    assert len(imviz_helper.app.data_collection) == 1
