@@ -6,6 +6,7 @@ from jdaviz.app import Application
 from jdaviz.core.config import get_configuration
 from jdaviz.configs.imviz.helper import Imviz
 from jdaviz.configs.imviz.plugins.viewers import ImvizImageView
+from jdaviz.configs.imviz.tests.utils import BaseImviz_WCS_NoWCS
 
 
 @pytest.mark.parametrize(
@@ -22,12 +23,24 @@ def test_create_destroy_viewer(imviz_helper, desired_name, actual_name):
     assert viewer is imviz_helper.app._viewer_store.get(actual_name), list(imviz_helper.app._viewer_store.keys())  # noqa
     assert imviz_helper.app.get_viewer_ids() == viewer_names
 
-    # Make sure plugins that store viewer_items differently are consistent.
-    assert imviz_helper.plugins['Imviz Line Profiles (XY)']._obj.viewer_items == viewer_names
+    # Make sure plugins that store viewer_items are updated.
     assert sorted(imviz_helper.plugins['Compass'].viewer.labels) == viewer_names
 
+    po = imviz_helper.plugins['Plot Options']
+    po.multiselect = True
+    po.viewer = viewer_names
+
     imviz_helper.destroy_viewer(actual_name)
     assert imviz_helper.app.get_viewer_ids() == ['imviz-0']
+    assert po.viewer.selected == ['imviz-0']
+    assert po.viewer.labels == ['imviz-0']
+
+
+def test_get_viewer_created(imviz_helper):
+    # This viewer has no reference but has ID.
+    viewer1 = imviz_helper.create_image_viewer()
+    viewer2 = imviz_helper.app.get_viewer('imviz-1')
+    assert viewer1 is viewer2
 
 
 def test_destroy_viewer_invalid(imviz_helper):
@@ -54,7 +67,7 @@ def test_destroy_viewer_with_subset(imviz_helper):
 
     # Create a Subset.
     reg = CirclePixelRegion(center=PixCoord(x=4, y=4), radius=2)
-    imviz_helper.load_regions(reg)
+    imviz_helper.plugins['Subset Tools']._obj.import_region(reg)
 
     # Remove the second viewer.
     imviz_helper.destroy_viewer('second')
@@ -82,3 +95,26 @@ def test_mastviz_config():
 
     assert im.app.get_viewer_ids() == ['mastviz-0']
     assert im.app.data_collection[0].shape == (2, 2)
+
+
+def test_zoom_center_radius_init(imviz_helper):
+    """Regression test for https://github.com/spacetelescope/jdaviz/issues/3217"""
+    arr = np.ones((10, 10))
+    imviz_helper.load_data(arr, data_label='my_array')
+    assert imviz_helper.default_viewer._obj.state.zoom_center_x > 0
+    assert imviz_helper.default_viewer._obj.state.zoom_center_y > 0
+    assert imviz_helper.default_viewer._obj.state.zoom_radius > 0
+
+
+class TestDeleteData(BaseImviz_WCS_NoWCS):
+
+    def test_plot_options_after_destroy(self):
+        self.imviz.create_image_viewer(viewer_name="imviz-1")
+        self.imviz.app.add_data_to_viewer('imviz-1', 'no_wcs[SCI,1]')
+
+        po = self.imviz.plugins['Plot Options']
+        po.open_in_tray()
+        po.viewer = "imviz-1"
+        po.stretch_function = "Square Root"
+        self.imviz.destroy_viewer("imviz-1")
+        assert len(po.layer.choices) == 2
