@@ -1,3 +1,4 @@
+import warnings
 from zipfile import ZipFile
 
 import pytest
@@ -7,7 +8,6 @@ from astropy.utils.data import download_file
 from jdaviz.utils import PRIHDR_KEY, COMMENTCARD_KEY
 
 
-@pytest.mark.filterwarnings('ignore')
 @pytest.mark.remote_data
 @pytest.mark.parametrize('instrument_arg', ('nirspec', None))
 def test_nirspec_parser(mosviz_helper, tmp_path, instrument_arg):
@@ -83,7 +83,6 @@ def test_nirspec_parser(mosviz_helper, tmp_path, instrument_arg):
                                              data_label=data_label)
 
 
-@pytest.mark.filterwarnings('ignore')
 @pytest.mark.remote_data
 def test_nirspec_level2_parser(mosviz_helper, tmp_path):
     '''
@@ -91,7 +90,8 @@ def test_nirspec_level2_parser(mosviz_helper, tmp_path):
     Also tests no instrument keyword fallback, and IntraRow linking
     '''
 
-    test_data = 'https://stsci.box.com/shared/static/mytqf082lpbfia7wlwjq6p1h5cggd9h6.zip'
+    # nirspec_level2_test_data.zip
+    test_data = 'https://stsci.box.com/shared/static/4hgfhul8el3ciu3p59aqb1m0dk3b0v8y.zip'
     fn = download_file(test_data, cache=True, timeout=100)
     with ZipFile(fn, 'r') as sample_data_zip:
         sample_data_zip.extractall(tmp_path)
@@ -105,7 +105,6 @@ def test_nirspec_level2_parser(mosviz_helper, tmp_path):
 
 
 @pytest.mark.remote_data
-@pytest.mark.filterwarnings('ignore', match="'(MJy/sr)^2' did not parse as fits unit")
 def test_niriss_parser(mosviz_helper, tmp_path):
     '''
     Tests loading a NIRISS dataset
@@ -147,7 +146,11 @@ def test_niriss_parser(mosviz_helper, tmp_path):
     imview = mosviz_helper.app.get_viewer(mosviz_helper._default_image_viewer_reference_name)
     x_pixcenter = (imview.state.x_max + imview.state.x_min)/2.0
     y_pixcenter = (imview.state.y_max + imview.state.y_min)/2.0
-    viewer_center_coord = imview.layers[0].layer.coords.pixel_to_world(x_pixcenter, y_pixcenter)
+    with warnings.catch_warnings():
+        # https://github.com/spacetelescope/gwcs/pull/522
+        warnings.filterwarnings(
+            "ignore", message="The bounding_box was set in C order.*")
+        viewer_center_coord = imview.layers[0].layer.coords.pixel_to_world(x_pixcenter, y_pixcenter)
     assert_allclose(viewer_center_coord.ra.deg, dc_tab["R.A."][0])
     assert_allclose(viewer_center_coord.dec.deg, dc_tab["Dec."][0])
 
@@ -177,14 +180,13 @@ def test_niriss_parser(mosviz_helper, tmp_path):
 
 
 @pytest.mark.remote_data
-@pytest.mark.filterwarnings('ignore', match="'(MJy/sr)^2' did not parse as fits unit")
 def test_nircam_parser(mosviz_helper, tmp_path):
     '''
     Tests loading a NIRCam dataset
     '''
 
-    # Download data
-    test_data = 'https://stsci.box.com/shared/static/itk7pav073nubwn58pig002m9796qzpw.zip'
+    # Download data (nircam_parser_test_data.zip)
+    test_data = 'https://stsci.box.com/shared/static/g3tymg7c1yj67gy0hpfreo80inqdm1ps.zip'
     fn = download_file(test_data, cache=True, timeout=100)
 
     # Extract to a known, temporary folder
@@ -216,12 +218,10 @@ def test_missing_srctype(mosviz_helper, tmp_path):
 
     This dataset was our original simulated NIRISS dataset that is missing SRCTYPE.
 
-    NOTE: Under some conditions, a warning is raised when TemporaryDirectory attempts to
-    clean itself up. Most likely a race condition between TempDir and pytest
     '''
 
-    # Download data
-    test_data = 'https://stsci.box.com/shared/static/l2azhcqd3tvzhybdlpx2j2qlutkaro3z.zip'
+    # Download data (niriss_parser_test_data.zip)
+    test_data = 'https://stsci.box.com/shared/static/7ndets0vjjsa97la2hvjvm6xvnu5b594.zip'
     fn = download_file(test_data, cache=True, timeout=30)
 
     # Extract to a known, temporary folder
