@@ -1,6 +1,7 @@
 import warnings
 
 from astropy import units as u
+from regions.core.core import Region
 from glue.core.subset_group import GroupedSubset
 from specutils import SpectralRegion, Spectrum1D
 
@@ -39,10 +40,10 @@ class Specviz(ConfigHelper, LineListMixin):
         self.app.hub.subscribe(self, RedshiftMessage,
                                handler=self._redshift_listener)
 
-    def load_spectrum(self, data, data_label=None, format=None, show_in_viewer=True,
-                      concat_by_file=False):
+    def load_data(self, data, data_label=None, format=None, show_in_viewer=True,
+                  concat_by_file=False, cache=None, local_path=None, timeout=None):
         """
-        Loads a data file or `~specutils.Spectrum1D` object into Specviz.
+        Load data into Specviz.
 
         Parameters
         ----------
@@ -59,15 +60,29 @@ class Specviz(ConfigHelper, LineListMixin):
             If True and there is more than one available extension, concatenate
             the extensions within each spectrum file passed to the parser and
             add a concatenated spectrum to the data collection.
+        cache : None, bool, or str
+            Cache the downloaded file if the data are retrieved by a query
+            to a URL or URI.
+        local_path : str, optional
+            Cache remote files to this path. This is only used if data is
+            requested from `astroquery.mast`.
+        timeout : float, optional
+            If downloading from a remote URI, set the timeout limit for
+            remote requests in seconds (passed to
+            `~astropy.utils.data.download_file` or
+            `~astroquery.mast.Conf.timeout`).
         """
         super().load_data(data,
                           parser_reference='specviz-spectrum1d-parser',
                           data_label=data_label,
                           format=format,
                           show_in_viewer=show_in_viewer,
-                          concat_by_file=concat_by_file)
+                          concat_by_file=concat_by_file,
+                          cache=cache,
+                          local_path=local_path,
+                          timeout=timeout)
 
-    def get_spectra(self, data_label=None, subset_to_apply=None, apply_slider_redshift="Warn"):
+    def get_spectra(self, data_label=None, spectral_subset=None, apply_slider_redshift="Warn"):
         """Returns the current data loaded into the main viewer
 
         """
@@ -75,37 +90,39 @@ class Specviz(ConfigHelper, LineListMixin):
         # Just to save line length
         get_data_method = self.app._jdaviz_helper.get_data
         viewer = self.app.get_viewer(self._default_spectrum_viewer_reference_name)
-        function_kwargs = {'function': getattr(viewer.state, "function")} if self.app.config == 'cubeviz' else {}  # noqa
+        all_subsets = self.app.get_subsets(object_only=True)
 
         if data_label is not None:
             spectrum = get_data_method(data_label=data_label,
-                                       subset_to_apply=subset_to_apply,
+                                       spectral_subset=spectral_subset,
                                        cls=Spectrum1D)
             spectra[data_label] = spectrum
         else:
             for layer_state in viewer.state.layers:
                 lyr = layer_state.layer
-                if subset_to_apply is not None:
-                    if lyr.label == subset_to_apply:
+                if spectral_subset is not None:
+                    if lyr.label == spectral_subset:
                         spectrum = get_data_method(data_label=lyr.data.label,
-                                                   subset_to_apply=subset_to_apply,
-                                                   cls=Spectrum1D,
-                                                   **function_kwargs)
+                                                   spectral_subset=spectral_subset,
+                                                   cls=Spectrum1D)
                         spectra[lyr.data.label] = spectrum
                     else:
                         continue
+                elif (isinstance(lyr, GroupedSubset) and lyr.label in all_subsets.keys() and
+                        isinstance(all_subsets[lyr.label][0], Region)):
+                    # spatial subsets appear as automatically extracted entries, we can skip
+                    # the layer representing the subset itself.
+                    continue
+                elif (isinstance(lyr, GroupedSubset) and lyr.label in all_subsets.keys() and
+                        isinstance(all_subsets[lyr.label], SpectralRegion)):
+                    spectrum = get_data_method(data_label=lyr.data.label,
+                                               spectral_subset=lyr.label,
+                                               cls=Spectrum1D)
+                    spectra[f'{lyr.data.label} ({lyr.label})'] = spectrum
                 else:
-                    if isinstance(lyr, GroupedSubset):
-                        spectrum = get_data_method(data_label=lyr.data.label,
-                                                   subset_to_apply=lyr.label,
-                                                   cls=Spectrum1D,
-                                                   **function_kwargs)
-                        spectra[f'{lyr.data.label} ({lyr.label})'] = spectrum
-                    else:
-                        spectrum = get_data_method(data_label=lyr.label,
-                                                   cls=Spectrum1D,
-                                                   **function_kwargs)
-                        spectra[lyr.label] = spectrum
+                    spectrum = get_data_method(data_label=lyr.label,
+                                               cls=Spectrum1D)
+                    spectra[lyr.label] = spectrum
 
         if not apply_slider_redshift:
             if data_label is not None:
@@ -128,20 +145,24 @@ class Specviz(ConfigHelper, LineListMixin):
 
             return output_spectra
 
-    def get_spectral_regions(self):
+    def get_spectral_regions(self, use_display_units=False):
         """
         A simple wrapper around the app-level call to retrieve only spectral
         subsets, which are now returned as SpectralRegions by default.
 
+        Parameters
+        ----------
+        use_display_units : bool, optional
+            Whether to convert to the display units defined in the
+            :ref:`Unit Conversion <unit-conversion>` plugin.
+
         Returns
         -------
         spec_regs : dict
             Mapping from the names of the subsets to the subsets expressed
             as `specutils.SpectralRegion` objects.
         """
-        return self.app.get_subsets_from_viewer(
-            self._default_spectrum_viewer_reference_name, subset_type="spectral"
-        )
+        return self.app.get_subsets(spectral_only=True, use_display_units=use_display_units)
 
     def x_limits(self, x_min=None, x_max=None):
         """Sets the limits of the x-axis
@@ -207,7 +228,7 @@ class Specviz(ConfigHelper, LineListMixin):
             # If user's value has a unit, convert it to the current axis' units
             elif isinstance(min_val, u.Quantity):
                 # Convert user's value to axis' units
-                min_val = min_val.to(axis.unit).value
+                min_val = min_val.to_value(axis.unit)
             # If auto, set to min axis wavelength value
             elif min_val == "auto":
                 min_val = min(axis).value
@@ -217,7 +238,7 @@ class Specviz(ConfigHelper, LineListMixin):
             # If user's value has a unit, convert it to the current axis' units
             if isinstance(max_val, u.Quantity):
                 # Convert user's value to axis' units
-                max_val = max_val.to(axis.unit).value
+                max_val = max_val.to_value(axis.unit)
             # If auto, set to max axis wavelength value
             elif max_val == "auto":
                 max_val = max(axis).value
@@ -273,19 +294,22 @@ class Specviz(ConfigHelper, LineListMixin):
             self._default_spectrum_viewer_reference_name
         ).figure.axes[axis].tick_format = fmt
 
-    def get_data(self, data_label=None, cls=None, subset_to_apply=None):
+    def get_data(self, data_label=None, spectral_subset=None, cls=None,
+                 use_display_units=False):
         """
         Returns data with name equal to data_label of type cls with subsets applied from
-        subset_to_apply.
+        spectral_subset.
 
         Parameters
         ----------
         data_label : str, optional
             Provide a label to retrieve a specific data set from data_collection.
+        spectral_subset : str, optional
+            Spectral subset applied to data.
         cls : `~specutils.Spectrum1D`, optional
             The type that data will be returned as.
-        subset_to_apply : str, optional
-            Subset that is to be applied to data before it is returned.
+        use_display_units: bool, optional
+            Whether to convert to the display units defined in the <unit-conversion> plugin.
 
         Returns
         -------
@@ -293,15 +317,5 @@ class Specviz(ConfigHelper, LineListMixin):
             Data is returned as type cls with subsets applied.
 
         """
-        if self.app.config == 'cubeviz':
-            # then this is a specviz instance inside cubeviz and we want to default to the
-            # viewer's collapse function
-            default_sp_viewer = self.app.get_viewer(self._default_spectrum_viewer_reference_name)
-            function = getattr(default_sp_viewer.state, 'function', None)
-            if cls is None:
-                cls = Spectrum1D
-        else:
-            function = None
-
-        return self._get_data(data_label=data_label, cls=cls, subset_to_apply=subset_to_apply,
-                              function=function)
+        return self._get_data(data_label=data_label, spectral_subset=spectral_subset,
+                              cls=cls, use_display_units=use_display_units)
