@@ -1,5 +1,4 @@
 import os
-import warnings
 
 import numpy as np
 from glue.core.message import (SubsetDeleteMessage,
@@ -8,28 +7,25 @@ from glue_jupyter.common.toolbar_vuetify import read_icon
 from traitlets import Bool, List, Float, Unicode, observe
 from astropy import units as u
 from specutils import analysis, Spectrum1D
-from specutils.manipulation import extract_region
 
-from jdaviz.core.custom_traitlets import FloatHandleEmpty
 from jdaviz.core.events import (AddDataMessage,
                                 RemoveDataMessage,
                                 SpectralMarksChangedMessage,
                                 LineIdentifyMessage,
-                                RedshiftMessage)
-from jdaviz.core.marks import (LineAnalysisContinuum,
-                               LineAnalysisContinuumCenter,
-                               LineAnalysisContinuumLeft,
-                               LineAnalysisContinuumRight,
-                               ShadowLine)
+                                RedshiftMessage,
+                                GlobalDisplayUnitChanged,
+                                SnackbarMessage)
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin,
                                         DatasetSelectMixin,
                                         SpectralSubsetSelectMixin,
                                         DatasetSpectralSubsetValidMixin,
-                                        SubsetSelect,
-                                        SPATIAL_DEFAULT_TEXT)
+                                        SpectralContinuumMixin,
+                                        skip_if_no_updates_since_last_active,
+                                        with_spinner)
 from jdaviz.core.user_api import PluginUserApi
 from jdaviz.core.tools import ICON_DIR
+from jdaviz.core.validunits import check_if_unit_is_per_solid_angle
 
 
 __all__ = ['LineAnalysis']
@@ -67,7 +63,7 @@ def _coerce_unit(quantity):
 
 @tray_registry('specviz-line-analysis', label="Line Analysis", viewer_requirements='spectrum')
 class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelectMixin,
-                   DatasetSpectralSubsetValidMixin):
+                   DatasetSpectralSubsetValidMixin, SpectralContinuumMixin):
     """
     The Line Analysis plugin returns specutils analysis for a single spectral line.
     See the :ref:`Line Analysis Plugin Documentation <line-analysis>` for more details.
@@ -77,37 +73,29 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
 
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
     * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
       Dataset to use for computing line statistics.
-    * ``spatial_subset`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`):
-      Select which region's collapsed spectrum to analyze or ``Entire Cube``.
     * ``spectral_subset`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`):
       Subset to use for the line, or ``Entire Spectrum``.
     * ``continuum`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`):
       Subset to use for the continuum, or ``Surrounding`` to use a region surrounding the
       subset set in ``spectral_subset``.
-    * :attr:`width`:
+    * ```continuum_width``:
       Width, relative to the overall line spectral region, to fit the linear continuum
       (excluding the region containing the line). If 1, will use endpoints within line region
       only.
-    * :meth:`show_continuum_marks`
     * :meth:`get_results`
 
     """
     dialog = Bool(False).tag(sync=True)
     template_file = __file__, "line_analysis.vue"
+    uses_active_status = Bool(True).tag(sync=True)
 
-    spatial_subset_items = List().tag(sync=True)
-    spatial_subset_selected = Unicode().tag(sync=True)
-
-    continuum_subset_items = List().tag(sync=True)
-    continuum_subset_selected = Unicode().tag(sync=True)
-
-    width = FloatHandleEmpty(3).tag(sync=True)
     results_computing = Bool(False).tag(sync=True)
     results = List().tag(sync=True)
     results_centroid = Float().tag(sync=True)  # stored in AA units
-    line_items = List([]).tag(sync=True)
+    line_menu_items = List([]).tag(sync=True)
     sync_identify = Bool(True).tag(sync=True)
     sync_identify_icon_enabled = Unicode(read_icon(os.path.join(ICON_DIR, 'line_select.svg'), 'svg+xml')).tag(sync=True)  # noqa
     sync_identify_icon_disabled = Unicode(read_icon(os.path.join(ICON_DIR, 'line_select_disabled.svg'), 'svg+xml')).tag(sync=True)  # noqa
@@ -117,33 +105,17 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
 
     def __init__(self, *args, **kwargs):
 
-        self._default_spectrum_viewer_reference_name = kwargs.get(
-            "spectrum_viewer_reference_name", "spectrum-viewer"
-        )
-
         super().__init__(**kwargs)
 
         self.update_results(None)
 
-        self.continuum = SubsetSelect(self,
-                                      'continuum_subset_items',
-                                      'continuum_subset_selected',
-                                      default_text='Surrounding',
-                                      allowed_type='spectral')
-
         # when accessing the selected data, access the spectrum-viewer version
         self.dataset._viewers = [self._default_spectrum_viewer_reference_name]
         # require entries to be in spectrum-viewer (not other cubeviz images, etc)
         self.dataset.add_filter('layer_in_spectrum_viewer')
 
-        if self.app.state.settings.get("configuration") == "cubeviz":
-            self.spatial_subset = SubsetSelect(self,
-                                               'spatial_subset_items',
-                                               'spatial_subset_selected',
-                                               default_text=SPATIAL_DEFAULT_TEXT,
-                                               allowed_type='spatial')
-        else:
-            self.spatial_subset = None
+        # continuum selection is mandatory for line-analysis
+        self._continuum_remove_none_option()
 
         self.hub.subscribe(self, AddDataMessage,
                            handler=self._on_viewer_data_changed)
@@ -158,12 +130,24 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
                            handler=self._on_plotted_lines_changed)
         self.hub.subscribe(self, LineIdentifyMessage,
                            handler=self._on_identified_line_changed)
+        self.hub.subscribe(self, GlobalDisplayUnitChanged,
+                           handler=self._on_global_display_unit_changed)
+
+    @property
+    def _default_spectrum_viewer_reference_name(self):
+        return getattr(
+            self.app._jdaviz_helper, '_default_spectrum_viewer_reference_name', 'spectrum-viewer'
+        )
 
     @property
     def user_api(self):
-        return PluginUserApi(self, expose=('dataset', 'spatial_subset', 'spectral_subset',
-                                           'continuum', 'width',
-                                           'show_continuum_marks', 'get_results'))
+        return PluginUserApi(self, expose=('dataset', 'spectral_subset',
+                                           'continuum', 'continuum_width', 'get_results'))
+
+    @property
+    def line_items(self):
+        # Return list of only the table indices ("name_rest" in line table) from line_menu_items
+        return [item["value"] for item in self.line_menu_items]
 
     def _on_viewer_data_changed(self, msg):
         viewer_id = self.app._viewer_item_by_reference(
@@ -173,13 +157,15 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
             return
 
         viewer = self.app.get_viewer(self._default_spectrum_viewer_reference_name)
-        get_data_kwargs = {'data_label': msg.data.label}
-        if self.config == 'cubeviz':
-            get_data_kwargs['function'] = getattr(viewer.state, 'function', None)
-        viewer_data = self.app._jdaviz_helper.get_data(**get_data_kwargs)
+        viewer_data_labels = [layer.layer.label for layer in viewer.layers]
+        if msg.data.label not in viewer_data_labels:
+            return
+
+        viewer_data = self.app._jdaviz_helper.get_data(data_label=msg.data.label)
 
         # If no data is currently plotted, don't attempt to update
         if viewer_data is None:
+            self.disabled_msg = 'Line Analysis unavailable without spectral data'
             return
 
         if viewer_data.spectral_axis.unit == u.pix:
@@ -200,87 +186,44 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
             The glue message passed to this callback method.
         """
         if (msg.subset.label in [self.spectral_subset_selected,
-                                 self.spatial_subset_selected,
-                                 self.continuum_subset_selected]
-                and self.plugin_opened):
-            self._calculate_statistics()
+                                 self.continuum_subset_selected]):
+            self._calculate_statistics(msg)
 
-    @observe('plugin_opened')
-    def _on_plugin_opened_changed(self, *args):
+    def _on_global_display_unit_changed(self, msg):
+        self._calculate_statistics(msg)
+
+    @observe('is_active')
+    def _is_active_changed(self, msg):
         if self.disabled_msg:
             return
-        # toggle continuum lines in spectrum viewer based on whether this plugin
-        # is currently open in the tray
-        self.show_continuum_marks(self.plugin_opened)
 
-    def show_continuum_marks(self, show=True):
-        """
-        Show (or hide) the marks indicating the continuum on the spectrum viewer.
-
-        Parameters
-        ----------
-        show : bool
-            Whether to show (or hide) the marks
-        """
-        for pos, mark in self.marks.items():
-            mark.visible = show
-        if show:
-            self._calculate_statistics(ignore_plugin_closed=True)
-
-    @property
-    def marks(self):
-        marks = {}
-        viewer = self.app.get_viewer(self._default_spectrum_viewer_reference_name)
-        for mark in viewer.figure.marks:
-            if isinstance(mark, LineAnalysisContinuum):
-                # NOTE: we don't use isinstance anymore because of nested inheritance
-                if mark.__class__.__name__ == 'LineAnalysisContinuumLeft':
-                    marks['left'] = mark
-                elif mark.__class__.__name__ == 'LineAnalysisContinuumCenter':
-                    marks['center'] = mark
-                elif mark.__class__.__name__ == 'LineAnalysisContinuumRight':
-                    marks['right'] = mark
-
-        if not len(marks):
-            if not viewer.state.reference_data:
-                # we don't have data yet for scales, defer initializing
-                return {}
-            # then haven't been initialized yet, so initialize with empty
-            # marks that will be populated once the first analysis is done.
-            marks = {'left': LineAnalysisContinuumLeft(viewer, visible=self.plugin_opened),
-                     'center': LineAnalysisContinuumCenter(viewer, visible=self.plugin_opened),
-                     'right': LineAnalysisContinuumRight(viewer, visible=self.plugin_opened)}
-            shadows = [ShadowLine(mark, shadow_width=2) for mark in marks.values()]
-            # NOTE: += won't trigger the figure to notice new marks
-            viewer.figure.marks = viewer.figure.marks + shadows + list(marks.values())
-
-        return marks
-
-    def update_results(self, results=None, mark_x={}, mark_y={}):
-        for pos, mark in self.marks.items():
-            mark.update_xy(mark_x.get(pos, []), mark_y.get(pos, []))
+        for pos, mark in self.continuum_marks.items():
+            mark.visible = self.is_active
+        self._calculate_statistics(msg)
 
+    def update_results(self, results=None):
         if results is None:
             self.results = [{'function': function, 'result': ''} for function in FUNCTIONS]
+            self._update_continuum_marks()
         else:
             self.results = results
 
-        self.results_computing = False
-
     def get_results(self):
         # user-facing API call to force updating and retrieving results, even if the plugin
         # is closed
 
         if not self.spectral_subset_valid:
             valid, spec_range, subset_range = self._check_dataset_spectral_subset_valid(return_ranges=True)  # noqa
-            raise ValueError(f"spectral subset '{self.spectral_subset.selected}' {subset_range} is outside data range of '{self.dataset.selected}' {spec_range}")  # noqa
+            raise ValueError(f"spectral subset '{self.spectral_subset.selected}' {subset_range}"
+                             f" is outside data range of '{self.dataset.selected}' {spec_range}")
 
-        self._calculate_statistics(ignore_plugin_closed=True)
+        self._calculate_statistics()
         return self.results
 
     def _on_plotted_lines_changed(self, msg):
         self.line_marks = msg.marks
-        self.line_items = msg.names_rest
+        self.line_menu_items = [{"title": f"{mark.name} {mark.rest_value} {mark.xunit}", "value": name_rest}  # noqa
+                                for mark, name_rest in zip(msg.marks, msg.names_rest)]
         if self.selected_line not in self.line_items:
             # default to identified line if available
             self.selected_line = self.identified_line
@@ -294,17 +237,23 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
             # in which case we'll default to the identified line
             self.selected_line = self.identified_line
 
-    @observe("spatial_subset_selected", "spectral_subset_selected", "dataset_selected",
-             "continuum_subset_selected", "width")
-    def _calculate_statistics(self, *args, **kwargs):
+    @observe("dataset_selected", "spectral_subset_selected",
+             "continuum_subset_selected", "continuum_width")
+    @skip_if_no_updates_since_last_active()
+    @with_spinner('results_computing')
+    def _calculate_statistics(self, msg={}):
         """
         Run the line analysis functions on the selected data/subset and
         display the results.
         """
-        if not hasattr(self, 'dataset') or self.app._jdaviz_helper is None or self.dataset_selected == '':  # noqa
+        if not hasattr(self, 'dataset') or self.app._jdaviz_helper is None:  # noqa
             # during initial init, this can trigger before the component is initialized
             return
 
+        if self.disabled_msg != '':
+            self.update_results(None)
+            return
+
         # call directly since this observe may be triggered before the spectral_subset_valid
         # traitlet is updated
         if not self._check_dataset_spectral_subset_valid():
@@ -312,110 +261,13 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
             self.update_results(None)
             return
 
-        # show spinner with overlay
-        self.results_computing = True
-
-        full_spectrum = self.dataset.selected_spectrum_for_spatial_subset(self.spatial_subset_selected)  # noqa
-
-        if (full_spectrum is None or self.width == "" or
-                (not self.plugin_opened and not kwargs.get('ignore_plugin_closed'))):
-            # this can happen DURING a unit conversion change
+        spectrum, continuum, spec_subtracted = self._get_continuum(self.dataset,
+                                                                   self.spectral_subset,
+                                                                   update_marks=True)
+        if spectrum is None:
             self.update_results(None)
             return
 
-        spectral_axis = full_spectrum.spectral_axis
-
-        if self.continuum_subset_selected == self.spectral_subset_selected:
-            # already raised a validation error in the UI
-            self.update_results(None)
-            return
-
-        sr = self.app.get_subsets_from_viewer(self._default_spectrum_viewer_reference_name,
-                                              subset_type="spectral").get(self.spectral_subset_selected) # noqa
-
-        if self.spectral_subset_selected == "Entire Spectrum":
-            spectrum = full_spectrum
-        else:
-            spectrum = extract_region(full_spectrum, sr, return_single_spectrum=True)
-
-        # compute continuum
-        if self.continuum_subset_selected == "Surrounding" and self.spectral_subset_selected == "Entire Spectrum": # noqa
-            # we know we'll just use the endpoints, so let's be efficient and not even
-            # try extracting from the region
-            continuum_mask = np.array([0, len(spectral_axis)-1])
-            mark_x = {'left': np.array([]),
-                      'center': np.array([min(spectral_axis.value), max(spectral_axis.value)]),
-                      'right': np.array([])}
-
-        elif self.continuum_subset_selected == "Surrounding":
-            # self.spectral_subset_selected != "Entire Spectrum"
-            if self.width > 10 or self.width < 1:
-                # DEV NOTE: if changing the limits, make sure to also update the form validation
-                # rules in line_analysis.vue
-                self.update_results(None)
-                return
-
-            spectral_region_width = sr.upper - sr.lower
-            # convert width from total relative width, to width per "side"
-            width = (self.width - 1) / 2
-            left, = np.where((spectral_axis < sr.lower) &
-                             (spectral_axis > sr.lower - spectral_region_width*width))
-
-            if not len(left):
-                # then no points matching the width are available outside the line region,
-                # so we'll default to the left-most point of the line region.
-                left, = np.where(spectral_axis == min(spectrum.spectral_axis))
-
-            right, = np.where((spectral_axis > sr.upper) &
-                              (spectral_axis < sr.upper + spectral_region_width*width))
-
-            if not len(right):
-                # then no points matching the width are available outside the line region,
-                # so we'll default to the right-most point of the line region.
-                right, = np.where(spectral_axis == max(spectrum.spectral_axis))
-
-            continuum_mask = np.concatenate((left, right))
-            mark_x = {'left': np.array([min(spectral_axis.value[continuum_mask]), sr.lower.value]),
-                      'center': np.array([sr.lower.value, sr.upper.value]),
-                      'right': np.array([sr.upper.value, max(spectral_axis.value[continuum_mask])])}
-
-        else:
-            # we'll access the mask of the continuum and then apply that to the spectrum.  For a
-            # spatially-collapsed spectrum in cubeviz, this will access the mask from the full
-            # cube, but still apply that to the spatially-collapsed spectrum.
-            continuum_mask = ~self._specviz_helper.get_data(
-                self.dataset.selected,
-                subset_to_apply=self.continuum_subset_selected).mask
-            spectral_axis_nanmasked = spectral_axis.value.copy()
-            spectral_axis_nanmasked[~continuum_mask] = np.nan
-            if self.spectral_subset_selected == "Entire Spectrum":
-                mark_x = {'left': spectral_axis_nanmasked,
-                          'center': spectral_axis.value,
-                          'right': []}
-            else:
-                mark_x = {'left': spectral_axis_nanmasked[spectral_axis.value < sr.lower.value],
-                          'right': spectral_axis_nanmasked[spectral_axis.value > sr.upper.value]}
-                # Center should extend (at least) across the line region between the full
-                # range defined by the continuum subset(s).
-                # OK for mark_x to be all NaNs.
-                with warnings.catch_warnings():
-                    warnings.simplefilter('ignore', category=RuntimeWarning)
-                    mark_x_min = np.nanmin(mark_x['left'])
-                    mark_x_max = np.nanmax(mark_x['right'])
-                left_min = np.nanmin([mark_x_min, sr.lower.value])
-                right_max = np.nanmax([mark_x_max, sr.upper.value])
-                mark_x['center'] = np.array([left_min, right_max])
-
-        continuum_x = spectral_axis[continuum_mask].value
-        min_x = min(spectral_axis.value)
-        continuum_y = full_spectrum.flux[continuum_mask].value
-        # DEV NOTE: could replace this with internal calls to the model fitting infrastructure
-        # to enable other model-types and to give visual feedback (by labeling the model
-        # as line_analysis:continuum, for example)
-        slope, intercept = np.polyfit(continuum_x-min_x, continuum_y, deg=1)
-        continuum = slope * (spectrum.spectral_axis.value-min_x) + intercept
-        mark_y = {k: slope * (v-min_x) + intercept for k, v in mark_x.items()}
-
         def _uncertainty(result):
             if getattr(result, 'uncertainty', None) is not None:
                 # we'll keep the uncertainty and result in the same unit (so
@@ -427,7 +279,6 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
                 return ''
 
         temp_results = []
-        spec_subtracted = spectrum - continuum
 
         if spec_subtracted.mask is not None:
             # temporary fix while mask may contain None:
@@ -438,10 +289,26 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
             # don't need these if statements
             if function == "Line Flux":
                 flux_unit = spec_subtracted.flux.unit
+                if flux_unit == u.dimensionless_unscaled:
+                    add_flux = True
+                    flux_unit = u.Unit(self.spectrum_viewer.state.y_display_unit)
+                else:
+                    add_flux = False
+
+                solid_angle_in_flux_unit = check_if_unit_is_per_solid_angle(flux_unit,
+                                                                            return_unit=True)
+                if solid_angle_in_flux_unit is None:
+                    # use dimensionless_unscaled as a placeholder unit.
+                    # is_equivalent() checks won't pass anyway if theres no
+                    # solid angle in the unit, so it won't matter what this is
+                    solid_angle_in_flux_unit = u.dimensionless_unscaled
+
+                solid_angle_string = solid_angle_in_flux_unit.to_string()
+
                 # If the flux unit is equivalent to Jy, or Jy per spaxel for Cubeviz,
                 # enforce integration in frequency space
                 if (flux_unit.is_equivalent(u.Jy) or
-                        flux_unit.is_equivalent(u.Jy/u.sr)):
+                   flux_unit.is_equivalent(u.Jy / solid_angle_in_flux_unit)):
                     # Perform integration in frequency space
                     freq_spec = Spectrum1D(
                         spectral_axis=spec_subtracted.spectral_axis.to(u.Hz,
@@ -449,11 +316,24 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
                         flux=spec_subtracted.flux,
                         uncertainty=spec_subtracted.uncertainty)
 
-                    raw_result = analysis.line_flux(freq_spec)
-                    # When flux is equivalent to Jy, lineflux result should be shown in W/m2
-                    if flux_unit.is_equivalent(u.Jy/u.sr):
+                    try:
+                        if add_flux:
+                            raw_result = analysis.line_flux(freq_spec) * flux_unit
+                        else:
+                            raw_result = analysis.line_flux(freq_spec)
+                    except ValueError as e:
+                        # can happen if interpolation out-of-bounds or any error from specutils
+                        # let's avoid the whole app crashing and instead expose the error to the
+                        # user
+                        self.hub.broadcast(SnackbarMessage(
+                            f"failed to calculate line analysis statistics: {e}", sender=self,
+                            color="warning"))
+                        self.update_results(None)
+                        return
 
-                        final_unit = u.Unit('W/(m2 sr)')
+                    # When flux is equivalent to Jy, lineflux result should be shown in W/m2
+                    if flux_unit.is_equivalent(u.Jy/solid_angle_in_flux_unit):
+                        final_unit = u.Unit(f'W/(m2 {solid_angle_string})')
                     else:
                         final_unit = u.Unit('W/m2')
 
@@ -464,17 +344,30 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
                 # If the flux unit is instead equivalent to power density
                 # (Jy, but defined in wavelength), enforce integration in wavelength space
                 elif (flux_unit.is_equivalent(u.Unit('W/(m2 m)')) or
-                        flux_unit.is_equivalent(u.Unit('W/(m2 m sr)'))):
+                        flux_unit.is_equivalent(u.Unit(f'W/(m2 m {solid_angle_string})'))):
                     # Perform integration in wavelength space using MKS unit (meters)
                     wave_spec = Spectrum1D(
                         spectral_axis=spec_subtracted.spectral_axis.to(u.m,
                                                                        equivalencies=u.spectral()),
                         flux=spec_subtracted.flux,
                         uncertainty=spec_subtracted.uncertainty)
-                    raw_result = analysis.line_flux(wave_spec)
+                    try:
+                        if add_flux:
+                            raw_result = analysis.line_flux(wave_spec) * flux_unit
+                        else:
+                            raw_result = raw_result = analysis.line_flux(wave_spec)
+                    except ValueError as e:
+                        # can happen if interpolation out-of-bounds or any error from specutils
+                        # let's avoid the whole app crashing and instead expose the error to the
+                        # user
+                        self.hub.broadcast(SnackbarMessage(
+                            f"failed to calculate line analysis statistics: {e}", sender=self,
+                            color="warning"))
+                        self.update_results(None)
+                        return
                     # When flux is equivalent to Jy, lineflux result should be shown in W/m2
-                    if flux_unit.is_equivalent(u.Unit('W/(m2 m)'/u.sr)):
-                        final_unit = u.Unit('W/(m2 sr)')
+                    if flux_unit.is_equivalent(u.Unit('W/(m2 m)'/solid_angle_in_flux_unit)):
+                        final_unit = u.Unit(f'W/(m2 {solid_angle_string})')
                     else:
                         final_unit = u.Unit('W/m2')
 
@@ -506,7 +399,15 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
                 temp_result = FUNCTIONS[function](spec_subtracted, region=None)
                 self.results_centroid = temp_result.to_value(u.AA, equivalencies=u.spectral())
             else:
-                temp_result = FUNCTIONS[function](spec_subtracted)
+                # if the minimum flux is negative, translate the spectrum until it is
+                # non-negative for the these line analysis functions:
+                if spec_subtracted.flux.min() < 0:
+                    spec_subtracted_nonneg_flux = (
+                            spec_subtracted - np.min((spectrum - continuum).flux)
+                    )
+                else:
+                    spec_subtracted_nonneg_flux = spec_subtracted
+                temp_result = FUNCTIONS[function](spec_subtracted_nonneg_flux)
 
             temp_result = _coerce_unit(temp_result)
             temp_results.append({'function': function,
@@ -518,13 +419,13 @@ class LineAnalysis(PluginTemplateMixin, DatasetSelectMixin, SpectralSubsetSelect
             # default to the identified line
             self.selected_line = self.identified_line
 
-        self.update_results(temp_results, mark_x, mark_y)
+        self.update_results(temp_results)
 
     def _compute_redshift_for_selected_line(self):
         index = self.line_items.index(self.selected_line)
         line_mark = self.line_marks[index]
-        rest_value = (line_mark.rest_value * line_mark._x_unit).to_value(u.AA,
-                                                                         equivalencies=u.spectral())
+        rest_value = (line_mark.rest_value * line_mark.xunit).to_value(u.AA,
+                                                                       equivalencies=u.spectral())
         return (self.results_centroid - rest_value) / rest_value
 
     @observe('sync_identify')
