@@ -1,23 +1,24 @@
-import numpy as np
 import pytest
+import numpy as np
 from astropy import units as u
 from astropy.table import QTable
-from glue.core.roi import XRangeROI
-from glue.core.edit_subset_mode import NewMode
+from astropy.tests.helper import assert_quantity_allclose
+from numpy.testing import assert_allclose
 from regions import RectanglePixelRegion, PixCoord
-from specutils import Spectrum1D
+from specutils import Spectrum1D, SpectralRegion
 
 from jdaviz.configs.specviz.plugins.line_analysis.line_analysis import _coerce_unit
+from jdaviz.core.custom_units import PIX2
 from jdaviz.core.events import LineIdentifyMessage
 from jdaviz.core.marks import LineAnalysisContinuum
 
 
 def test_plugin(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -25,12 +26,14 @@ def test_plugin(specviz_helper, spectrum1d):
     assert len(continuum_marks) == 3
     assert np.all([cm.visible for cm in continuum_marks])
 
-    # closing tray/plugin should hide the continuum
-    specviz_helper.app.state.drawer = False
+    # disabling keep_active should hide the continuum
+    plugin.keep_active = False
     assert np.all([cm.visible is False for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6500, 7400))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             7400 * unit))
     specviz_helper.app.state.drawer = True
 
     assert 'Subset 1' in plugin.spectral_subset.labels
@@ -55,17 +58,20 @@ def test_spatial_subset(cubeviz_helper, image_cube_hdu_obj):
     cubeviz_helper.load_data(image_cube_hdu_obj, data_label="Test Cube")
 
     # add a spatial region
-    cubeviz_helper.load_regions(RectanglePixelRegion(center=PixCoord(x=3, y=5), width=4, height=6))
+    cubeviz_helper.plugins['Subset Tools']._obj.import_region(
+        RectanglePixelRegion(center=PixCoord(x=3, y=5), width=4, height=6))
 
     # create a spectral region
-    spectrum_viewer = cubeviz_helper.app.get_viewer('spectrum-viewer')
-    spectrum_viewer.apply_roi(XRangeROI(3.623e-7, 3.627e-7))  # meters
+    unit = u.Unit(cubeviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    cubeviz_helper.plugins['Subset Tools']._obj.combination_mode.selected = 'new'
+    cubeviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(3.623e-7 * unit,
+                                                                             3.627e-7 * unit))
     cubeviz_helper.app.state.drawer = True
 
     plugin = cubeviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
-    plugin.spatial_subset_selected = 'Subset 1'
+    plugin.dataset_selected = 'Spectrum (Subset 1, sum)'
     plugin.spectral_subset_selected = 'Subset 2'
     plugin.continuum_subset_selected = 'Surrounding'
     plugin.width = 1
@@ -77,18 +83,61 @@ def test_spatial_subset(cubeviz_helper, image_cube_hdu_obj):
         assert u.Unit(result['unit']) != u.dimensionless_unscaled
 
 
+@pytest.mark.parametrize('sq_angle_unit', [u.sr, PIX2])
+def test_cubeviz_units(cubeviz_helper, spectrum1d_cube_custom_fluxunit,
+                       sq_angle_unit):
+    """
+    Tests that the plugin produces the correct result when loaded cube
+    is in flux/pix2 and flux/sr, and that the results remain consistant
+    between translations of the spectral y axis flux<>surface brightness.
+    """
+    cube = spectrum1d_cube_custom_fluxunit(fluxunit=u.Jy / sq_angle_unit,
+                                           shape=(25, 25, 4), with_uncerts=True)
+    cubeviz_helper.load_data(cube, data_label="Test Cube")
+
+    uc = cubeviz_helper.plugins['Unit Conversion']
+    assert uc.spectral_y_type == 'Flux'  # initial selection should be Flux
+
+    plugin = cubeviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
+    plugin.keep_active = True
+
+    plugin.dataset_selected = 'Spectrum (sum)'
+    plugin.continuum_subset_selected = 'Surrounding'
+
+    results = plugin.results
+    assert results[0]['unit'] == 'W / m2'
+
+    # now change to surface brightness
+    uc.spectral_y_type = 'Surface Brightness'
+
+    results = plugin.results
+    line_flux_before_unit_conversion = results[0]
+    # convert back and forth between unit<>str for string format consistency
+    unit_str = u.Unit(f'W / {sq_angle_unit.to_string()} m2').to_string()
+    assert line_flux_before_unit_conversion['unit'] == unit_str
+
+    # change flux unit and make sure result stays the same after conversion
+    uc.flux_unit.selected = 'MJy'
+    results = plugin.results
+    np.testing.assert_allclose(float(results[0]['result']),
+                               float(line_flux_before_unit_conversion['result']))
+    np.testing.assert_allclose(float(results[0]['uncertainty']),
+                               float(line_flux_before_unit_conversion['uncertainty']))
+
+
 def test_user_api(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
-    sv = specviz_helper.app.get_viewer('spectrum-viewer')
-    sv.apply_roi(XRangeROI(6500, 7400))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             7400 * unit))
 
     la = specviz_helper.plugins['Line Analysis']
-    la.open_in_tray()
+    la.keep_active = True
 
-    # spectral subset does not support multiselect
-    assert "multiselect" not in la.spectral_subset.__repr__()
+    # spectral subset now supports multiselect
+    assert "multiselect" in la.spectral_subset.__repr__()
     with pytest.raises(ValueError):
         la.spectral_subset.select_all()
     with pytest.raises(ValueError):
@@ -109,7 +158,7 @@ def test_user_api(specviz_helper, spectrum1d):
 
 def test_line_identify(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     lt = QTable()
     lt['linename'] = ['O III', 'Halpha']
@@ -119,6 +168,7 @@ def test_line_identify(specviz_helper, spectrum1d):
 
     ll_plugin = specviz_helper.app.get_tray_item_from_name('g-line-list')
     la_plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
+    la_plugin.plugin_opened = True
     rest_names = [line['name_rest'] for line in ll_plugin.list_contents['Test List']['lines']]
 
     # will default to no selection
@@ -130,7 +180,7 @@ def test_line_identify(specviz_helper, spectrum1d):
     # but selecting a line from line-list (or clicking) should change the dropdown value
     # since sync is enabled by default
     assert la_plugin.sync_identify is True
-    # think this is the problem and we need to send the rest name here!
+
     msg = LineIdentifyMessage(rest_names[1],
                               sender=specviz_helper)
     specviz_helper.app.session.hub.broadcast(msg)
@@ -162,17 +212,21 @@ def test_line_identify(specviz_helper, spectrum1d):
 
     # manually update redshift
     la_plugin.vue_line_assign()
-    assert la_plugin.selected_line_redshift == -1.0
+    assert_allclose(la_plugin.results_centroid, 7307.4232674401555)
+    line_mark = la_plugin.line_marks[la_plugin.line_items.index(la_plugin.selected_line)]
+    assert_allclose(line_mark.rest_value, 5007)
+    z = la_plugin._compute_redshift_for_selected_line()
+    assert_allclose(z, (la_plugin.results_centroid - line_mark.rest_value)/line_mark.rest_value)
+    assert_allclose(la_plugin.selected_line_redshift, z)
 
 
 def test_coerce_unit():
     q_input = 1 * u.Unit('1E-20 erg m / (Angstrom cm**2 s)')
     q_input.uncertainty = 0.1 * u.Unit('1E-20 erg m / (Angstrom cm**2 s)')
+    q_unit = u.Unit('erg / (cm**2 s)')
     q_coerced = _coerce_unit(q_input)
-    assert q_coerced.unit == u.Unit('erg / (cm**2 s)')
-    assert np.allclose(q_coerced.value, 1e-20 * u.m.to(u.Angstrom))
-    assert q_coerced.uncertainty.unit == u.Unit('erg / (cm**2 s)')
-    assert np.allclose(q_coerced.uncertainty.value, 0.1 * 1e-20 * u.m.to(u.Angstrom))
+    assert_quantity_allclose(q_coerced, 1e-10 * q_unit)
+    assert_quantity_allclose(q_coerced.uncertainty, 1e-11 * q_unit)
     q_input.uncertainty = None
     q_coerced = _coerce_unit(q_input)
     assert not hasattr(q_coerced, 'uncertainty')
@@ -180,10 +234,10 @@ def test_coerce_unit():
 
 def test_continuum_surrounding_spectral_subset(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -192,7 +246,9 @@ def test_continuum_surrounding_spectral_subset(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6500, 7400))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             7400 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
@@ -207,10 +263,10 @@ def test_continuum_surrounding_spectral_subset(specviz_helper, spectrum1d):
 
 def test_continuum_spectral_same_value(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -219,7 +275,9 @@ def test_continuum_spectral_same_value(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6500, 7400))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             7400 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
@@ -234,10 +292,10 @@ def test_continuum_spectral_same_value(specviz_helper, spectrum1d):
 
 def test_continuum_surrounding_invalid_width(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -246,23 +304,25 @@ def test_continuum_surrounding_invalid_width(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6500, 7400))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             7400 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
     assert 'Subset 1' in plugin.spectral_subset.labels
     plugin.continuum_subset_selected = 'Surrounding'
     plugin.spectral_subset_selected = 'Subset 1'
-    plugin.width = 11
+    plugin.continuum_width = 11
     assert plugin.results[0]['result'] == ''
 
 
 def test_continuum_subset_spectral_entire(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -271,7 +331,9 @@ def test_continuum_subset_spectral_entire(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6500, 7400))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             7400 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
@@ -286,10 +348,10 @@ def test_continuum_subset_spectral_entire(specviz_helper, spectrum1d):
 
 def test_continuum_subset_spectral_subset2(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -298,12 +360,14 @@ def test_continuum_subset_spectral_subset2(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6200, 7000))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6200 * unit,
+                                                                             7000 * unit))
     specviz_helper.app.state.drawer = True
 
-    sv.session.edit_subset_mode._mode = NewMode
-    sv.session.edit_subset = []
-    sv.apply_roi(XRangeROI(7100, 7700))
+    specviz_helper.plugins['Subset Tools']._obj.combination_mode.selected = 'new'
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(7100 * unit,
+                                                                             7700 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
@@ -319,10 +383,10 @@ def test_continuum_subset_spectral_subset2(specviz_helper, spectrum1d):
 
 def test_continuum_surrounding_no_right(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -331,7 +395,9 @@ def test_continuum_surrounding_no_right(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6500, 8000))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             8000 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
@@ -347,10 +413,10 @@ def test_continuum_surrounding_no_right(specviz_helper, spectrum1d):
 
 def test_continuum_surrounding_no_left(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -359,7 +425,9 @@ def test_continuum_surrounding_no_left(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6000, 7500))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6000 * unit,
+                                                                             7500 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
@@ -375,10 +443,10 @@ def test_continuum_surrounding_no_left(specviz_helper, spectrum1d):
 
 def test_subset_changed(specviz_helper, spectrum1d):
     label = "Test 1D Spectrum"
-    specviz_helper.load_spectrum(spectrum1d, data_label=label)
+    specviz_helper.load_data(spectrum1d, data_label=label)
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
-    plugin.open_in_tray()
+    plugin.keep_active = True
 
     # continuum should be created, plotted, and visible
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
@@ -387,7 +455,9 @@ def test_subset_changed(specviz_helper, spectrum1d):
     assert np.all([cm.visible for cm in continuum_marks])
 
     # add a region and rerun stats for that region
-    sv.apply_roi(XRangeROI(6000, 7500))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6000 * unit,
+                                                                             7500 * unit))
     specviz_helper.app.state.drawer = True
 
     plugin = specviz_helper.app.get_tray_item_from_name('specviz-line-analysis')
@@ -397,7 +467,8 @@ def test_subset_changed(specviz_helper, spectrum1d):
     plugin.continuum_subset_selected = 'Surrounding'
     plugin.width = 3
 
-    sv.apply_roi(XRangeROI(6500, 7500))
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(6500 * unit,
+                                                                             7500 * unit))
     specviz_helper.app.state.drawer = True
 
     # Values have not yet been validated
@@ -406,17 +477,19 @@ def test_subset_changed(specviz_helper, spectrum1d):
 
 def test_invalid_subset(specviz_helper, spectrum1d):
     # 6000-8000
-    specviz_helper.load_spectrum(spectrum1d, data_label="right_spectrum")
+    specviz_helper.load_data(spectrum1d, data_label="right_spectrum")
 
     # 5000-7000
     sp2 = Spectrum1D(spectral_axis=spectrum1d.spectral_axis - 1000*spectrum1d.spectral_axis.unit,
                      flux=spectrum1d.flux * 1.25)
-    specviz_helper.load_spectrum(sp2, data_label="left_spectrum")
+    specviz_helper.load_data(sp2, data_label="left_spectrum")
 
     # apply subset that overlaps on left_spectrum, but not right_spectrum
     # NOTE: using a subset that overlaps the right_spectrum (reference) results in errors when
     # retrieving the subset (https://github.com/spacetelescope/jdaviz/issues/1868)
-    specviz_helper.app.get_viewer('spectrum-viewer').apply_roi(XRangeROI(5000, 6000))
+    unit = u.Unit(specviz_helper.plugins['Unit Conversion'].spectral_unit.selected)
+    specviz_helper.plugins['Subset Tools']._obj.import_region(SpectralRegion(5000 * unit,
+                                                                             6000 * unit))
 
     plugin = specviz_helper.plugins['Line Analysis']
     plugin.dataset = 'right_spectrum'
@@ -427,7 +500,7 @@ def test_invalid_subset(specviz_helper, spectrum1d):
     plugin.spectral_subset = 'Subset 1'
     assert not plugin._obj.spectral_subset_valid
 
-    with pytest.raises(ValueError, match=r"spectral subset 'Subset 1' \(5000.0, 5888.888888888889\) is outside data range of 'right_spectrum' \(6000.0, 8000.0\)"):  # noqa
+    with pytest.raises(ValueError, match=r"spectral subset 'Subset 1' \(5000.0, 6000.0\) is outside data range of 'right_spectrum' \(6000.0, 8000.0\)"):  # noqa
         plugin.get_results()
 
     plugin.dataset = 'left_spectrum'
