@@ -8,14 +8,16 @@ from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin,
                                         SelectPluginComponent,
                                         DatasetSelect,
-                                        AddResults)
+                                        AddResults,
+                                        skip_if_no_updates_since_last_active,
+                                        skip_if_not_tray_instance,
+                                        with_spinner)
 from jdaviz.core.user_api import PluginUserApi
 from jdaviz.core.custom_traitlets import IntHandleEmpty, FloatHandleEmpty
 from jdaviz.core.marks import PluginLine
 
 from astropy.modeling import models
 from astropy.nddata import StdDevUncertainty, VarianceUncertainty, UnknownUncertainty
-from astropy import units
 from specreduce import tracing
 from specreduce import background
 from specreduce import extract
@@ -42,6 +44,7 @@ class SpectralExtraction(PluginTemplateMixin):
 
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
     * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.close_in_tray`
     * :attr:`interactive_extract`
     * ``trace_dataset`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
       controls the input dataset for generating the trace.
@@ -89,6 +92,17 @@ class SpectralExtraction(PluginTemplateMixin):
     * ``ext_type`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`)
     * :attr:`ext_width` :
       full width of the extraction window.
+    * ``horne_ext_profile`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
+      For Horne extract, choice of 'Gaussian' or 'Self (interpolated)' to use
+      empirical profile from data.
+    * :attr:`self_prof_n_bins` :
+      Number of bins to use when computing the self-derived profile for Horne Extract.
+    * :attr:`self_prof_interp_degree_x` :
+      Interpolation degree (in X) to use when computing the self-derived profile
+      for Horne Extract.
+    * :attr:`self_prof_interp_degree_y` :
+      Interpolation degree (in Y) to use when computing the self-derived profile
+      for Horne Extract.
     * ``ext_add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
     * :meth:`import_extract`
     * :meth:`export_extract`
@@ -96,6 +110,7 @@ class SpectralExtraction(PluginTemplateMixin):
     """
     dialog = Bool(False).tag(sync=True)
     template_file = __file__, "spectral_extraction.vue"
+    uses_active_status = Bool(True).tag(sync=True)
 
     active_step = Unicode().tag(sync=True)
     interactive_extract = Bool(True).tag(sync=True)
@@ -129,6 +144,7 @@ class SpectralExtraction(PluginTemplateMixin):
     trace_results_label_overwrite = Bool().tag(sync=True)
     trace_add_to_viewer_items = List().tag(sync=True)
     trace_add_to_viewer_selected = Unicode().tag(sync=True)
+    trace_spinner = Bool(False).tag(sync=True)
 
     # BACKGROUND
     bg_dataset_items = List().tag(sync=True)
@@ -155,6 +171,7 @@ class SpectralExtraction(PluginTemplateMixin):
     bg_results_label_overwrite = Bool().tag(sync=True)
     bg_add_to_viewer_items = List().tag(sync=True)
     bg_add_to_viewer_selected = Unicode().tag(sync=True)
+    bg_img_spinner = Bool(False).tag(sync=True)
 
     bg_spec_results_label = Unicode().tag(sync=True)
     bg_spec_results_label_default = Unicode().tag(sync=True)
@@ -163,6 +180,7 @@ class SpectralExtraction(PluginTemplateMixin):
     bg_spec_results_label_overwrite = Bool().tag(sync=True)
     bg_spec_add_to_viewer_items = List().tag(sync=True)
     bg_spec_add_to_viewer_selected = Unicode().tag(sync=True)
+    bg_spec_spinner = Bool(False).tag(sync=True)
 
     bg_sub_results_label = Unicode().tag(sync=True)
     bg_sub_results_label_default = Unicode().tag(sync=True)
@@ -171,6 +189,7 @@ class SpectralExtraction(PluginTemplateMixin):
     bg_sub_results_label_overwrite = Bool().tag(sync=True)
     bg_sub_add_to_viewer_items = List().tag(sync=True)
     bg_sub_add_to_viewer_selected = Unicode().tag(sync=True)
+    bg_sub_spinner = Bool(False).tag(sync=True)
 
     # EXTRACT
     ext_dataset_items = List().tag(sync=True)
@@ -182,6 +201,13 @@ class SpectralExtraction(PluginTemplateMixin):
     ext_type_items = List().tag(sync=True)
     ext_type_selected = Unicode().tag(sync=True)
 
+    horne_ext_profile_items = List().tag(sync=True)
+    horne_ext_profile_selected = Unicode().tag(sync=True)
+
+    self_prof_n_bins = IntHandleEmpty(10).tag(sync=True)
+    self_prof_interp_degree_x = IntHandleEmpty(1).tag(sync=True)
+    self_prof_interp_degree_y = IntHandleEmpty(1).tag(sync=True)
+
     ext_width = FloatHandleEmpty(0).tag(sync=True)
 
     ext_uncert_warn = Bool(False).tag(sync=True)
@@ -194,18 +220,12 @@ class SpectralExtraction(PluginTemplateMixin):
     ext_results_label_overwrite = Bool().tag(sync=True)
     ext_add_to_viewer_items = List().tag(sync=True)
     ext_add_to_viewer_selected = Unicode().tag(sync=True)
+    # uses default "spinner"
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._default_spectrum_viewer_reference_name = kwargs.get(
-            "spectrum_viewer_reference_name", "spectrum-viewer"
-        )
-        self._default_spectrum_2d_viewer_reference_name = kwargs.get(
-            "spectrum_2d_viewer_reference_name", "spectrum-2d-viewer"
-        )
 
         self._marks = {}
-        self._do_marks = kwargs.get('interactive', True)
 
         # TRACE
         self.trace_trace = DatasetSelect(self,
@@ -311,6 +331,11 @@ class SpectralExtraction(PluginTemplateMixin):
                                               selected='ext_type_selected',
                                               manual_options=['Boxcar', 'Horne'])
 
+        self.horne_ext_profile = SelectPluginComponent(self,
+                                                       items='horne_ext_profile_items',
+                                                       selected='horne_ext_profile_selected',
+                                                       manual_options=['Gaussian', 'Self (interpolated)'])  # noqa
+
         self.ext_add_results = AddResults(self, 'ext_results_label',
                                           'ext_results_label_default',
                                           'ext_results_label_auto',
@@ -323,6 +348,14 @@ class SpectralExtraction(PluginTemplateMixin):
         self.ext_add_results.label_whitelist_overwrite = ['Spectrum 1D']
         self.ext_results_label_default = 'Spectrum 1D'
 
+    @property
+    def _default_spectrum_viewer_reference_name(self):
+        return self.app._jdaviz_helper._default_spectrum_viewer_reference_name
+
+    @property
+    def _default_spectrum_2d_viewer_reference_name(self):
+        return self.app._jdaviz_helper._default_spectrum_2d_viewer_reference_name
+
     @property
     def user_api(self):
         return PluginUserApi(self, expose=('interactive_extract',
@@ -340,6 +373,10 @@ class SpectralExtraction(PluginTemplateMixin):
                                            'export_bg', 'export_bg_img', 'export_bg_sub',
                                            'ext_dataset', 'ext_trace', 'ext_type',
                                            'ext_width', 'ext_add_results',
+                                           'horne_ext_profile',
+                                           'self_prof_n_bins',
+                                           'self_prof_interp_degree_x',
+                                           'self_prof_interp_degree_y',
                                            'import_extract',
                                            'export_extract', 'export_extract_spectrum'))
 
@@ -397,7 +434,6 @@ class SpectralExtraction(PluginTemplateMixin):
             Step in the extraction process to visualize.  Must be one of: 'trace', 'bg', 'ext'.
         """
         if step is not None:
-            self.plugin_opened = True
             if step == 'trace':
                 self._interaction_in_trace_step()
             elif step == 'bg':
@@ -407,44 +443,26 @@ class SpectralExtraction(PluginTemplateMixin):
             else:
                 raise ValueError("step must be one of: trace, bg, ext")
 
-    def clear_marks(self):
-        """
-        Manually clear the live-preview marks.
-        """
-        self.plugin_opened = False
-
-    @observe('plugin_opened', 'interactive_extract')
-    def _update_plugin_marks(self, *args):
-        if not self._do_marks:
+    @observe('is_active', 'active_step')
+    @skip_if_not_tray_instance()
+    def _update_plugin_marks(self, msg={}):
+        if self.app._jdaviz_helper is None:
+            return
+        if not len(self._marks):
+            # plugin has never been opened, no need to create marks just to hide them,
+            # we'll create marks when the plugin is first opened
             return
 
-        if not self.plugin_opened:
+        if not (self.is_active):
             for step, mark in self.marks.items():
                 mark.visible = False
             return
 
         if self.active_step == '':
             # on load, default to 'extract' (this will then trigger the observe to update the marks)
-            self._interaction_in_ext_step()
+            self.active_step = 'ext'
             return
 
-        # update extracted 1d spectrum preview, regardless of the step
-        if self.interactive_extract:
-            try:
-                sp1d = self.export_extract_spectrum(add_data=False)
-            except Exception as e:
-                # NOTE: ignore error, but will be raised when clicking ANY of the export buttons
-                # NOTE: FitTrace or manual background are often giving a
-                # "background regions overlapped" error from specreduce
-                self.ext_specreduce_err = repr(e)
-                self.marks['extract'].clear()
-            else:
-                self.ext_specreduce_err = ''
-                self.marks['extract'].update_xy(sp1d.spectral_axis.value,
-                                                sp1d.flux.value)
-        else:
-            self.marks['extract'].clear()
-
         display_marks = {'trace': ['trace', 'extract'],
                          'bg': ['trace',
                                 'bg1_center', 'bg1_lower', 'bg1_upper',
@@ -465,8 +483,9 @@ class SpectralExtraction(PluginTemplateMixin):
             # TODO: replace with cache property?
             return self._marks
 
-        if not self._do_marks:
+        if not self._tray_instance:
             return {}
+
         viewer2d = self.app.get_viewer(self._default_spectrum_2d_viewer_reference_name)
         viewer1d = self.app.get_viewer(self._default_spectrum_viewer_reference_name)
 
@@ -476,22 +495,22 @@ class SpectralExtraction(PluginTemplateMixin):
 
         # the marks haven't been initialized yet, so initialize with empty
         # marks that will be populated once the first analysis is done.
-        self._marks = {'trace': PluginLine(viewer2d, visible=self.plugin_opened),
-                       'ext_lower': PluginLine(viewer2d, visible=self.plugin_opened),
-                       'ext_upper': PluginLine(viewer2d, visible=self.plugin_opened),
-                       'bg1_center': PluginLine(viewer2d, visible=self.plugin_opened,
+        self._marks = {'trace': PluginLine(viewer2d, visible=self.is_active),
+                       'ext_lower': PluginLine(viewer2d, visible=self.is_active),
+                       'ext_upper': PluginLine(viewer2d, visible=self.is_active),
+                       'bg1_center': PluginLine(viewer2d, visible=self.is_active,
                                                 line_style='dotted'),
-                       'bg1_lower': PluginLine(viewer2d, visible=self.plugin_opened),
-                       'bg1_upper': PluginLine(viewer2d, visible=self.plugin_opened),
-                       'bg2_center': PluginLine(viewer2d, visible=self.plugin_opened,
+                       'bg1_lower': PluginLine(viewer2d, visible=self.is_active),
+                       'bg1_upper': PluginLine(viewer2d, visible=self.is_active),
+                       'bg2_center': PluginLine(viewer2d, visible=self.is_active,
                                                 line_style='dotted'),
-                       'bg2_lower': PluginLine(viewer2d, visible=self.plugin_opened),
-                       'bg2_upper': PluginLine(viewer2d, visible=self.plugin_opened)}
+                       'bg2_lower': PluginLine(viewer2d, visible=self.is_active),
+                       'bg2_upper': PluginLine(viewer2d, visible=self.is_active)}
         # NOTE: += won't trigger the figure to notice new marks
         viewer2d.figure.marks = viewer2d.figure.marks + list(self._marks.values())
 
-        self._marks['extract'] = PluginLine(viewer1d, visible=self.plugin_opened)
-        self._marks['bg_spec'] = PluginLine(viewer1d, visible=self.plugin_opened, stroke_width=1)  # noqa
+        self._marks['extract'] = PluginLine(viewer1d, visible=self.is_active)
+        self._marks['bg_spec'] = PluginLine(viewer1d, visible=self.is_active, stroke_width=1)  # noqa
 
         # NOTE: += won't trigger the figure to notice new marks
         viewer1d.figure.marks = viewer1d.figure.marks + [self._marks['extract'],
@@ -499,14 +518,46 @@ class SpectralExtraction(PluginTemplateMixin):
 
         return self._marks
 
-    @observe('trace_dataset_selected', 'trace_type_selected',
+    @observe('interactive_extract')
+    @skip_if_no_updates_since_last_active()
+    @skip_if_not_tray_instance()
+    def _update_interactive_extract(self, event={}):
+        # also called by any of the _interaction_in_*_step
+        if self.interactive_extract:
+            try:
+                sp1d = self.export_extract_spectrum(add_data=False)
+            except Exception as e:
+                # NOTE: ignore error, but will be raised when clicking ANY of the export buttons
+                # NOTE: FitTrace or manual background are often giving a
+                # "background regions overlapped" error from specreduce
+                self.ext_specreduce_err = repr(e)
+                self.marks['extract'].clear()
+            else:
+                self.ext_specreduce_err = ''
+                self.marks['extract'].update_xy(sp1d.spectral_axis.value,
+                                                sp1d.flux.value)
+        else:
+            self.marks['extract'].clear()
+
+        if self.interactive_extract and self.active_step == 'bg':
+            try:
+                spec = self.export_bg_spectrum()
+            except Exception:
+                self.marks['bg_spec'].clear()
+            else:
+                self.marks['bg_spec'].update_xy(spec.spectral_axis, spec.flux)
+        else:
+            self.marks['bg_spec'].clear()
+
+    @observe('is_active', 'trace_dataset_selected', 'trace_type_selected',
              'trace_trace_selected', 'trace_offset', 'trace_order',
              'trace_pixel', 'trace_peak_method_selected',
              'trace_do_binning', 'trace_bins', 'trace_window', 'active_step')
+    @skip_if_not_tray_instance()
+    @skip_if_no_updates_since_last_active()
     def _interaction_in_trace_step(self, event={}):
-        if not self.plugin_opened or not self._do_marks:
-            return
-        if event.get('name', '') == 'active_step' and event.get('new') != 'trace':
+        if ((event.get('name', '') in ('active_step', 'is_active') and self.active_step != 'trace')
+                or not self.is_active):
             return
 
         try:
@@ -518,16 +569,19 @@ class SpectralExtraction(PluginTemplateMixin):
             self.marks['trace'].update_xy(range(len(trace.trace)),
                                           trace.trace)
             self.marks['trace'].line_style = 'solid'
-        self.active_step = 'trace'
-        self._update_plugin_marks()
 
-    @observe('bg_dataset_selected', 'bg_type_selected',
+        self._update_interactive_extract(event)
+
+        self.active_step = 'trace'
+
+    @observe('is_active', 'bg_dataset_selected', 'bg_type_selected',
              'bg_trace_selected', 'bg_trace_pixel',
              'bg_separation', 'bg_width', 'bg_statistic_selected', 'active_step')
+    @skip_if_not_tray_instance()
+    @skip_if_no_updates_since_last_active()
     def _interaction_in_bg_step(self, event={}):
-        if not self.plugin_opened or not self._do_marks:
-            return
-        if event.get('name', '') == 'active_step' and event.get('new') != 'bg':
+        if ((event.get('name', '') in ('active_step', 'is_active') and self.active_step != 'bg')
+                or not self.is_active):
             return
 
         try:
@@ -568,22 +622,19 @@ class SpectralExtraction(PluginTemplateMixin):
                 for mark in ['bg2_center', 'bg2_lower', 'bg2_upper']:
                     self.marks[mark].clear()
 
-        try:
-            spec = self.export_bg_spectrum()
-        except Exception:
-            self.marks['bg_spec'].clear()
-        else:
-            self.marks['bg_spec'].update_xy(spec.spectral_axis, spec.flux)
+        self._update_interactive_extract(event)
 
         self.active_step = 'bg'
-        self._update_plugin_marks()
 
-    @observe('ext_dataset_selected', 'ext_trace_selected',
-             'ext_type_selected', 'ext_width', 'active_step')
+    @observe('is_active', 'ext_dataset_selected', 'ext_trace_selected',
+             'ext_type_selected', 'ext_width', 'active_step',
+             'horne_ext_profile_selected', 'self_prof_n_bins',
+             'self_prof_interp_degree_x', 'self_prof_interp_degree_y')
+    @skip_if_not_tray_instance()
+    @skip_if_no_updates_since_last_active()
     def _interaction_in_ext_step(self, event={}):
-        if not self.plugin_opened or not self._do_marks:
-            return
-        if event.get('name', '') == 'active_step' and event.get('new') != 'ext':
+        if ((event.get('name', '') in ('active_step', 'is_active') and self.active_step not in ('ext', ''))  # noqa
+                or not self.is_active):
             return
 
         try:
@@ -606,8 +657,9 @@ class SpectralExtraction(PluginTemplateMixin):
                 for mark in ['ext_lower', 'ext_upper']:
                     self.marks[mark].clear()
 
+        self._update_interactive_extract(event)
+
         self.active_step = 'ext'
-        self._update_plugin_marks()
 
         # TODO: remove this, the traitlet, and the row in spectral_extraction.vue
         # when specutils handles the warning/exception
@@ -653,6 +705,7 @@ class SpectralExtraction(PluginTemplateMixin):
         else:  # pragma: no cover
             raise NotImplementedError(f"trace of type {trace.__class__.__name__} not supported")
 
+    @with_spinner('trace_spinner')
     def export_trace(self, add_data=False, **kwargs):
         """
         Create a specreduce Trace object from the input parameters
@@ -751,6 +804,7 @@ class SpectralExtraction(PluginTemplateMixin):
 
         self.bg_width = bg.width
 
+    @with_spinner('bg_spinner')
     def export_bg(self, **kwargs):
         """
         Create a specreduce Background object from the input parameters defined in the plugin.
@@ -788,6 +842,7 @@ class SpectralExtraction(PluginTemplateMixin):
 
         return bg
 
+    @with_spinner('bg_img_spinner')
     def export_bg_img(self, add_data=False, **kwargs):
         """
         Create a background 2D spectrum from the input parameters defined in the plugin.
@@ -814,6 +869,7 @@ class SpectralExtraction(PluginTemplateMixin):
                                 color='error', sender=self)
             )
 
+    @with_spinner('bg_spec_spinner')
     def export_bg_spectrum(self, add_data=False, **kwargs):
         """
         Create a background 1D spectrum from the input parameters defined in the plugin.
@@ -829,15 +885,12 @@ class SpectralExtraction(PluginTemplateMixin):
         if add_data:
             self.bg_spec_add_results.add_results_from_plugin(spec, replace=False)
 
-        # TEMPORARY: override spectral axis to be in pixels until properly supporting plotting
-        # in wavelength/frequency
-        spec._spectral_axis = np.arange(len(spec.spectral_axis)) * units.pix
-
         return spec
 
     def vue_create_bg_spec(self, *args):
         self.export_bg_spectrum(add_data=True)
 
+    @with_spinner('bg_sub_spinner')
     def export_bg_sub(self, add_data=False, **kwargs):
         """
         Create a background-subtracted 2D spectrum from the input parameters defined in the plugin.
@@ -901,16 +954,43 @@ class SpectralExtraction(PluginTemplateMixin):
         if self.ext_type_selected == 'Boxcar':
             ext = extract.BoxcarExtract(inp_sp2d, trace, width=self.ext_width)
         elif self.ext_type_selected == 'Horne':
+            spatial_profile = None
             if inp_sp2d.uncertainty is None:
                 inp_sp2d.uncertainty = VarianceUncertainty(np.ones_like(inp_sp2d.data))
+
             if not hasattr(inp_sp2d.uncertainty, 'uncertainty_type'):
                 inp_sp2d.uncertainty = StdDevUncertainty(inp_sp2d.uncert)
-            ext = extract.HorneExtract(inp_sp2d, trace)
+
+            if self.horne_ext_profile_selected == 'Self (interpolated)':
+
+                # check inputs
+                if self.self_prof_n_bins <= 0:
+                    raise ValueError('`self_prof_n_bins` must be greater than 0.')
+                if self.self_prof_interp_degree_x <= 0:
+                    raise ValueError('`self_prof_interp_degree_x` must be greater than 0.')
+                if self.self_prof_interp_degree_y <= 0:
+                    raise ValueError('`self_prof_interp_degree_y` must be greater than 0.')
+
+                # setup dict of interpolation options
+                n_bins_interpolated_profile = self.self_prof_n_bins
+                interp_degree = (self.self_prof_interp_degree_x, self.self_prof_interp_degree_y)
+                spatial_profile = {'name': 'interpolated_profile',
+                                   'n_bins_interpolated_profile': n_bins_interpolated_profile,
+                                   'interp_degree': interp_degree}
+
+            elif self.horne_ext_profile_selected == 'Gaussian':
+                spatial_profile = 'gaussian'
+
+            else:
+                raise ValueError("Horne extraction profile must either be 'Gaussian' or 'Self (interpolated)'")  # noqa
+
+            ext = extract.HorneExtract(inp_sp2d, trace, spatial_profile=spatial_profile)
         else:
             raise NotImplementedError(f"extraction type '{self.ext_type_selected}' not supported")  # noqa
 
         return ext
 
+    @with_spinner('spinner')
     def export_extract_spectrum(self, add_data=False, **kwargs):
         """
         Create an extracted 1D spectrum from the input parameters defined in the plugin.
@@ -924,10 +1004,6 @@ class SpectralExtraction(PluginTemplateMixin):
         extract = self.export_extract(**kwargs)
         spectrum = extract.spectrum
 
-        # TEMPORARY: override spectral axis to be in pixels until properly supporting plotting
-        # in wavelength/frequency
-        spectrum._spectral_axis = np.arange(len(spectrum.spectral_axis)) * units.pix
-
         if add_data:
             self.ext_add_results.add_results_from_plugin(spectrum, replace=False)
 
