@@ -1,12 +1,18 @@
+import astropy.units as u
 from glue.core.message import Message
 
 __all__ = ['NewViewerMessage', 'ViewerAddedMessage', 'ViewerRemovedMessage', 'LoadDataMessage',
            'AddDataMessage', 'SnackbarMessage', 'RemoveDataMessage',
            'AddLineListMessage', 'RowLockMessage',
-           'SliceSelectSliceMessage',
+           'SliceSelectSliceMessage', 'SliceValueUpdatedMessage',
            'SliceToolStateMessage',
            'TableClickMessage', 'LinkUpdatedMessage', 'ExitBatchLoadMessage',
-           'MarkersChangedMessage']
+           'AstrowidgetMarkersChangedMessage', 'MarkersPluginUpdate',
+           'CanvasRotationChangedMessage',
+           'GlobalDisplayUnitChanged', 'ChangeRefDataMessage',
+           'PluginTableAddedMessage', 'PluginTableModifiedMessage',
+           'PluginPlotAddedMessage', 'PluginPlotModifiedMessage',
+           'IconsUpdatedMessage']
 
 
 class NewViewerMessage(Message):
@@ -57,6 +63,23 @@ class ViewerRemovedMessage(Message):
         return self._viewer_id
 
 
+class ViewerRenamedMessage(Message):
+    """Message emitted after a viewer is destroyed by the application."""
+    def __init__(self, old_viewer_ref, new_viewer_ref, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self._old_viewer_ref = old_viewer_ref
+        self._new_viewer_ref = new_viewer_ref
+
+    @property
+    def old_viewer_ref(self):
+        return self._old_viewer_ref
+
+    @property
+    def new_viewer_ref(self):
+        return self._new_viewer_ref
+
+
 class LoadDataMessage(Message):
     def __init__(self, path, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -69,6 +92,9 @@ class LoadDataMessage(Message):
 
 
 class AddDataMessage(Message):
+    """
+    Emitted AFTER data is added to a viewer
+    """
     def __init__(self, data, viewer, viewer_id=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
@@ -110,6 +136,32 @@ class RemoveDataMessage(Message):
         return self._viewer_id
 
 
+class ChangeRefDataMessage(Message):
+    def __init__(self, data, viewer, viewer_id=None, old=None, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self._data = data
+        self._viewer = viewer
+        self._viewer_id = viewer_id
+        self._old = old
+
+    @property
+    def data(self):
+        return self._data
+
+    @property
+    def viewer(self):
+        return self._viewer
+
+    @property
+    def viewer_id(self):
+        return self._viewer_id
+
+    @property
+    def old(self):
+        return self._old
+
+
 class SnackbarMessage(Message):
     def __init__(self, text, color=None, timeout=5000, loading=False,
                  *args, **kwargs):
@@ -149,6 +201,9 @@ class ConfigurationLoadedMessage(Message):
 
 
 class AddDataToViewerMessage(Message):
+    """
+    Emitted to request data is added to a viewer (BEFORE the data is actually added)
+    """
     def __init__(self, viewer_reference, data_label, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
@@ -260,33 +315,46 @@ class TableClickMessage(Message):
 class SliceSelectSliceMessage(Message):
     '''Message generated by the cubeviz helper and processed by the slice plugin to sync
     slice selection across all viewers'''
-    def __init__(self, slice=None, *args, **kwargs):
+    def __init__(self, value, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._slice = slice
+        self._value = value
 
     @property
-    def slice(self):
-        return self._slice
+    def value(self):
+        return self._value
+
+
+class SliceValueUpdatedMessage(Message):
+    '''Message generated by the slice plugin when the selected slice is updated'''
+    def __init__(self, value, value_unit, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.value = value
+        self.value_unit = value_unit
 
 
 class SliceToolStateMessage(Message):
     '''Message generated by the select slice plot plugin when activated/deactivated'''
-    def __init__(self, change, *args, **kwargs):
+    def __init__(self, change, viewer, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._change = change
+        self._viewer = viewer
 
     @property
     def change(self):
         return self._change
 
+    @property
+    def viewer(self):
+        return self._viewer
+
 
 class LinkUpdatedMessage(Message):
     '''Message generated when the WCS/pixel linking is changed'''
-    def __init__(self, link_type, wcs_use_fallback, wcs_use_affine, *args, **kwargs):
+    def __init__(self, link_type, wcs_use_fallback, wcs_fast_approximation, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._link_type = link_type
         self._wcs_use_fallback = wcs_use_fallback
-        self._wcs_use_affine = wcs_use_affine
+        self._wcs_fast_approximation = wcs_fast_approximation
 
     @property
     def link_type(self):
@@ -297,8 +365,8 @@ class LinkUpdatedMessage(Message):
         return self._wcs_use_fallback
 
     @property
-    def wcs_use_affine(self):
-        return self._wcs_use_affine
+    def wcs_fast_approximation(self):
+        return self._wcs_fast_approximation
 
 
 class ExitBatchLoadMessage(Message):
@@ -307,8 +375,8 @@ class ExitBatchLoadMessage(Message):
         super().__init__(*args, **kwargs)
 
 
-class MarkersChangedMessage(Message):
-    '''Message generated when markers are added/removed from an image viewer'''
+class AstrowidgetMarkersChangedMessage(Message):
+    '''Message generated when markers are added/removed from an image viewer via astrowidgets API'''
     def __init__(self, has_markers, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._has_markers = has_markers
@@ -316,3 +384,100 @@ class MarkersChangedMessage(Message):
     @property
     def has_markers(self):
         return self._has_markers
+
+
+class MarkersPluginUpdate(Message):
+    '''Message when the length of the markers plugin table changes'''
+    def __init__(self, table_length, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.table_length = table_length
+
+
+class CanvasRotationChangedMessage(Message):
+    '''Message generated by canvas rotation plugin'''
+    def __init__(self, viewer_id, angle, flip_horizontal, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._viewer_id = viewer_id
+        self._angle = angle
+        self._flip_horizontal = flip_horizontal
+
+    @property
+    def viewer_id(self):
+        return self._viewer_id
+
+    @property
+    def angle(self):
+        return self._angle
+
+    @property
+    def flip_horizontal(self):
+        return self._flip_horizontal
+
+
+class GlobalDisplayUnitChanged(Message):
+    '''Message generated when the (x or y axis) unit of the spectrum viewer is
+    changed, which is used app-wide to inform display units that depend on the
+    unit choice and flux<>sb toggle of the spectrum viewer.'''
+
+    def __init__(self, axis, unit, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._axis = axis
+        self._unit = unit
+
+    @property
+    def axis(self):
+        return self._axis
+
+    @property
+    def unit(self):
+        return u.Unit(self._unit)
+
+
+class PluginTableAddedMessage(Message):
+    '''Message generated when a plugin table is initialized'''
+    def __init__(self, sender):
+        super().__init__(sender)
+
+    @property
+    def table(self):
+        return self.sender
+
+    @property
+    def plugin(self):
+        return self.sender._plugin
+
+
+class PluginTableModifiedMessage(PluginTableAddedMessage):
+    '''Message generated when the items in a plugin table are changed'''
+    def __init__(self, sender):
+        super().__init__(sender)
+
+
+class PluginPlotAddedMessage(Message):
+    '''Message generated when a plugin plot is initialized'''
+    def __init__(self, sender):
+        super().__init__(sender)
+
+    @property
+    def plot(self):
+        return self.sender
+
+    @property
+    def plugin(self):
+        return self.sender._plugin
+
+
+class PluginPlotModifiedMessage(PluginPlotAddedMessage):
+    '''Message generated when the items in a plugin plot are changed'''
+    def __init__(self, sender):
+        super().__init__(sender)
+
+
+class IconsUpdatedMessage(Message):
+    '''Message generated when the viewer or layer icons are updated'''
+    def __init__(self, icon_type, icons, **kwargs):
+        # icon_type = 'layer' or 'viewer'
+        super().__init__(**kwargs)
+        self.icon_type = icon_type
+        # icons might be a CallbackDict, cast to ensure its a dictionary
+        self.icons = dict(icons)
