@@ -12,23 +12,25 @@ from contextlib import contextmanager
 from inspect import isclass
 
 import numpy as np
-import astropy.units as u
-from astropy.wcs.wcsapi import BaseHighLevelWCS
-from astropy.nddata import CCDData
 from glue.core import HubListener
 from glue.core.edit_subset_mode import NewMode
 from glue.core.message import SubsetCreateMessage, SubsetDeleteMessage
 from glue.core.subset import Subset, MaskSubsetState
 from glue.config import data_translator
 from ipywidgets.widgets import widget_serialization
-from specutils import Spectrum1D
 
+import astropy.units as u
+from astropy.nddata import NDDataArray, CCDData, StdDevUncertainty
+from regions.core.core import Region
+from specutils import Spectrum1D, SpectralRegion
 
 from jdaviz.app import Application
-from jdaviz.core.events import SnackbarMessage, ExitBatchLoadMessage
+from jdaviz.core.events import SnackbarMessage, ExitBatchLoadMessage, SliceSelectSliceMessage
 from jdaviz.core.template_mixin import show_widget
+from jdaviz.utils import data_has_valid_wcs, flux_conversion, spectral_axis_conversion
 
-__all__ = ['ConfigHelper', 'ImageConfigHelper']
+
+__all__ = ['ConfigHelper', 'ImageConfigHelper', 'CubeConfigHelper']
 
 
 class ConfigHelper(HubListener):
@@ -108,6 +110,18 @@ class ConfigHelper(HubListener):
             kwargs['data_label'] = data_label
         self.app.load_data(data, parser_reference=parser_reference, **kwargs)
 
+    @property
+    def data_labels(self):
+        """
+        List of data labels loaded and available in jdaviz
+
+        Returns
+        -------
+        data_labels : list
+            list of strings
+        """
+        return [data.label for data in self.app.data_collection]
+
     @property
     def plugins(self):
         """
@@ -118,8 +132,48 @@ class ConfigHelper(HubListener):
         plugins : dict
             dict of plugin objects
         """
-        return {item['label']: widget_serialization['from_json'](item['widget'], None).user_api
-                for item in self.app.state.tray_items}
+        plugins = {item['label']: widget_serialization['from_json'](item['widget'], None).user_api
+                   for item in self.app.state.tray_items if item['is_relevant']}
+
+        # handle renamed plugins during deprecation
+        if 'Orientation' in plugins:
+            # renamed in 3.9
+            plugins['Links Control'] = plugins['Orientation']._obj.user_api
+            plugins['Links Control']._deprecation_msg = 'in the future, the formerly named \"Links Control\" plugin will only be available by its new name: \"Orientation\".'  # noqa
+        if 'Canvas Rotation' in plugins:
+            # removed in 3.9
+            plugins['Canvas Rotation']._deprecation_msg = 'this functionality will be removed in favor of the implementation for rotation in the \"Orientation\" plugin.'  # noqa
+        if 'Export' in plugins:
+            # renamed in 3.9
+            plugins['Export Plot'] = plugins['Export']._obj.user_api
+            plugins['Export Plot']._deprecation_msg = 'in the future, the formerly named \"Export Plot\" plugin will only be available by its new name: \"Export\".'  # noqa
+        if 'Image Profiles (XY)' in plugins:
+            # renamed in 4.0
+            plugins['Imviz Line Profiles (XY)'] = plugins['Image Profiles (XY)']._obj.user_api
+            plugins['Imviz Line Profiles (XY)']._deprecation_msg = 'in the future, the formerly named \"Imviz Line Profiles (XY)\" plugin will only be available by its new name: \"Image Profiles (XY)\".'  # noqa
+
+        return plugins
+
+    @property
+    def plugin_tables(self):
+        return self.app._plugin_tables
+
+    @property
+    def plugin_plots(self):
+        return self.app._plugin_plots
+
+    @property
+    def viewers(self):
+        """
+        Access API objects for any viewer.
+
+        Returns
+        -------
+        viewers : dict
+            dict of viewer objects
+        """
+        return {viewer._ref_or_id: viewer.user_api
+                for viewer in self.app._viewer_store.values()}
 
     @property
     def fitted_models(self):
@@ -231,6 +285,12 @@ class ConfigHelper(HubListener):
         elif models is None:
             models = self.fitted_models
 
+        data_shapes = {}
+        for label in models:
+            data_label = label.split(" (")[0]
+            if data_label not in data_shapes:
+                data_shapes[data_label] = self.app.data_collection[data_label].data.shape
+
         param_dict = {}
         parameters_cube = {}
         param_x_y = {}
@@ -241,7 +301,7 @@ class ConfigHelper(HubListener):
             # looks for that style and separates out the pertinent information.
             if " (" in label:
                 label_split = label.split(" (")
-                model_name = label_split[0] + "_3d"
+                model_name = label_split[0]
                 x = int(label_split[1].split(", ")[0])
                 y = int(label_split[1].split(", ")[1][:-1])
 
@@ -268,10 +328,7 @@ class ConfigHelper(HubListener):
         # on whether the model in question is 3d or 1d, respectively.
         for model_name in param_dict:
             if model_name in param_x_y:
-                x_size = len(param_x_y[model_name]['x'])
-                y_size = len(param_x_y[model_name]['y'])
-
-                parameters_cube[model_name] = {x: np.zeros(shape=(x_size, y_size))
+                parameters_cube[model_name] = {x: np.zeros(shape=data_shapes[model_name][:2])
                                                for x in param_dict[model_name]}
             else:
                 parameters_cube[model_name] = {x: 0
@@ -282,7 +339,7 @@ class ConfigHelper(HubListener):
         for label in models:
             if " (" in label:
                 label_split = label.split(" (")
-                model_name = label_split[0] + "_3d"
+                model_name = label_split[0]
 
                 # If the get_models method is used to build a dictionary of
                 # models and a value is set for the x or y parameters, that
@@ -407,17 +464,63 @@ class ConfigHelper(HubListener):
                       DeprecationWarning)
         return self.show(loc="sidecar:tab-after", title=title)
 
-    def _get_data(self, data_label=None, cls=None, subset_to_apply=None, function=None):
-        list_of_valid_function_values = ('minimum', 'maximum', 'mean',
-                                         'median', 'sum')
-        if function and function not in list_of_valid_function_values:
-            raise ValueError(f"function {function} not in list of valid"
-                             f" function values {list_of_valid_function_values}")
+    def _handle_display_units(self, data, use_display_units=True):
+        if use_display_units:
+            if isinstance(data, Spectrum1D):
+                spectral_unit = self.app._get_display_unit('spectral')
+                if not spectral_unit:
+                    return data
+                y_unit = self.app._get_display_unit('spectral_y')
+                # TODO: any other attributes (meta, wcs, etc)?
+                # TODO: implement uncertainty.to upstream
+                uncertainty = data.uncertainty
+                if uncertainty is not None:
+                    # convert the uncertainties to StdDevUncertainties, since
+                    # that is assumed in a few places in jdaviz:
+                    if uncertainty.unit is None:
+                        uncertainty.unit = data.flux.unit
+                    if hasattr(uncertainty, 'represent_as'):
+                        new_uncert = uncertainty.represent_as(
+                            StdDevUncertainty
+                        )
+                    else:
+                        # if not specified as NDUncertainty, assume stddev:
+                        new_uncert = uncertainty
+                    if ('_pixel_scale_factor' in data.meta):
+                        new_uncert_converted = flux_conversion(new_uncert.quantity.value,
+                                                               new_uncert.unit, y_unit, spec=data)
+                        new_uncert = StdDevUncertainty(new_uncert_converted, unit=y_unit)
+                    else:
+                        new_uncert = StdDevUncertainty(new_uncert, unit=data.flux.unit)
 
+                else:
+                    new_uncert = None
+                if ('_pixel_scale_factor' in data.meta):
+                    new_y = flux_conversion(data.flux.value, data.flux.unit,
+                                            y_unit, data) * u.Unit(y_unit)
+                else:
+                    new_y = flux_conversion(data.flux.value, data.flux.unit,
+                                            data.flux.unit, spec=data) * u.Unit(data.flux.unit)
+                new_spec = (spectral_axis_conversion(data.spectral_axis.value,
+                                                     data.spectral_axis.unit,
+                                                     spectral_unit)
+                            * u.Unit(spectral_unit))
+
+                data = Spectrum1D(spectral_axis=new_spec,
+                                  flux=new_y,
+                                  uncertainty=new_uncert,
+                                  mask=data.mask)
+            else:  # pragma: nocover
+                raise NotImplementedError(f"converting {data.__class__.__name__} to display units is not supported")  # noqa
+        return data
+
+    def _get_data(self, data_label=None, spatial_subset=None, spectral_subset=None,
+                  temporal_subset=None, mask_subset=None, cls=None, use_display_units=False):
         list_of_valid_subset_names = [x.label for x in self.app.data_collection.subset_groups]
-        if subset_to_apply and subset_to_apply not in list_of_valid_subset_names:
-            raise ValueError(f"Subset {subset_to_apply} not in list of valid"
-                             f" subset names {list_of_valid_subset_names}")
+        for subset in (spatial_subset, spectral_subset, mask_subset):
+            if subset and subset not in list_of_valid_subset_names:
+                raise ValueError(f"Subset {subset} not in list of valid"
+                                 f" subset names {list_of_valid_subset_names}")
 
         if data_label and data_label not in self.app.data_collection.labels:
             raise ValueError(f'{data_label} not in {self.app.data_collection.labels}.')
@@ -430,6 +533,21 @@ class ConfigHelper(HubListener):
         if cls is not None and not isclass(cls):
             raise TypeError(
                 "cls in get_data must be a class or None.")
+
+        if spectral_subset:
+            if mask_subset is not None:
+                raise ValueError("cannot use both mask_subset and spectral_subset")
+            # spectral_subset is applied as a mask, the only difference is that it has
+            # its own set of validity checks (whereas mask_subset can be used by downstream
+            # apps which would then need to do their own type checks, if necessary)
+            mask_subset = spectral_subset
+
+        if temporal_subset:
+            if mask_subset is not None:
+                raise ValueError("cannot use both mask_subset and spectral_subset")
+            mask_subset = temporal_subset
+
+        # End validity checks and start data retrieval
         data = self.app.data_collection[data_label]
 
         if not cls:
@@ -440,13 +558,17 @@ class ConfigHelper(HubListener):
             elif data.ndim == 2:
                 cls = CCDData
             elif data.ndim in [1, 3]:
-                cls = Spectrum1D
+                if self.app.config == 'rampviz':
+                    cls = NDDataArray
+                else:
+                    # for cubeviz, specviz, mosviz, this must be a spectrum:
+                    cls = Spectrum1D
 
         object_kwargs = {}
         if cls == Spectrum1D:
-            object_kwargs['statistic'] = function
+            object_kwargs['statistic'] = None
 
-        if not subset_to_apply:
+        if not spatial_subset and not mask_subset:
             if 'Trace' in data.meta:
                 if cls is not None:  # pragma: no cover
                     raise ValueError("cls not supported for Trace object")
@@ -454,35 +576,64 @@ class ConfigHelper(HubListener):
             else:
                 data = data.get_object(cls=cls, **object_kwargs)
 
-            return data
+            return self._handle_display_units(data, use_display_units)
 
-        if not cls and subset_to_apply:
+        if not cls and spatial_subset:
             raise AttributeError(f"A valid cls must be provided to"
-                                 f" apply subset {subset_to_apply} to data. "
+                                 f" apply subset {spatial_subset} to data. "
+                                 f"Instead, {cls} was given.")
+        elif not cls and mask_subset:
+            raise AttributeError(f"A valid cls must be provided to"
+                                 f" apply subset {mask_subset} to data. "
                                  f"Instead, {cls} was given.")
 
-        # Loop through each subset
-        for subsets in self.app.data_collection.subset_groups:
-            # If name matches the name in subsets_to_apply, continue
-            if subsets.label.lower() == subset_to_apply.lower():
-                # Loop through each data a subset applies to
-                for subset in subsets.subsets:
-                    # If the subset applies to data with the same name as data_label, continue
-                    if subset.data.label == data_label:
+        # Now we work on applying subsets to the data
+        all_subsets = self.app.get_subsets(object_only=True)
 
-                        handler, _ = data_translator.get_handler_for(cls)
-                        try:
-                            data = handler.to_object(subset, **object_kwargs)
-                        except Exception as e:
-                            warnings.warn(f"Not able to get {data_label} returned with"
-                                          f" subset {subsets.label} applied of type {cls}."
-                                          f" Exception: {e}")
-        return data
+        # Handle spatial subset
+        if spatial_subset and not isinstance(all_subsets[spatial_subset][0],
+                                             Region):
+            raise ValueError(f"{spatial_subset} is not a spatial subset.")
+        elif spatial_subset:
+            real_spatial = [sub for subsets in self.app.data_collection.subset_groups
+                            for sub in subsets.subsets
+                            if sub.data.label == data_label and subsets.label == spatial_subset][0]
+            handler, _ = data_translator.get_handler_for(cls)
+            try:
+                data = handler.to_object(real_spatial, **object_kwargs)
+            except Exception as e:
+                warnings.warn(f"Not able to get {data_label} returned with"
+                              f" subset {spatial_subset} applied of type {cls}."
+                              f" Exception: {e}")
 
-    def get_data(self, data_label=None, cls=None, subset_to_apply=None):
+        # Handle spectral subset, including case where spatial subset is also set
+        if spectral_subset and not isinstance(all_subsets[spectral_subset],
+                                              SpectralRegion):
+            raise ValueError(f"{spectral_subset} is not a spectral subset.")
+
+        if mask_subset:
+            real_spectral = [sub for subsets in self.app.data_collection.subset_groups
+                             for sub in subsets.subsets
+                             if sub.data.label == data_label and subsets.label == mask_subset][0] # noqa
+
+            handler, _ = data_translator.get_handler_for(cls)
+            try:
+                spec_subset = handler.to_object(real_spectral, **object_kwargs)
+            except Exception as e:
+                warnings.warn(f"Not able to get {data_label} returned with"
+                              f" subset {mask_subset} applied of type {cls}."
+                              f" Exception: {e}")
+            if spatial_subset:
+                # Return collapsed Spectrum1D object with spectral subset mask applied
+                data.mask = spec_subset.mask
+            else:
+                data = spec_subset
+
+        return self._handle_display_units(data, use_display_units)
+
+    def get_data(self, data_label=None, cls=None, use_display_units=False, **kwargs):
         """
-        Returns data with name equal to data_label of type cls with subsets applied from
-        subset_to_apply.
+        Returns data with name equal to data_label of type cls.
 
         Parameters
         ----------
@@ -490,16 +641,17 @@ class ConfigHelper(HubListener):
             Provide a label to retrieve a specific data set from data_collection.
         cls : `~specutils.Spectrum1D`, `~astropy.nddata.CCDData`, optional
             The type that data will be returned as.
-        subset_to_apply : str, optional
-            Subset that is to be applied to data before it is returned.
+        use_display_units : bool, optional
+            Whether to convert to the display units defined in the <unit-conversion> plugin.
 
         Returns
         -------
         data : cls
-            Data is returned as type cls with subsets applied.
+            Data is returned as type ``cls``.
 
         """
-        return self._get_data(data_label=data_label, cls=cls, subset_to_apply=subset_to_apply)
+        return self._get_data(data_label=data_label,
+                              cls=cls, use_display_units=use_display_units)
 
 
 class ImageConfigHelper(ConfigHelper):
@@ -517,7 +669,7 @@ class ImageConfigHelper(ConfigHelper):
     def default_viewer(self):
         """Default viewer instance. This is typically the first viewer
         (e.g., "imviz-0" or "cubeviz-0")."""
-        return self._default_viewer
+        return self._default_viewer.user_api
 
     def load_regions_from_file(self, region_file, region_format='ds9', max_num_regions=20,
                                **kwargs):
@@ -578,7 +730,6 @@ class ImageConfigHelper(ConfigHelper):
             * Astropy ``regions`` object
             * ``photutils`` apertures (limited support until ``photutils``
               fully supports ``regions``)
-            * Numpy boolean array (shape must match data, dtype should be ``np.bool_``)
 
         max_num_regions : int or `None`
             Maximum number of regions to load, starting from top of the list.
@@ -588,7 +739,7 @@ class ImageConfigHelper(ConfigHelper):
             Label of data to use for sky-to-pixel conversion for a region, or
             mask creation. Data must already be loaded into Jdaviz.
             If `None`, defaults to the reference data in the default viewer.
-            Choice of this data is particularly important when sky or masked
+            Choice of this data is particularly important when sky
             region is involved.
 
         return_bad_regions : bool
@@ -597,8 +748,7 @@ class ImageConfigHelper(ConfigHelper):
 
         kwargs : dict
             Extra keywords to be passed into the region's ``to_mask`` method.
-            **This is ignored if the region can be made interactive or
-            if a Numpy array is given.**
+            **This is ignored if the region can be made interactive.**
 
         Returns
         -------
@@ -609,12 +759,17 @@ class ImageConfigHelper(ConfigHelper):
             If not requested, return `None`.
 
         """
+        if len(self.app.data_collection) == 0:
+            raise ValueError('Cannot load regions without data.')
+
         from photutils.aperture import (CircularAperture, SkyCircularAperture,
                                         EllipticalAperture, SkyEllipticalAperture,
-                                        RectangularAperture, SkyRectangularAperture)
+                                        RectangularAperture, SkyRectangularAperture,
+                                        CircularAnnulus, SkyCircularAnnulus)
         from regions import (Regions, CirclePixelRegion, CircleSkyRegion,
                              EllipsePixelRegion, EllipseSkyRegion,
-                             RectanglePixelRegion, RectangleSkyRegion)
+                             RectanglePixelRegion, RectangleSkyRegion,
+                             CircleAnnulusPixelRegion, CircleAnnulusSkyRegion)
         from jdaviz.core.region_translators import regions2roi, aperture2regions
 
         # If user passes in one region obj instead of list, try to be smart.
@@ -630,7 +785,7 @@ class ImageConfigHelper(ConfigHelper):
 
         # Subset is global but reference data is viewer-dependent.
         if refdata_label is None:
-            data = self.default_viewer.state.reference_data
+            data = self.default_viewer._obj.state.reference_data
         else:
             data = self.app.data_collection[refdata_label]
 
@@ -639,28 +794,40 @@ class ImageConfigHelper(ConfigHelper):
         has_wcs = data_has_valid_wcs(data, ndim=2)
 
         for region in regions:
-            if isinstance(region, (SkyCircularAperture, SkyEllipticalAperture,
-                                   SkyRectangularAperture, CircleSkyRegion,
-                                   EllipseSkyRegion, RectangleSkyRegion)) and not has_wcs:
+            if (isinstance(region, (SkyCircularAperture, SkyEllipticalAperture,
+                                    SkyRectangularAperture, SkyCircularAnnulus,
+                                    CircleSkyRegion, EllipseSkyRegion,
+                                    RectangleSkyRegion, CircleAnnulusSkyRegion))
+                    and not has_wcs):
                 bad_regions.append((region, 'Sky region provided but data has no valid WCS'))
                 continue
 
+            if (isinstance(region, (CircularAperture, EllipticalAperture,
+                                    RectangularAperture, CircularAnnulus,
+                                    CirclePixelRegion, EllipsePixelRegion,
+                                    RectanglePixelRegion, CircleAnnulusPixelRegion))
+                    and self.app._align_by == "wcs"):
+                bad_regions.append((region, 'Pixel region provided by data is linked by WCS'))
+                continue
+
             # photutils: Convert to regions shape first
             if isinstance(region, (CircularAperture, SkyCircularAperture,
                                    EllipticalAperture, SkyEllipticalAperture,
-                                   RectangularAperture, SkyRectangularAperture)):
+                                   RectangularAperture, SkyRectangularAperture,
+                                   CircularAnnulus, SkyCircularAnnulus)):
                 region = aperture2regions(region)
 
             # regions: Convert to ROI.
             # NOTE: Out-of-bounds ROI will succeed; this is native glue behavior.
             if isinstance(region, (CirclePixelRegion, CircleSkyRegion,
                                    EllipsePixelRegion, EllipseSkyRegion,
-                                   RectanglePixelRegion, RectangleSkyRegion)):
+                                   RectanglePixelRegion, RectangleSkyRegion,
+                                   CircleAnnulusPixelRegion, CircleAnnulusSkyRegion)):
                 state = regions2roi(region, wcs=data.coords)
 
                 # TODO: Do we want user to specify viewer? Does it matter?
                 self.app.session.edit_subset_mode._mode = NewMode
-                self.default_viewer.apply_roi(state)
+                self.default_viewer._obj.apply_roi(state)
                 self.app.session.edit_subset_mode.edit_subset = None  # No overwrite next iteration # noqa
 
             # Last resort: Masked Subset that is static (if data is not a cube)
@@ -680,6 +847,7 @@ class ImageConfigHelper(ConfigHelper):
                         bad_regions.append((region, f'Failed to load: {repr(e)}'))
                         continue
 
+                # Boolean mask as input is supported but not advertised.
                 elif (isinstance(region, np.ndarray) and region.shape == data.shape
                         and region.dtype == np.bool_):
                     im = region
@@ -735,11 +903,14 @@ class ImageConfigHelper(ConfigHelper):
             ``regions`` objects.
 
         """
+        from glue_astronomy.translators.regions import roi_subset_state_to_region
+
         regions = {}
         failed_regs = set()
+        to_sky = self.app._align_by == 'wcs'
 
         # Subset is global, so we just use default viewer.
-        for lyr in self.default_viewer.layers:
+        for lyr in self.default_viewer._obj.layers:
             if (not hasattr(lyr, 'layer') or not isinstance(lyr.layer, Subset)
                     or lyr.layer.ndim not in (2, 3)):
                 continue
@@ -753,8 +924,11 @@ class ImageConfigHelper(ConfigHelper):
                 continue
 
             try:
-                region = subset_data.data.get_selection_definition(
-                    subset_id=subset_label, format='astropy-regions')
+                if self.app.config == "imviz" and to_sky:
+                    region = roi_subset_state_to_region(subset_data.subset_state, to_sky=to_sky)
+                else:
+                    region = subset_data.data.get_selection_definition(
+                        subset_id=subset_label, format='astropy-regions')
             except (NotImplementedError, ValueError):
                 failed_regs.add(subset_label)
             else:
@@ -773,7 +947,7 @@ class ImageConfigHelper(ConfigHelper):
         This is for internal testing only.
         """
         self.app.session.edit_subset_mode._mode = NewMode
-        tool = self.default_viewer.toolbar.tools[toolname]
+        tool = self.default_viewer._obj.toolbar.tools[toolname]
         tool.activate()
         tool.interact.brushing = True
         tool.interact.selected = [from_pix, to_pix]
@@ -797,14 +971,6 @@ class ImageConfigHelper(ConfigHelper):
             self.app.data_collection.remove_subset_group(subset_grp)
 
 
-def data_has_valid_wcs(data, ndim=None):
-    """Check if given glue Data has WCS that is compatible with APE 14."""
-    status = hasattr(data, 'coords') and isinstance(data.coords, BaseHighLevelWCS)
-    if ndim is not None:
-        status = status and data.coords.world_n_dim == ndim
-    return status
-
-
 def _next_subset_num(label_prefix, subset_groups):
     """Assumes ``prefix i`` format.
     Does not go back and fill in lower but available numbers. This is consistent with Glue.
@@ -825,3 +991,27 @@ def _next_subset_num(label_prefix, subset_groups):
                         max_i = i
 
     return max_i + 1
+
+
+class CubeConfigHelper(ImageConfigHelper):
+    """Base config helper class for cubes"""
+
+    _loaded_flux_cube = None
+    _loaded_uncert_cube = None
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def select_slice(self, value):
+        """
+        Select the slice closest to the provided value.
+
+        Parameters
+        ----------
+        value : float or int, optional
+            Slice value to select in units of the x-axis of the profile viewer.
+            The nearest slice will be selected if "snap to slice" is enabled in
+            the slice plugin.
+        """
+        msg = SliceSelectSliceMessage(value=value, sender=self)
+        self.app.hub.broadcast(msg)
