@@ -3,20 +3,24 @@ import numpy as np
 from astropy import units as u
 from bqplot import LinearScale
 from bqplot.marks import Lines, Label, Scatter
-from copy import deepcopy
 from glue.core import HubListener
 from specutils import Spectrum1D
+from jdaviz.utils import _eqv_pixar_sr, _eqv_flux_to_sb_pixel
 
+from jdaviz.core.events import GlobalDisplayUnitChanged
 from jdaviz.core.events import (SliceToolStateMessage, LineIdentifyMessage,
                                 SpectralMarksChangedMessage,
                                 RedshiftMessage)
 
 __all__ = ['OffscreenLinesMarks', 'BaseSpectrumVerticalLine', 'SpectralLine',
            'SliceIndicatorMarks', 'ShadowMixin', 'ShadowLine', 'ShadowLabelFixedY',
-           'PluginMark', 'PluginLine', 'PluginScatter',
+           'PluginMark', 'LinesAutoUnit', 'PluginLine', 'PluginScatter',
            'LineAnalysisContinuum', 'LineAnalysisContinuumCenter',
            'LineAnalysisContinuumLeft', 'LineAnalysisContinuumRight',
-           'LineUncertainties', 'ScatterMask', 'SelectedSpaxel', 'MarkersMark']
+           'LineUncertainties', 'ScatterMask', 'SelectedSpaxel', 'MarkersMark',
+           'CatalogMark', 'FootprintOverlay', 'ApertureMark']
+
+accent_color = "#c75d2c"
 
 
 class OffscreenLinesMarks(HubListener):
@@ -58,16 +62,122 @@ class OffscreenLinesMarks(HubListener):
         self.right.text = [f'{oob_right} \u25b6' if oob_right > 0 else '']
 
 
-class BaseSpectrumVerticalLine(Lines, HubListener):
+class PluginMark:
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.xunit = None
+        self.yunit = None
+        # whether to update existing marks when global display units are changed
+        self.auto_update_units = True
+        self.hub.subscribe(self, GlobalDisplayUnitChanged,
+                           handler=self._on_global_display_unit_changed)
+
+        if self.xunit is None:
+            self.set_x_unit()
+        if self.yunit is None:
+            self.set_y_unit()
+
+    @property
+    def hub(self):
+        return self.viewer.hub
+
+    def update_xy(self, x, y):
+        # If x and y are not in the previous units, they should be provided as quantities
+        if hasattr(x, 'value'):
+            xunit = x.unit
+            x = x.value
+        else:
+            xunit = None
+        self.x = np.asarray(x)
+        if xunit is not None:
+            self.xunit = u.Unit(xunit)
+
+        if hasattr(y, 'value'):
+            yunit = y.unit
+            y = y.value
+        else:
+            yunit = None
+        self.y = np.asarray(y)
+        if yunit is not None:
+            self.yunit = u.Unit(yunit)
+
+    def append_xy(self, x, y):
+        self.x = np.append(self.x, x)
+        self.y = np.append(self.y, y)
+
+    def set_x_unit(self, unit=None):
+        if unit is None:
+            if not hasattr(self.viewer.state, 'x_display_unit'):
+                return
+            unit = self.viewer.state.x_display_unit
+        unit = u.Unit(unit)
+
+        if self.xunit is not None and not np.all([s == 0 for s in self.x.shape]):
+            x = (self.x * self.xunit).to_value(unit, u.spectral())
+            self.xunit = unit
+            self.x = x
+        self.xunit = unit
+
+    def set_y_unit(self, unit=None):
+        if unit is None:
+            if not hasattr(self.viewer.state, 'y_display_unit'):
+                return
+            unit = self.viewer.state.y_display_unit
+        unit = u.Unit(unit)
+
+        if self.yunit is not None and not np.all([s == 0 for s in self.y.shape]):
+            if self.viewer.default_class is Spectrum1D:
+                # used to obtain spectral density equivalencies with previous data and units
+                eqv = u.spectral_density(self.x*self.xunit)
+
+                spec = self.viewer.state.reference_data.get_object(cls=Spectrum1D)
+
+                if ('_pixel_scale_factor' in spec.meta):
+                    eqv += _eqv_pixar_sr(spec.meta['_pixel_scale_factor'])
+
+                # add equiv for flux <> flux/pix2
+                eqv += _eqv_flux_to_sb_pixel()
+
+                y = (self.y * self.yunit).to_value(unit, equivalencies=eqv)
+            else:
+                y = (self.y * self.yunit).to_value(unit)
+            self.yunit = unit
+            self.y = y
+
+        self.yunit = unit
+
+    def _on_global_display_unit_changed(self, msg):
+        if not self.auto_update_units:
+            return
+        if self.viewer.__class__.__name__ in ['SpecvizProfileView', 'CubevizProfileView']:
+            axis_map = {'spectral': 'x', 'spectral_y': 'y'}
+        elif self.viewer.__class__.__name__ == 'MosvizProfile2DView':
+            axis_map = {'spectral': 'x'}
+        else:
+            return
+        axis = axis_map.get(msg.axis, None)
+        if axis is not None:
+            scale = self.scales.get(axis, None)
+            # if PluginMark mark is LinearScale(0, 1), prevent it from entering unit conversion
+            # machinery so it maintains it's position in viewer.
+            if isinstance(scale, LinearScale) and (scale.min, scale.max) == (0, 1):
+                return
+
+            getattr(self, f'set_{axis}_unit')(msg.unit)
+
+    def clear(self):
+        self.update_xy([], [])
+
+
+class BaseSpectrumVerticalLine(Lines, PluginMark, HubListener):
     def __init__(self, viewer, x, **kwargs):
-        # we'll store the current units so that we can automatically update the
-        # positioning on a change to the x-units
-        self._x_unit = viewer.state.reference_data.get_object(cls=Spectrum1D).spectral_axis.unit
+        self.viewer = viewer
 
         # the location of the marker will need to update automatically if the
         # underlying data changes (through a unit conversion, for example)
-        viewer.state.add_callback("reference_data",
-                                  self._update_reference_data)
+        if hasattr(viewer.state, 'reference_data'):
+            viewer.state.add_callback("reference_data",
+                                      self._update_reference_data)
 
         scales = viewer.scales
 
@@ -77,20 +187,24 @@ class BaseSpectrumVerticalLine(Lines, HubListener):
                          **kwargs)
 
     def _update_reference_data(self, reference_data):
-        if reference_data is None:
+        # don't update x units before initialization or in rampviz
+        if reference_data is None or self.viewer.jdaviz_app.config == 'rampviz':
             return
-        self._update_data(reference_data.get_object(cls=Spectrum1D).spectral_axis)
 
-    def _update_data(self, x_all):
+        self._update_unit(reference_data.get_object(cls=Spectrum1D).spectral_axis.unit)
+
+    def _update_unit(self, new_unit):
         # the x-units may have changed.  We want to convert the internal self.x
-        # from self._x_unit to the new units (x_all.unit)
-        new_unit = x_all.unit
-        if new_unit == self._x_unit:
+        # from self.xunit to the new units (x_all.unit)
+        if self.xunit is None:
+            self.xunit = new_unit
             return
-        old_quant = self.x[0]*self._x_unit
-        x = old_quant.to_value(x_all.unit, equivalencies=u.spectral())
+        if new_unit == self.xunit:
+            return
+        old_quant = self.x[0]*self.xunit
+        x = old_quant.to_value(new_unit, equivalencies=u.spectral())
         self.x = [x, x]
-        self._x_unit = new_unit
+        self.xunit = new_unit
 
 
 class SpectralLine(BaseSpectrumVerticalLine):
@@ -110,7 +224,7 @@ class SpectralLine(BaseSpectrumVerticalLine):
         # setting redshift will set self.x and enable the obs_value property,
         # but to do that we need x_unit set first (would normally be assigned
         # in the super init)
-        self._x_unit = viewer.state.reference_data.get_object(cls=Spectrum1D).spectral_axis.unit
+        self.xunit = u.Unit(viewer.state.x_display_unit)
         self.redshift = redshift
 
         viewer.session.hub.subscribe(self, LineIdentifyMessage,
@@ -131,6 +245,11 @@ class SpectralLine(BaseSpectrumVerticalLine):
     def obs_value(self):
         return self.x[0]
 
+    def set_x_unit(self, unit=None):
+        prev_unit = self.xunit
+        super().set_x_unit(unit=unit)
+        self._rest_value = (self._rest_value * prev_unit).to_value(unit, u.spectral())
+
     @property
     def redshift(self):
         return self._redshift
@@ -138,16 +257,16 @@ class SpectralLine(BaseSpectrumVerticalLine):
     @redshift.setter
     def redshift(self, redshift):
         self._redshift = redshift
-        if str(self._x_unit.physical_type) == 'length':
+        if str(self.xunit.physical_type) == 'length':
             obs_value = self._rest_value*(1+redshift)
-        elif str(self._x_unit.physical_type) == 'frequency':
+        elif str(self.xunit.physical_type) == 'frequency':
             obs_value = self._rest_value/(1+redshift)
         else:
             # catch all for anything else (wavenumber, energy, etc)
-            rest_angstrom = (self._rest_value*self._x_unit).to_value(u.Angstrom,
-                                                                     equivalencies=u.spectral())
+            rest_angstrom = (self._rest_value*self.xunit).to_value(u.Angstrom,
+                                                                   equivalencies=u.spectral())
             obs_angstrom = rest_angstrom*(1+redshift)
-            obs_value = (obs_angstrom*u.Angstrom).to_value(self._x_unit,
+            obs_value = (obs_angstrom*u.Angstrom).to_value(self.xunit,
                                                            equivalencies=u.spectral())
         self.x = [obs_value, obs_value]
 
@@ -166,46 +285,45 @@ class SpectralLine(BaseSpectrumVerticalLine):
     def _process_identify_change(self, msg):
         self.identify = msg.name_rest == self.table_index
 
-    def _update_data(self, x_all):
-        new_unit = x_all.unit
-        if new_unit == self._x_unit:
+    def _update_unit(self, new_unit):
+        if self.xunit is None:
+            self.xunit = new_unit
+            return
+        if new_unit == self.xunit:
             return
 
-        old_quant = self._rest_value*self._x_unit
+        old_quant = self._rest_value*self.xunit
         self._rest_value = old_quant.to_value(new_unit, equivalencies=u.spectral())
         # re-compute self.x from current redshift (instead of converting that as well)
         self.redshift = self._redshift
-        self._x_unit = new_unit
+        self.xunit = new_unit
 
 
 class SliceIndicatorMarks(BaseSpectrumVerticalLine, HubListener):
     """Subclass on bqplot Lines to handle slice/wavelength indicator.
     """
-    def __init__(self, viewer, slice=0, **kwargs):
+    def __init__(self, viewer, value=0, **kwargs):
         self._viewer = viewer
+        self._value = None
         self._oob = False  # out-of-bounds, either False, 'left', or 'right'
         self._active = False
+        # TODO: new viewers need to respect plugin settings
         self._show_if_inactive = True
-        self._show_wavelength = True
+        self._show_value = True
 
-        self.slice = slice
-        x_all = viewer.data()[0].spectral_axis
-        # _update_data will set self._x_all, self._x_unit, self.x
-        self._update_data(x_all)
-
-        viewer.state.add_callback("x_min", lambda x_min: self._handle_oob(update_label=True))
-        viewer.state.add_callback("x_max", lambda x_max: self._handle_oob(update_label=True))
+        viewer.state.add_callback("x_min", lambda x_min: self._value_handle_oob(update_label=True))
+        viewer.state.add_callback("x_max", lambda x_max: self._value_handle_oob(update_label=True))
         viewer.session.hub.subscribe(self, SliceToolStateMessage,
                                      handler=self._on_change_state)
 
         super().__init__(viewer=viewer,
-                         x=self.x[0],
+                         x=[value, value],
                          stroke_width=2,
                          marker='diamond',
                          fill='none', close_path=False,
                          labels=['slice'], labels_visibility='none', **kwargs)
 
-        self._handle_oob()
+        self.value = value
 
         # instead of using the Lines label which is limited, we'll use a Label object which
         # will follow the x-coordinate of the slice indicator line, with a fixed y-value
@@ -215,63 +333,72 @@ class SliceIndicatorMarks(BaseSpectrumVerticalLine, HubListener):
 
         # default to the initial state of the tool since we can't control if this will
         # happen before or after the initialization of the tool
-        self._on_change_state({'active': True})
+        tool_active = self.viewer.toolbar.active_tool_id == 'jdaviz:selectslice'
+        self._on_change_state({'active': tool_active})
 
     @property
     def marks(self):
         return [self, self.label]
 
-    def _handle_oob(self, x_coord=None, update_label=False):
-        if x_coord is None:
-            x_coord = self._slice_to_x(self.slice)
+    def _on_global_display_unit_changed(self, msg):
+        # Updating the value is handled by the plugin itself, need to update unit string.
+        if msg.axis in ["spectral", "x"]:
+            self.xunit = msg.unit
+        self._update_label()
+
+    def _value_handle_oob(self, x=None, update_label=False):
+        if x is None:
+            x = self.value
+        else:
+            self._value = x
         x_min, x_max = self._viewer.state.x_min, self._viewer.state.x_max
         if x_min is None or x_max is None:
-            self.x = [x_coord, x_coord]
+            self.x = [x, x]
             return
         x_range = x_max - x_min
         padding_fig = 0.01
         padding = padding_fig * x_range
         x_min += padding
         x_max -= padding
-        if x_coord < x_min:
+        # ensure y-scale has been set (we'll only be overriding x, but scatter viewers complain
+        # if y-scale is not set)
+        self.scales.setdefault('y', LinearScale(min=0, max=1))
+        if x < x_min:
             self.x = [padding_fig, padding_fig]
             self.scales = {**self.scales, 'x': LinearScale(min=0, max=1)}
             self.line_style = 'dashed'
             self._oob = 'left'
-        elif x_coord > x_max:
+        elif x > x_max:
             self.x = [1-padding_fig, 1-padding_fig]
             self.scales = {**self.scales, 'x': LinearScale(min=0, max=1)}
             self.line_style = 'dashed'
             self._oob = 'right'
         else:
-            self.x = [x_coord, x_coord]
+            self.x = [x, x]
             self.scales = {**self.scales, 'x': self._viewer.scales['x']}
             self.line_style = 'solid'
             self._oob = False
         if update_label:
             self._update_label()
 
-    def _slice_to_x(self, slice=0):
-        if not isinstance(slice, int):
-            raise TypeError(f"slice must be of type int, not {type(slice)}")
-        return self._x_all[slice]
-
     def _update_colors_opacities(self):
-        # orange (accent) if active, import button blue otherwise (see css in app.vue)
+        # orange (accent) if active, import button blue otherwise (see css in main_styles.vue)
         if not self._show_if_inactive and not self._active:
             self.label.visible = False
             self.visible = False
             return
 
         self.visible = True
-        self.label.visible = self._show_wavelength
+        self.label.visible = self._show_value
         self.colors = ["#c75109" if self._active else "#007BA1"]
         self.opacities = [1.0 if self._active else 0.9]
 
-    def _on_change_state(self, msg):
+    def _on_change_state(self, msg={}):
         if isinstance(msg, dict):
             changes = msg
         else:
+            if msg.viewer is not None and msg.viewer != self.viewer:
+                return
             changes = msg.change
 
         for k, v in changes.items():
@@ -279,45 +406,37 @@ class SliceIndicatorMarks(BaseSpectrumVerticalLine, HubListener):
                 self._active = v
             elif k == 'show_indicator':
                 self._show_if_inactive = v
-            elif k == 'show_wavelength':
-                self._show_wavelength = v
+            elif k == 'show_value':
+                self._show_value = v
 
         self._update_colors_opacities()
 
     def _update_label(self):
+        def _formatted_value(value):
+            power = abs(np.log10(value))
+            if power >= 3:
+                # use scientific notation
+                return f'{value:0.4e}'
+            else:
+                return f'{value:0.4f}'
+
+        valuestr = _formatted_value(self.value)
+        xunit = str(self.xunit) if self.xunit is not None else ''
         # U+00A0 is a blank space, U+25C0 a left arrow triangle, and U+25B6 a right arrow triangle
         if self._oob == 'left':
-            self.labels = [f'\u00A0 \u25c0 {self._slice_to_x(self.slice):0.4e} {self._x_unit} \u00A0']  # noqa
+            self.labels = [f'\u00A0 \u25c0 {valuestr} {xunit} \u00A0']  # noqa
         elif self._oob == 'right':
-            self.labels = [f'{self._slice_to_x(self.slice):0.4e} {self._x_unit} \u25b6 \u00A0']
+            self.labels = [f'{valuestr} {xunit} \u25b6 \u00A0']
         else:
-            self.labels = [f'\u00A0 {self._slice_to_x(self.slice):0.4e} {self._x_unit} \u00A0']
+            self.labels = [f'\u00A0 {valuestr} {xunit} \u00A0']
 
     @property
-    def slice(self):
-        return self._slice
+    def value(self):
+        return self._value
 
-    @slice.setter
-    def slice(self, slice):
-        self._slice = slice
-        # if this is within the init, the data may not have been set yet,
-        # in which case we'll just set self._slice for the first time, but
-        # do not need to update self.x or label (yet)
-        if hasattr(self, '_x_all'):
-            x_coord = self._slice_to_x(slice)
-            self._handle_oob(x_coord)
-            self._update_label()
-
-    def _update_data(self, x_all):
-        # we want to preserve slice number, so we'll do a bit more than the
-        # default unit-conversion in the base class
-        self._x_all = x_all.value
-        self._x_unit = str(x_all.unit)
-        x_coord = self._slice_to_x(self.slice)
-        self._handle_oob(x_coord)
-        if self.labels_visibility == 'label':
-            # update label with new value/unit
-            self._update_label()
+    @value.setter
+    def value(self, value):
+        self._value_handle_oob(value, update_label=True)
 
 
 class ShadowMixin:
@@ -374,51 +493,6 @@ class ShadowLine(Lines, HubListener, ShadowMixin):
                               ['stroke_width', 'marker_size'])
 
 
-class ShadowSpatialSpectral(Lines, HubListener, ShadowMixin):
-    """
-    Shadow the mark of a spatial subset collapsed spectrum, with the mask from a spectral subset,
-    and the styling from the spatial subset.
-    """
-    def __init__(self, spatial_spectrum_mark, spectral_subset_mark):
-        # spatial_spectrum_mark: Lines mark corresponding to the spatially-collapsed spectrum
-        # from a spatial subset
-        # spectral_subset_mark: Lines mark on the FULL cube corresponding to the glue-highlight
-        # of the spectral subset
-        super().__init__(scales=spatial_spectrum_mark.scales, marker=None)
-
-        self._spatial_mark_id = self._get_id(spatial_spectrum_mark)
-        self._setup_shadowing(spatial_spectrum_mark,
-                              ['scales', 'y', 'visible', 'line_style'],
-                              ['x'])
-
-        self._spectral_mark_id = self._get_id(spectral_subset_mark)
-        self._setup_shadowing(spectral_subset_mark,
-                              ['stroke_width', 'x', 'y', 'visible', 'opacities', 'colors'])
-
-    @property
-    def spatial_spectrum_mark(self):
-        return self._shadowing[self._spatial_mark_id]
-
-    @property
-    def spectral_subset_mark(self):
-        return self._shadowing[self._spectral_mark_id]
-
-    def _on_shadowing_changed(self, change):
-        if hasattr(self, '_spectral_mark_id'):
-            if change['name'] == 'y':
-                # at initial setup, the arrays may not be populated yet
-                if self.spatial_spectrum_mark.y.shape == self.spectral_subset_mark.y.shape:
-                    # force a copy or else we'll overwrite the mask to the spatial mark!
-                    change['new'] = deepcopy(self.spatial_spectrum_mark.y)
-                    change['new'][np.isnan(self.spectral_subset_mark.y)] = np.nan
-
-            elif change['name'] == 'visible':
-                # only show if BOTH shadowing marks are set to visible
-                change['new'] = self.spectral_subset_mark.visible and self.spatial_spectrum_mark.visible  # noqa
-
-        return super()._on_shadowing_changed(change)
-
-
 class ShadowLabelFixedY(Label, ShadowMixin):
     """Label whose position shadows that of a parent ``shadowing``
     line and will flip alignment based on whether it is left or
@@ -484,33 +558,35 @@ class ShadowLabelFixedY(Label, ShadowMixin):
             self._update_align()
 
 
-class PluginMark():
-    def update_xy(self, x, y):
-        self.x = np.asarray(x)
-        self.y = np.asarray(y)
-
-    def append_xy(self, x, y):
-        self.x = np.append(self.x, x)
-        self.y = np.append(self.y, y)
-
-    def clear(self):
-        self.update_xy([], [])
+class LinesAutoUnit(PluginMark, Lines, HubListener):
+    def __init__(self, viewer, *args, **kwargs):
+        self.viewer = viewer
+        super().__init__(*args, **kwargs)
 
 
 class PluginLine(Lines, PluginMark, HubListener):
     def __init__(self, viewer, x=[], y=[], **kwargs):
+        self.viewer = viewer
         # color is same blue as import button
-        super().__init__(x=x, y=y, colors=["#007BA1"], scales=viewer.scales, **kwargs)
+        kwargs.setdefault('colors', [accent_color])
+        self.label = kwargs.get('label')
+        super().__init__(x=x, y=y, scales=kwargs.pop('scales', viewer.scales), **kwargs)
 
 
 class PluginScatter(Scatter, PluginMark, HubListener):
     def __init__(self, viewer, x=[], y=[], **kwargs):
-        # color is same blue as import button
-        super().__init__(x=x, y=y, colors=["#007BA1"], scales=viewer.scales, **kwargs)
+        self.viewer = viewer
+        # default color is same blue as import button
+        kwargs.setdefault('colors', [accent_color])
+        super().__init__(x=x, y=y, scales=kwargs.pop('scales', viewer.scales), **kwargs)
 
 
 class LineAnalysisContinuum(PluginLine):
-    pass
+    def __init__(self, *args, **kwargs):
+        # units do not need to be updated because the plugin itself reruns
+        # the computation and automatically changes the arrays themselves
+        self.auto_update_units = kwargs.pop('auto_update_units', False)
+        super().__init__(*args, **kwargs)
 
 
 class LineAnalysisContinuumCenter(LineAnalysisContinuum):
@@ -529,9 +605,9 @@ class LineAnalysisContinuumRight(LineAnalysisContinuumLeft):
     pass
 
 
-class LineUncertainties(Lines):
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
+class LineUncertainties(LinesAutoUnit):
+    def __init__(self, viewer, *args, **kwargs):
+        super().__init__(viewer, *args, **kwargs)
 
 
 class ScatterMask(Scatter):
@@ -548,3 +624,40 @@ class MarkersMark(PluginScatter):
     def __init__(self, viewer, **kwargs):
         kwargs.setdefault('marker', 'circle')
         super().__init__(viewer, **kwargs)
+
+
+class CatalogMark(PluginScatter):
+    def __init__(self, viewer, **kwargs):
+        kwargs.setdefault('marker', 'circle')
+        super().__init__(viewer, **kwargs)
+
+
+class FootprintOverlay(PluginLine):
+    def __init__(self, viewer, overlay, **kwargs):
+        self._overlay = overlay
+        kwargs.setdefault('stroke_width', 2)
+        kwargs.setdefault('close_path', True)
+        kwargs.setdefault('opacities', [0.8])
+        kwargs.setdefault('fill', 'inside')
+        kwargs.setdefault('fill_opacities', [0.2])
+        super().__init__(viewer, **kwargs)
+
+    @property
+    def overlay(self):
+        return self._overlay
+
+
+class ApertureMark(PluginLine):
+    def __init__(self, viewer, id, **kwargs):
+        self._id = id
+        super().__init__(viewer, **kwargs)
+
+
+class HistogramMark(Lines):
+    def __init__(self, min_max_value, scales, **kwargs):
+        # Vertical line in LinearScale
+        y = [0, 1]
+        colors = [accent_color]
+        line_style = "solid"
+        super().__init__(x=min_max_value, y=y, scales=scales, colors=colors, line_style=line_style,
+                         **kwargs)
